var Victor = Victor || {};
Victor.highlightChoices = Victor.highlightChoices|| {};
//=============================================================================
// inVictor - HighlightChoices v2.0.1 20/07/2021
// INV - HighlightChoices.js
//=============================================================================
/*
TERMS of USE

·Free to use in comercial and non-comercial games, as long as you give credits to inVictor.

·Do not sell or claim authorship of this plugin wholy or partially.

·You can edit and adapt the code, as long as you still give credit. 

·Do not redistribute this plugin.

·I reserve the right to change these terms of use.

Version 2.0.2 UPDATE

Included mouse support.


Version 2.0.1 UPDATE

Fixed an error that would crash the game if there is no current Cancel option.

Version 2.0.0 UPDATE
Now the parameters can replace the whole command text. What is its purpose? Well, now you can 
have "troll" NPCs that have changing lists.


Use commandText to print the regular command message.

Use commandCancel to print the message from the cancel option.

Also now you can change the text code "on the fly" by changing the content of 
$gameSystem._commandChoice. For example, a Script call $gameSystem._commandChoice = "Potato" 
before the event command Show Choices will make the selected choice to display Potato.
$gameSystem._commandChoice resets back to default after displaying the Choices.

*/

/*:
 @target MZ
 @plugindesc Highlights the selected choice in a Choice List.
 @author inVictor || Version 2.0.1

 @param escapeCode
 @text Escape Code
 @desc Escape code for the highlight option (\c[x], \i[x], "commandText", "commandCancel", etc)
 @type text
 @default \c[14]commandText


 @help

 This plugin makes Window_ChoiceList change colour of the selected choice. 

 Special thanks to TheoAllen for giving me the idea.
 
 Incompatibilities:
 
 Any plugin that overwrites Window_ChoiceList.prototype.drawItem will 
 collide with this one.

*/

($=> {
	
	const HighlightPluginName = document.currentScript.src.match(/.+\/(.+)\.js/)[1];
	const parameters = PluginManager.parameters(HighlightPluginName);
	const escapeCode = String(parameters['escapeCode']);
	
	let initialiseCommandChoice = Game_System.prototype.initialize;
	Game_System.prototype.initialize = function () {
		initialiseCommandChoice.apply(this, arguments);
		this._commandChoice = escapeCode;
	};
	
	Game_System.prototype.resetCommandChoice = function(){
		this._commandChoice = escapeCode;
	};
	
	$.drawItem = function(index) {
		const rect = this.itemLineRect(index);
		let text = this.commandName(index);
		if (this.index() === index) text = $gameSystem._commandChoice;
		text = text.replace("commandText", this.commandName(index));
		if ($gameMessage.choiceCancelType() >= 0){
			const cancelText = this.commandName($gameMessage.choiceCancelType());
			text = text.replace("commandCancel", cancelText);
		};
		this.drawTextEx(text + "\\c[0]", rect.x, rect.y, rect.width);
		$gameSystem._commandChoice = escapeCode;
	};

	const setCursor = $.setCursorRect;
	$.setCursorRect = function(x, y, width, height){
		setCursor.apply(this,arguments);
		this.refresh();
	};
	
	const resetCommandChoice = $.close;
	$.close = function() {
		resetCommandChoice.apply(this,arguments);
		$gameSystem.resetCommandChoice();
	};

})(Window_ChoiceList.prototype);