var Victor = Victor || {};
Victor.WhereWasI = Victor.WhereWasI|| {};
//=============================================================================
// inVictor - WhereWasI v1.0.0 09/09/2020
// INV - WhereWasI.js
//=============================================================================
/*
TERMS of USE

·Free to use in comercial and non-comercial games, as long as you give credits to inVictor.

·Do not sell or claim authorship of this plugin wholy or partially.

·You can edit and adapt the code, as long as you still give credit. 

·Do not redistribute this plugin.

·I reserve the right to change these terms of use.

*/
var Imported = Imported || {};
Imported.WhereWasI = "WhereWasI";
/*:
* @target MZ
* @plugindesc This plugin allows marking the player with an icon as well as events 
* marked as 'teleport'. These markers are toggleable by pressing a key.
* @author inVictor || Version 1.0.0
* @help
* -inVictor's Where Was I-
*
* This plugin allows marking the player with an icon as well as events marked
* as 'teleport'. These markers are toggleable by pressing a key.
*
* Just type 'teleport' in an event notebox and the icon will appear when 
* marks are triggered by pressing the desired key.
*
* Additionally, this plugin includes one new key to your controls:
* Default trigger key is H (72). You need to write the key code of the key you 
* want to use as trigger. Use this website to check key codes for each key:
* https://keycode.info/
*
* You can also change which gamepad key toggles the markers. Default is 
* "Select" on an XBOX controller.
*
* Special thanks to Dekita, U.K.L, waynee95, Comuns Plugins and the LunaTechs.
*
* @param enable
* @text Enabled by default
* @type boolean
* @on Enable
* @off Disable
* @default false
*
* @param Keys
*  
* @param trigger Key
* @desc This key toggles the markers ON/OFF. 
* Default is 72 (H)
* @type number
* @min 0
* @default 72
* @parent Keys
*
* @param trigger Gamepad Key
* @desc This gamepad key toggles the markers ON/OFF. 
* Default is 8 (Select)
* @type number
* @min 0
* @default 8
* @parent Keys
*
* @param Icons
*
* @param Player icon ID
* @desc The icon ID for the player marker. 0 means no marker.
* @type number
* @min 0
* @default 1
* @parent Icons
*
* @param Event icon ID
* @desc This icon ID for the event marker. 0 means no marker.
* @type number
* @min 0
* @default 1
* @parent Icons
*
* @param Player Icon X
* @desc This icon ID for the event marker. 0 means no marker.
* @type number
* @min 0
* @default 0.5
* @decimals 1
* @parent Icons
*
* @param Player Icon Y
* @desc This icon ID for the event marker. 0 means no marker.
* @type number
* @min 0
* @default 3
* @parent Icons
*
* @param Event Icon X
* @desc This icon ID for the event marker. 0 means no marker.
* @type number
* @min 0
* @default 0.5
* @parent Icons
*
* @param Event Icon Y
* @desc This icon ID for the event marker. 0 means no marker.
* @type number
* @min 0
* @default 0.5
* @parent Icons
*
*/
//--------------------------------------------------------------------------------------
// GAME MAP
//--------------------------------------------------------------------------------------
($=> {
	
	// PLUGIN PARAMETERS
	const parameters = PluginManager.parameters('INV_WhereWasI');
	const enable = JSON.parse(parameters['enable'].toLowerCase());
	const key = parameters['trigger Key'];
	const gamepadKey = parameters['trigger Gamepad Key'];
	
	// Initialise markerVisible variable
	const alias_InitialiseMap = $.initialize;
	$.initialize = function() {
		alias_InitialiseMap.call(this);
		$gameSystem._markerVisible = enable || false;
	};

	// Activation keys
	Input.keyMapper[key] = 'help'; // H key
	Input.gamepadMapper[gamepadKey] = 'help'; // Select button

	const GameMap_keys = $.update;
	$.update = function(sceneActive) {
		GameMap_keys.call(this, arguments);
		if (Input.isTriggered('help')) {
			($gameSystem.markerVisible() === true) ? $gameSystem._markerVisible = false : $gameSystem._markerVisible = true;
		}
	};
})(Game_Map.prototype);
//--------------------------------------------------------------------------------------
// GAME SYSTEM
//--------------------------------------------------------------------------------------

	// markerVisible shortcut
	Game_System.prototype.markerVisible = function(){
		return this._markerVisible;
	};

//--------------------------------------------------------------------------------------
// GAME PLAYER
//--------------------------------------------------------------------------------------
($=> {

	const parameters = PluginManager.parameters('INV_WhereWasI');
	const playerIcon = parameters['Player icon ID'];
	
	// Initialise player marker
	const alias_initMembers = $.initMembers;
	$.initMembers = function() {
		alias_initMembers.call(this);
		this.initPlayerMarker();
	};

	// Player marker icon
	$.initPlayerMarker = function() {
		this._iconId = playerIcon;
	};

	// If not visible, no icon
	$.iconId = function() {
		if ($gameSystem.markerVisible() === false) return;
		if (this._iconId === undefined) this.initPlayerMarker();
		return this._iconId;
	};
})(Game_Player.prototype);
//--------------------------------------------------------------------------------------
// GAME EVENT
//--------------------------------------------------------------------------------------
($=> {
	const parameters = PluginManager.parameters('INV_WhereWasI');
	const eventIcon = parameters['Event icon ID'];

	const alias_initMembersEV = $.initMembers;
	$.initMembers = function() {
		alias_initMembersEV.call(this);
		this.initPlayerMarker();
	};

	$.initPlayerMarker = function() {
		this._iconId = eventIcon;
	};

	$.iconId = function() {
		if ($gameSystem.markerVisible() === false) return;
		if (this._iconId === undefined) this.initPlayerMarker();
		return this._iconId;
	};
})(Game_Event.prototype);
//--------------------------------------------------------------------------------------
// SPRITESET MAP
//--------------------------------------------------------------------------------------
($=> {
	const parameters = PluginManager.parameters('INV_WhereWasI');
	const playerX = parameters['Player Icon X'];
	const playerY = parameters['Player Icon Y'];
	
	// initialise markers
	const alias_initMembersSprite = $.initialize;
	$.initialize = function() {
		alias_initMembersSprite.call(this);
		this.createPlayerMarker();
		this.createEventMarker();
	};

	// Create player marker
	$.createPlayerMarker = function() {
		this._playerMarker = new SpriteMarker($gamePlayer);
		this.addChild(this._playerMarker);
		this._playerMarker.anchor.y = playerY;
		this._playerMarker.anchor.x = playerX;
	};

	// Create event marker
	$.createEventMarker = function() {
		this._eventSprites = [];
		for (const event of $gameMap.events()) {
			if (event.event().name.contains('teleport') || event.event().note.contains('teleport')){
				const newsprite = new SpriteMarker(event);
				this._eventSprites.push(newsprite);
			}
		}
		for (const sprite of this._eventSprites) {
			this.addChild(sprite);
		}
	};
})(Spriteset_Map.prototype);
//--------------------------------------------------------------------------------------
// SPRITEMARKER - New class!
//--------------------------------------------------------------------------------------
class SpriteMarker extends Sprite {
    constructor(target_character) {
        super();
		const parameters = PluginManager.parameters('INV_WhereWasI');
		const eventX = parameters['Event Icon X'];
		const eventY = parameters['Event Icon Y'];
	
        this.bitmap = ImageManager.loadSystem('IconSet');
        this.setFrame(0, 0, 0, 0);
		this.anchor.y = eventY;
		this.anchor.x = eventX;
        //this.anchor.set(0.5);
		this._target = target_character;
    }
    update() {
		super.update()
	    this.x = this._target.screenX();
        this.y = this._target.screenY();
        if ((!$gameSystem.markerVisible()) || this._target._iconId !== this._lastIconID) {
			this.reframe();
        }
	}
    reframe() {
		let id = 0;
        if ($gameSystem.markerVisible()) id = this._target._iconId;
	    const pw = ImageManager.iconWidth;
        const ph = ImageManager.iconHeight;
        const sx = id % 16 * pw;
        const sy = Math.floor(id / 16) * ph;
        this.setFrame(sx, sy, pw, ph);
        this._lastIconID = id;
    }
}