//=============================================================================
// keke_PattoMute - パッとミュート
// バージョン: 1.1.0
//=============================================================================
// Copyright (c) 2021 ケケー
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
//=============================================================================

/*:
 * @target MZ
 * @plugindesc ゲームの音を1ボタンでミュートする
 * @author ケケー
 * @url https://kekeelabo.com
 * 
 *
 *
 * @help
 * 【ver.1.1.1】
 * キーボードの M を押すとゲームの音が消える
 * もう一度押すと元に戻る
 * ツクールMZ/MV両対応
 *
 *【利用規約】
 * MITライセンスのもと、自由に使ってくれて大丈夫です
 * 
 * 
 * 
 * Mute the game sound by pressing M on the keyboard
 * Press again to undo
 * Compatible with both Maker MZ/MV
 *
 *【terms of service】
 * Feel free to use it under the MIT license.
 */



(() => {
    
    
    
    //- 新しいキーを追加
    Input.keyMapper['77'] = 'm'


    //- ミュートの更新(MV)
    if (typeof(ColorManager) == "undefined") {
    const _Scene_Base_update = Scene_Base.prototype.update;
    Scene_Base.prototype.update = function() {
        _Scene_Base_update.call(this);
        if (Input.isTriggered('m')) {
            let masterVol = AudioManager.masterVolume;
            if (masterVol) {
                AudioManager.masterVolume = 0;
            } else {
                AudioManager.masterVolume = 1;
            }
        };
    };
    };
    
    
    //- ミュートの更新(MZ)
    if (typeof(ColorManager) != "undefined") {
    const _Scene_Base_update = Scene_Base.prototype.update;
    Scene_Base.prototype.update = function() {
        _Scene_Base_update.call(this);
        if (Input.isTriggered('m')) {
            let mute = $gameTemp._muteKe;
            if (!mute) {
                $gameTemp._bgmVolPreKe = AudioManager.bgmVolume;
                $gameTemp._bgsVolPreKe = AudioManager.bgsVolume;
                $gameTemp._meVolPreKe = AudioManager.meVolume;
                $gameTemp._seVolPreKe = AudioManager.seVolume;
                AudioManager.bgmVolume = 0;
                AudioManager.bgsVolume = 0;
                AudioManager.meVolume = 0;
                AudioManager.seVolume = 0;
                $gameTemp._muteKe = true
            } else {
                AudioManager.bgmVolume = $gameTemp._bgmVolPreKe;
                AudioManager.bgsVolume = $gameTemp._bgsVolPreKe;
                AudioManager.meVolume = $gameTemp._meVolPreKe;
                AudioManager.seVolume = $gameTemp._seVolPreKe;
                $gameTemp._muteKe = null;
            }
        };
    };
    };

})();