//-----------------------------------------------------------------------------
//  MrSerji_MZ_MasterVolume.js
//-----------------------------------------------------------------------------
/*:
* @author MrSerji
* @url https://mrserji.itch.io/
* @plugindesc This plugin adds the "Master Volume" setting option to the options.
* + A few extras.
*
* @help Master Volume
* MZ Version 1.00
*
*-----------------------------------------------------------------------------
* This plugin offers you the following:
*-----------------------------------------------------------------------------
* - Adds the option for "Master" above the "normal" audio settings.
* - Lets you set the "Start Volume" for all audio sources.
* - Lets you set the "steps" (Offset) that are made when changing sounds.
*
*-----------------------------------------------------------------------------
* Terms of use:
*-----------------------------------------------------------------------------
* - This plugin can be used in both commercial and non-commercial projects.
* - Please add "MrSerji" to the credits if you use it.
* - Redistribution only via the associated page on Itch.io
* - The code may be modified
*
*-----------------------------------------------------------------------------
* Changelog
*-----------------------------------------------------------------------------
* Ver 1.00 Release
*
*-----------------------------------------------------------------------------
* Script Calls
*-----------------------------------------------------------------------------
* None
*
*-----------------------------------------------------------------------------
* Plugin Commands
*-----------------------------------------------------------------------------
* None
*
*-----------------------------------------------------------------------------
* Parameters (Not visible in the help window!)
*-----------------------------------------------------------------------------
* @param defaultMaster
* @text Master Volume
* @desc Sets how loud the Master Volume should be at the start of the game (0-100)
* @type number
* @default 100
* @min 0
* @max 100
*
* @param defaultMasterName
* @text Master Volume Name
* @desc Set the name of this menu option
* @type text
* @default Master Volume
*
* @param defaultBGM
* @text BGM Volume
* @desc Sets how loud the BGM Volume should be at the start of the game (0-100)
* @type number
* @default 40
* @min 0
* @max 100
*
* @param defaultBGS
* @text BGS Volume
* @desc Sets how loud the BGS Volume should be at the start of the game (0-100)
* @type number
* @default 40
* @min 0
* @max 100
*
* @param defaultSE
* @text SE Volume
* @desc Sets how loud the SE Volume should be at the start of the game (0-100)
* @type number
* @default 40
* @min 0
* @max 100
*
* @param defaultME
* @text ME Volume
* @desc Sets how loud the ME Volume should be at the start of the game (0-100)
* @type number
* @default 40
* @min 0
* @max 100
*
* @param volumeOffset
* @text Volumen Offset
* @desc Sets the offset of the volume (0-100)
* (Maker default 20)
* @type number
* @default 10
* @min 0
* @max 100
*
*/

//-----------------------------------------------------------------------------
// IIFE
//-----------------------------------------------------------------------------
(function () {
//-----------------------------------------------------------------------------
// Parse Plugin Parameters by Waynee95
// https://github.com/waynee95/mv-code-snippets/blob/master/Plugin%20Snippets/ParsePluginParameters.md
//-----------------------------------------------------------------------------
var JSONSuperParse = function (string) {
    var temp;
    try {
        temp = JsonEx.parse(typeof string === 'object' ? JsonEx.stringify(string) : string);
    } catch (e) {
        return string;
    }
    if (typeof temp === 'object') {
        Object.keys(temp).forEach(function (key) {
            temp[key] = JSONSuperParse(temp[key]);
            if (temp[key] === '') {
                temp[key] = null;
            }
        });
    }
    return temp;
};

var masterVolumeParam = JSONSuperParse(PluginManager.parameters('MrSerji_MZ_MasterVolume'));
console.log(masterVolumeParam);

//-----------------------------------------------------------------------------
// Plugin Commands
//-----------------------------------------------------------------------------
// None

//-----------------------------------------------------------------------------
// Code Itself
//-----------------------------------------------------------------------------
//Add "Master Volume" to Options
const _MasterVolume_Window_Options_prototype_addGeneralOptions = Window_Options.prototype.addGeneralOptions;
Window_Options.prototype.addGeneralOptions = function() {
    _MasterVolume_Window_Options_prototype_addGeneralOptions.apply(this, arguments);
    this.addCommand(masterVolumeParam.defaultMasterName, "masterVolume");
};
//Adjusts the volume at the first game start
ConfigManager["masterVolume"] = masterVolumeParam.defaultMaster
ConfigManager["bgmVolume"] = masterVolumeParam.defaultBGM
ConfigManager["bgsVolume"] = masterVolumeParam.defaultBGS
ConfigManager["seVolume"] = masterVolumeParam.defaultSE
ConfigManager["meVolume"] = masterVolumeParam.defaultME
//Sets the offset of the volume
Window_Options.prototype.volumeOffset = function() {
    return masterVolumeParam.volumeOffset;
};
//Changes the output volume
const _MasterVolume_Window_Options_prototype_changeVolume = Window_Options.prototype.changeVolume;
Window_Options.prototype.changeVolume = function(symbol, forward, wrap) {
    _MasterVolume_Window_Options_prototype_changeVolume.apply(this, arguments);
    WebAudio.setMasterVolume(ConfigManager["masterVolume"]/100)
};
//Integrates the "Master Volume" into the Save of the ConfigManager
const _MasterVolume_ConfigManager_save = ConfigManager.save;
ConfigManager.save = function() {
    _MasterVolume_ConfigManager_save.apply(this, arguments);
    StorageManager.saveObject("mastervolume", this.makeData());
};
//Saves the "Master Volume" in a new Savefile
ConfigManager.makeData = function() {
    const mastervolume = {};
    mastervolume.masterVolume = this.masterVolume;
    return mastervolume;
};
//Load the "Master Volume" from a Savefile
ConfigManager.applyData = function(mastervolume) {
    this.masterVolume = this.readVolume(mastervolume, "masterVolume");
};
//-----------------------------------------------------------------------------
// IIFE - End
//-----------------------------------------------------------------------------
})();
//-----------------------------------------------------------------------------
// Plugin - End
//-----------------------------------------------------------------------------
