//-----------------------------------------------------------------------------
//  MrSerji_MZ_MouseSettings
//-----------------------------------------------------------------------------
/*:
* @author MrSerji
* @url https://mrserji.itch.io/
* @plugindesc With the help of this plugin it is possible to make minor adjustments to the default system.
*
* @help MouseSettings
* MZ Version 1.00
*
*-----------------------------------------------------------------------------
* This plugin offers you the following:
*-----------------------------------------------------------------------------
* - Set whether you can control the hero on the map with the mouse.
* - Sets whether the menu can be activated with the right mouse button.
*
*-----------------------------------------------------------------------------
* Terms of use:
*-----------------------------------------------------------------------------
* - This plugin can be used in both commercial and non-commercial projects.
* - Please add "MrSerji" to the credits if you use it.
* - Redistribution only via the associated page on Itch.io
* - The code may be modified
*
*-----------------------------------------------------------------------------
* Changelog
*-----------------------------------------------------------------------------
* Ver 1.00 Release
*
*-----------------------------------------------------------------------------
* Script Calls
*-----------------------------------------------------------------------------
* RMT to Activate Menu: Enable
* PluginManager.callCommand(this, "MrSerji_MZ_MouseSettings", "changeMouseRMTMenu", {"RMTMenu":true})
*
* RMT to Activate Menu: Disable
* PluginManager.callCommand(this, "MrSerji_MZ_MouseSettings", "changeMouseRMTMenu", {"RMTMenu":false})
*
* Mouse Click on Map: Enable
* PluginManager.callCommand(this, "MrSerji_MZ_MouseSettings", "changeMouseMap", {"MouseClick":true})
*
* Mouse Click on Map: Disable
* PluginManager.callCommand(this, "MrSerji_MZ_MouseSettings", "changeMouseMap", {"MouseClick":false})
*
*-----------------------------------------------------------------------------
* Plugin Commands
*-----------------------------------------------------------------------------
* Plugin commands are available
*
*-----------------------------------------------------------------------------
* Parameters (Not visible in the help window!)
*-----------------------------------------------------------------------------
* @param mouseRMTMenu
* @text RMT to Activate Menu
* @desc Enable the possibility to access the menu via the RMT
* @type boolean
* @default true
* @on Activate
* @off Deactivate
*
* @command changeMouseRMTMenu
* @text RMT to Activate Menu
* @desc Enable the possibility to access the menu via the RMT
*
* @arg RMTMenu
* @text RMT to Activate Menu
* @desc Set whether you can use the right mouse button to access the menu
* @type boolean
* @default true
* @on Activate
* @off Deactivate
*
* @param mouseMap
* @text Mouse Click on Map
* @desc Enable map movement with the mouse
* @type boolean
* @default true
* @on Activate
* @off Deactivate
*
* @command changeMouseMap
* @text Mouse Click on Map
* @desc Enable map movement with the mouse
*
* @arg MouseClick
* @text Mouse Click on Map
* @desc Sets whether you can use the mouse to move around the map
* @type boolean
* @default true
* @on Activate
* @off Deactivate
*
*/

//-----------------------------------------------------------------------------
// IIFE
//-----------------------------------------------------------------------------
(function () {
//-----------------------------------------------------------------------------
// Parse Plugin Parameters by Waynee95
// https://github.com/waynee95/mv-code-snippets/blob/master/Plugin%20Snippets/ParsePluginParameters.md
//-----------------------------------------------------------------------------
var JSONSuperParse = function (string) {
    var temp;
    try {
        temp = JsonEx.parse(typeof string === 'object' ? JsonEx.stringify(string) : string);
    } catch (e) {
        return string;
    }
    if (typeof temp === 'object') {
        Object.keys(temp).forEach(function (key) {
            temp[key] = JSONSuperParse(temp[key]);
            if (temp[key] === '') {
                temp[key] = null;
            }
        });
    }
    return temp;
};

var parameters = JSONSuperParse(PluginManager.parameters('MrSerji_MZ_MouseSettings'));
console.log(parameters);

//-----------------------------------------------------------------------------
// Plugin Commands
//-----------------------------------------------------------------------------
const pluginName = 'MrSerji_MZ_MouseSettings';

PluginManager.registerCommand(pluginName, "changeMouseRMTMenu", function(args) {
  parameters.mouseRMTMenu = JSON.parse(args.RMTMenu)
});

PluginManager.registerCommand(pluginName, "changeMouseMap", function(args) {
  parameters.mouseMap = JSON.parse(args.MouseClick)
});

//-----------------------------------------------------------------------------
// Code Itself
//-----------------------------------------------------------------------------
//Menu_Call
Scene_Map.prototype.isMenuCalled = function() {
  if (parameters.mouseRMTMenu == true) {
    return Input.isTriggered("menu") || TouchInput.isCancelled();
  } else {
    return Input.isTriggered("menu");
  }
};
//Map_Controll
Scene_Map.prototype.processMapTouch = function() {
  if (parameters.mouseMap == true) {
    if (TouchInput.isTriggered() || this._touchCount > 0) {
        if (TouchInput.isPressed() && !this.isAnyButtonPressed()) {
            if (this._touchCount === 0 || this._touchCount >= 15) {
                this.onMapTouch();
            }
            this._touchCount++;
        } else {
            this._touchCount = 0;
        }
    }
  }
};
//-----------------------------------------------------------------------------
// IIFE - End
//-----------------------------------------------------------------------------
})();
//-----------------------------------------------------------------------------
// Plugin - End
//-----------------------------------------------------------------------------
