//-----------------------------------------------------------------------------
//  MrSerji_MZ_SystemSettings
//-----------------------------------------------------------------------------
/*:
* @author MrSerji
* @url https://mrserji.itch.io/
* @plugindesc With the help of this plugin it is possible to make minor adjustments to the default system.
*
* @help SystemSettings
* MZ Version 1.00
*
*-----------------------------------------------------------------------------
* This plugin offers you the following:
*-----------------------------------------------------------------------------
* - The system colors are freely settable, based on the "Window" file
* - The color for the text outlines can be set with an RGB code
* - It is possible to configure additional functions for the menu and its
* display.
*
*-----------------------------------------------------------------------------
* Terms of use:
*-----------------------------------------------------------------------------
* - This plugin can be used in both commercial and non-commercial projects.
* - Please add "MrSerji" to the credits if you use it.
* - Redistribution only via the associated page on Itch.io
* - The code may be modified
*
*-----------------------------------------------------------------------------
* Changelog
*-----------------------------------------------------------------------------
* Ver 1.00 Release
*
*-----------------------------------------------------------------------------
* Script Calls
*-----------------------------------------------------------------------------
* None
*
*-----------------------------------------------------------------------------
* Plugin Commands
*-----------------------------------------------------------------------------
* None
*
*-----------------------------------------------------------------------------
* Parameters (Not visible in the help window!)
*-----------------------------------------------------------------------------
* @param menuSettings_struct
* @text Menu
* @type struct<menuSettings>
* @desc Modify the menu a little
* @default {"menu":"false","blur":"false","bgOpacity":"255","wBgOpacity":"255"}
*
* @param colorRearragement_struct
* @text Color Arrangement
* @type struct<colorRearragement>
* @desc Change the color arrangement of the systems
* @default {"colors":"true","Main":"0","System":"1","Crisis":"2","Death":"4","powerUp":"4","powerDown":"5","mpCost":"11","tpCost":"13","bgGauge":"16","LowHpGauge":"8","HighHpGauge":"9","LowMpGauge":"10","HighMpGauge":"11","LowTpGauge":"12","HighTpGauge":"13","LowCtGauge":"16","HighCtGauge":"17"}
*
* @param outlineText_struct
* @text Outline Color
* @type struct<outlineText>
* @desc Sets the color for the text outlines
* @default {"outlineRed":"0","outlineGreen":"0","outlineBlue":"0","outlineOpacity":"60"}
*
*/

/*~struct~colorRearragement:
*
* @param colors
* @type boolean
* @text Color Rearrangement
* @desc Sets whether the colors should be rearranged or not.
* @default true
* @on Use
* @off Don't Use
*
* @param Main
* @type number
* @desc Sets the font color for normal texts (0 - 31)
*(Maker default 0)
* @parent colors
* @default 0
* @min 0
* @max 31
*
* @param System
* @type number
* @desc Sets the font color for system texts (0 - 31)
*(Maker default 16)
* @parent colors
* @default 1
* @min 0
* @max 31
*
* @param Crisis
* @type number
* @desc Sets the font color for the status: Crisis (0 - 31)
*(Maker default 17)
* @parent colors
* @default 2
* @min 0
* @max 31
*
* @param Death
* @type number
* @desc Sets the font color for the status: Death (0 - 31)
*(Maker default 18)
* @parent colors
* @default 4
* @min 0
* @max 31
*
* @param powerUp
* @text Power UP
* @type number
* @desc Sets the font color for the indicator: Power Up (0 - 31)
*(Maker default 24)
* @parent colors
* @default 4
* @min 0
* @max 31
*
* @param powerDown
* @text Power Down
* @type number
* @desc Sets the font color for the indicator: Power Down (0 - 31)
*(Maker default 25)
* @parent colors
* @default 5
* @min 0
* @max 31
*
* @param mpCost
* @text MP Cost
* @type number
* @desc Sets the font color for the MP cost (0 - 31)
*(Maker default 23)
* @parent colors
* @default 11
* @min 0
* @max 31
*
* @param tpCost
* @text TP Cost
* @type number
* @desc Sets the font color for the TP cost (0 - 31)
*(Maker default 29)
* @parent colors
* @default 13
* @min 0
* @max 31
*
* @param bgGauge
* @text Gauge Background
* @type number
* @desc Sets the background color for gauges (0 - 31)
*(Maker default 19)
* @parent colors
* @default 16
* @min 0
* @max 31
*
* @param LowHpGauge
* @text HP Gauge (Low)
* @type number
* @desc Sets the color when the HP gauge goes "low" (0 - 31)
*(Maker default 20)
* @parent colors
* @default 8
* @min 0
* @max 31
*
* @param HighHpGauge
* @text HP Gauge (High)
* @type number
* @desc Sets the color when the HP gauge is "High" (0 - 31)
*(Maker default 21)
* @parent colors
* @default 9
* @min 0
* @max 31
*
* @param LowMpGauge
* @text MP Gauge (Low)
* @type number
* @desc Sets the color when the MP gauge goes "low" (0 - 31)
*(Maker default 22)
* @parent colors
* @default 10
* @min 0
* @max 31
*
* @param HighMpGauge
* @text MP Gauge (High)
* @type number
* @desc Sets the color when the MP gauge is "High" (0 - 31)
*(Maker default 23)
* @parent colors
* @default 11
* @min 0
* @max 31
*
* @param LowTpGauge
* @text TP Gauge (Low)
* @type number
* @desc Sets the color when the TP gauge goes "low" (0 - 31)
*(Maker default 28)
* @parent colors
* @default 12
* @min 0
* @max 31
*
* @param HighTpGauge
* @text TP Gauge (High)
* @type number
* @desc Sets the color when the TP gauge is "High" (0 - 31)
*(Maker default 29)
* @parent colors
* @default 13
* @min 0
* @max 31
*
* @param LowCtGauge
* @text CT Gauge (Low)
* @type number
* @desc Sets the color when the Combat Time is low (0 - 31)
*[Only if "Time Progress" is active!] (Maker default 26)
* @parent colors
* @default 16
* @min 0
* @max 31
*
* @param HighCtGauge
* @text CT Gauge (High)
* @type number
* @desc Sets the color when the Combat Time increases (0 - 31)
*[Only if "Time Progress" is active!] (Maker default 27)
* @parent colors
* @default 17
* @min 0
* @max 31
*
*/

/*~struct~outlineText:
*
* @param outlineRed
* @text Red
* @type number
* @desc Adjusts the "red" amount from the outline color (0-255)
* (Maker default 0)
* @default 0
* @min 0
* @max 255
*
* @param outlineGreen
* @text Green
* @type number
* @desc Adjusts the "green" amount from the outline color (0-255)
* (Maker default 0)
* @default 0
* @min 0
* @max 255
*
* @param outlineBlue
* @text Blue
* @type number
* @desc Adjusts the "blue" amount from the outline color (0-255)
* (Maker default 0)
* @default 0
* @min 0
* @max 255
*
* @param outlineOpacity
* @text Opacity
* @type number
* @desc Sets the opacity of the text outlines (0-100)
* (Maker default 60)
* @default 60
* @min 0
* @max 100
*
*/

/*~struct~menuSettings:
*
* @param menu
* @type boolean
* @text Menu Settings
* @desc Sets whether the menus should use other settings or not
* @default false
* @on Use
* @off Don't Use
*
* @param blur
* @type boolean
* @text Remove Blur Effect
* @desc Removes the blur effect from menus
* @parent menu
* @default false
* @on Remove
* @off Keep
*
* @param notActiveOpacity
* @text Inactive Menu Options
* @type number
* @desc Sets the opacity of the menu options that cannot be used
*(0-255) (Maker default 125)
* @parent menu
* @default 125
* @min 0
* @max 255
*
* @param bgOpacity
* @text Background Opacity
* @type number
* @desc Sets the opacity of menu backgrounds (0 - 255)
*(Maker default 192)
* @parent menu
* @default 255
* @min 0
* @max 255
*
* @param wBgOpacity
* @text Window Opacity
* @type number
* @desc Sets the opacity of window backgrounds (0 - 255)
*(Maker default 192)
* @parent menu
* @default 255
* @min 0
* @max 255
*
*/

 //-----------------------------------------------------------------------------
 // IIFE
 //-----------------------------------------------------------------------------
 (function () {
 //-----------------------------------------------------------------------------
 // Parse Plugin Parameters by Waynee95
 // https://github.com/waynee95/mv-code-snippets/blob/master/Plugin%20Snippets/ParsePluginParameters.md
 //-----------------------------------------------------------------------------
 var JSONSuperParse = function (string) {
     var temp;
     try {
         temp = JsonEx.parse(typeof string === 'object' ? JsonEx.stringify(string) : string);
     } catch (e) {
         return string;
     }
     if (typeof temp === 'object') {
         Object.keys(temp).forEach(function (key) {
             temp[key] = JSONSuperParse(temp[key]);
             if (temp[key] === '') {
                 temp[key] = null;
             }
         });
     }
     return temp;
 };

 var parameters = JSONSuperParse(PluginManager.parameters('MrSerji_MZ_SystemSettings'));
 console.log(parameters);

//-----------------------------------------------------------------------------
// Code Itself
//-----------------------------------------------------------------------------
//Checks if the colors should be rearranged
if (parameters.colorRearragement_struct.colors == true) {
  //Change the Systemcolors
  ColorManager.normalColor = function() {
    return this.textColor(parameters.colorRearragement_struct.Main);
  };
  ColorManager.systemColor = function() {
    return this.textColor(parameters.colorRearragement_struct.System);
  };
  ColorManager.crisisColor = function() {
    return this.textColor(parameters.colorRearragement_struct.Crisis);
  };
  ColorManager.deathColor = function() {
    return this.textColor(parameters.colorRearragement_struct.Death);
  };
  ColorManager.powerUpColor = function() {
    return this.textColor(parameters.colorRearragement_struct.powerUp);
  };
  ColorManager.powerDownColor = function() {
    return this.textColor(parameters.colorRearragement_struct.powerDown);
  };
  ColorManager.mpCostColor = function() {
    return this.textColor(parameters.colorRearragement_struct.mpCost);
  };
  ColorManager.tpCostColor = function() {
    return this.textColor(parameters.colorRearragement_struct.tpCost);
  };
  ColorManager.gaugeBackColor = function() {
    return this.textColor(parameters.colorRearragement_struct.bgGauge);
  };
  ColorManager.hpGaugeColor1 = function() {
    return this.textColor(parameters.colorRearragement_struct.LowHpGauge);
  };
  ColorManager.hpGaugeColor2 = function() {
    return this.textColor(parameters.colorRearragement_struct.HighHpGauge);
  };
  ColorManager.mpGaugeColor1 = function() {
    return this.textColor(parameters.colorRearragement_struct.LowMpGauge);
  };
  ColorManager.mpGaugeColor2 = function() {
    return this.textColor(parameters.colorRearragement_struct.HighMpGauge);
  };
  ColorManager.tpGaugeColor1 = function() {
    return this.textColor(parameters.colorRearragement_struct.LowTpGauge);
  };
  ColorManager.tpGaugeColor2 = function() {
    return this.textColor(parameters.colorRearragement_struct.HighTpGauge);
  };
  ColorManager.ctGaugeColor1 = function() {
    return this.textColor(parameters.colorRearragement_struct.LowCtGauge);
  };
  ColorManager.ctGaugeColor2 = function() {
    return this.textColor(parameters.colorRearragement_struct.HighCtGauge);
  };
};
//Changes the outline color
ColorManager.outlineColor = function() {
  return "rgba(" + parameters.outlineText_struct.outlineRed + ","
  + parameters.outlineText_struct.outlineGreen + ","
  + parameters.outlineText_struct.outlineBlue +","
  + (parameters.outlineText_struct.outlineOpacity / 100) +")";
};
//Checks if the menu should be changed
if (parameters.menuSettings_struct.menu == true) {
  //Setting: Blur
  Scene_MenuBase.prototype.createBackground = function() {
    this._backgroundSprite = new Sprite();
    this._backgroundSprite.bitmap = SceneManager.backgroundBitmap();
    this.addChild(this._backgroundSprite);
    this.setBackgroundOpacity(192);
    if (parameters.menuSettings_struct.blur == false) {
      this._backgroundFilter = new PIXI.filters.BlurFilter();
      this._backgroundSprite.filters = [this._backgroundFilter];
    };
  };
  //Setting: Background Opacity
  Scene_MenuBase.prototype.setBackgroundOpacity = function(opacity) {
      this._backgroundSprite.opacity = parameters.menuSettings_struct.bgOpacity;
  };
  //Setting: Window Opacity
  Window_Base.prototype.updateBackOpacity = function() {
      this.backOpacity = parameters.menuSettings_struct.wBgOpacity;
  };
  //Setting: Inactive Menu Option Opacity
  Window_Base.prototype.translucentOpacity = function() {
      return parameters.menuSettings_struct.notActiveOpacity;
  };
};
//-----------------------------------------------------------------------------
// IIFE - End
//-----------------------------------------------------------------------------
})();
//-----------------------------------------------------------------------------
// Plugin - End
//-----------------------------------------------------------------------------
