//=============================================================================
// RPG Maker MZ - Alt Menu Pokemon X and Y 
//=============================================================================

/*:
 * @target MZ
 * @plugindesc An alternate menu designed to look like Pokemon 
 * X and Y from Nintendo
 * @author Francis Kevin (RetroShadow)
 *
 * @help 
 *
 *-------------------------------
 * PLUGIN NOTES                 |
 * ------------------------------
 * Title: RS_AltMenuPokemonXY
 * Plugin Version: 1.00
 * Date of Release: AUG. 28. 2020
 * Last Updated: AUG. 28. 2020
 * Original Code by: Yoji Ojima
 * Code edited by: Francis Kevin (RetroShadow)
 * Development Engine: RPG Maker MZ
 * Can be used in MV: NO
 *
 *
 * -------------------------
 * | PLUGIN INTRODUCTION   |
 * -------------------------
 *
 * This plugin is designed to have your main menu look like you are playing 
 * Pokemon X or Y from the Nintendo 3DS (by Nintendo). In the Pokemon
 * select screen, you would see your Pokemon name, HP, and level. 
 * In this version with RPG Maker MZ, your character's stats
 * (level, HP, MP,TP, and class) are shown in the style of that menu
 *
 * ---------------------------------------------
 * What you need to know before using the plugin|
 *----------------------------------------------
 * 1) This plugin is plug and play. You can use this plugin directly with no 
 * other plugins needed
 *
 * 2) In regards to using this plugin with VisuStella's plugins: This will work 
 * with most of the plugins from VisuStella as this is mostly a main menu style
 * plugin. The exceptions are the Core Engine and Main Menu Manager. 
 * 
 *
 * 3) Customization Options: Customization options are on the way, but they will 
 * take time. Please be patient as this is my first plugin with MZ.
 * 
 *
 * 4) MZ Plugin Only: While I had made an MV version of this plugin in the past, 
 * the way that this is coded is for MZ only. Some code features from MV do not
 * work in MZ. 
 *
 * 5) No plugin commands will be used for this plugin. There are also no
 *script calls, javascript calls, and notetags that you need for this 
 *plugin
 *
 *---------------------------------------------------
 *| PLUGIN TERMS AND CONDITIONS - READ CAREFULLY    |
 *--------------------------------------------------
 *
 * 1) Feel free to use this plugin in free and in commercial projects. 
 *  
 * 2) Feel free to share the plugin with all of your friends who also 
 *use RPG Maker MZ
 *
 * 3) For crediting, you may put "Francis Kevin" (by real name) or 
 *RetroShadow (my stream name)
 *
 * 4) A free copy of your game would be nice and you want, I can give 
 * your game some views for free when I stream it. 
 *
 *
 * 5) Plugin Promises: You promise not to sell the plugin as I make my
 * plugins for free. I will also promise you that I will make my 
 * plugins for free for you all. 
 *
 *
 * 6) If you notice problems or errors in the plugin, let me know by 
 * emailing me: retroyuukigaming@gmail.com. Use the subject "RS Plugin (name)"
 * and let me know what happened so that I can try to fix it. 
 * 
 *---------------------------------------
 * PLUGIN VERSION HISTORY AND UPDATES   |
 * --------------------------------------
 *
 * Version 1.00 (AUG. 28. 2020)
 * - Created and finished the plugin
 * - Basic framework and style finished
 * - Added level and class to the status area
 * - Username: RetroShadow 
 *
 *
 * 
 *
 *
 *
 *

 */





  //
  // process parameters
  //

   const pluginName = 'RS_AltMenuPokemonXY.js';





(() => {
    Scene_MenuBase.prototype.commandWindowHeight = function() {
        return this.calcWindowHeight(2, true);
    };

    Scene_MenuBase.prototype.goldWindowHeight = function() {
        return this.calcWindowHeight(1, true);
    };

    Scene_Menu.prototype.commandWindowRect = function() {
        const ww = Graphics.boxWidth;
        const wh = this.commandWindowHeight();
        const wx = 0;
        const wy = this.mainAreaTop();
        return new Rectangle(wx, 0, 200, 500);
    };

    Scene_Menu.prototype.statusWindowRect = function() {
        const h1 = this.commandWindowHeight();
        const h2 = this.goldWindowHeight();
        const ww = Graphics.boxWidth;
        const wx = 0;
        const wy = this.mainAreaTop() + this.commandWindowHeight();
        return new Rectangle(200, 0, 600, 500);
    };

    Scene_ItemBase.prototype.actorWindowRect = function() {
        const rect = Scene_Menu.prototype.statusWindowRect();
        rect.y = this.mainAreaBottom() - rect.height;
        return rect;
    };

    Window_MenuCommand.prototype.maxCols = function() {
        return 1;
    };

    Window_MenuCommand.prototype.numVisibleRows = function() {
        return MainMenu;
    };

    Window_MenuStatus.prototype.maxCols = function() {
        return 2;
    };

    Window_MenuStatus.prototype.numVisibleRows = function() {
        return 3;
    };

    Window_MenuStatus.prototype.drawItemImage = function(index) {
        const actor = this.actor(index);
        const rect = this.itemRectWithPadding(index);
        const w = Math.min(rect.width, 144);
        const h = Math.min(rect.height, 100);
        const lineHeight = this.lineHeight();
        this.changePaintOpacity(actor.isBattleMember());
        this.drawActorFace(actor, rect.x, rect.y + lineHeight * 1, 100, 100);
        this.changePaintOpacity(true);
    };

    Window_MenuStatus.prototype.drawItemStatus = function(index) {
        const actor = this.actor(index);
        const rect = this.itemRectWithPadding(index);
        const x = rect.x;
        const y = rect.y;
        const width = rect.width;
        const bottom = y + rect.height;
        const lineHeight = this.lineHeight();
        this.drawActorName(actor, x + 140, y + lineHeight * 0, width);
        this.placeBasicGauges(actor, x + 140, bottom - lineHeight * 3 ,width);
        this.drawActorLevel(actor, x, y + 40 + lineHeight * 2, width);
        this.drawActorClass(actor, x + 130, y + 40 + lineHeight * 2, width);
        this.drawActorIcons(actor, x, bottom - lineHeight * 4.25, width);



    };
})();
