//=============================================================================
// TAA_SmartHelpWindows.js
// Author: taaspider
//=============================================================================

var TAA = TAA || {};
TAA.shw = {};
TAA.shw.Version = "1.2.0";
TAA.shw.PluginName = "TAA_SmartHelpWindows";
TAA.shw.alias = {};
TAA.shw.trace = 0;
var Imported = Imported || {};
var Yanfly = Yanfly || {};

/*:
 * @target MV MZ
 *
 * @plugindesc [1.2.0] Change help window behavior
 * @author T. A. A. (taaspider)
 * @url http://taaspider.itch.io/
 * 
 * @help
 * =============================================================================
 * Terms of Use
 * =============================================================================
 * Any plugins developed by taaspider are free for use for both commercial and 
 * noncommercial RPG Maker games, unless specified otherwise. Just remember to
 * credit "Taaspider".
 * 
 * Redistribution of parts or the whole of taaspider plugins is forbidden, unless
 * it comes from the official website: http://taaspider.itch.io. You are allowed 
 * to edit and change the plugin code for your own use, but you're definitely not 
 * allowed to sell or reuse any part of the code as your own. Although not 
 * required to use my plugins, a free copy of your game would be nice!
 * 
 * If you enjoy my work, consider offering a donation when downloading my plugins, 
 * or offering a monthly pledge to my Patreon account. It would be of great help!
 * Also, follow me on facebook to get firsthand news on my activities!
 *  Facebook: https://www.facebook.com/taaspider 
 *  Patreon: https://www.patreon.com/taaspider
 *  
 * =============================================================================
 * Introduction
 * =============================================================================
 * 
 * WARNING: This plugin requires RPG Maker MV 1.5.0 or above! Please make sure 
 * your RPG Maker MV software is up to date before using this plugin.
 * You don't need any specific version if you're using MZ.
 * 
 * WARNING 2: This plugin messes A LOT with any scene it is enabled on. To ensure
 * it works properly, place it as far below in your plugin list as possible.
 * 
 * -----------------------------------------------------------------------------
 * 
 * By default, help windows are included by the engine in all menu scenes as a
 * way to convey additional information to the player, either by showing an item,
 * weapon, armor or skill description, or by showing a descriptive text for the
 * scene (like showing "Which slot to save?" in the save scene, for example).
 * This window takes a considerable area of the screen, even when there's no text
 * to show.
 * 
 * This plugin aims to effectively remove the help window from all selected scenes,
 * while also allowing for its existence (in whichever window from the scene you
 * want it to be enabled). However, when the window is activated using this plugin
 * it is show close to the selected item, and the window is resized to fit the text
 * it has to show without taking more space than needed to convey the required 
 * information. Also, the player has the power to show or hide this window as he
 * pleases, so that experienced players can remove unnecessary information from 
 * the screen.
 * 
 * =============================================================================
 * Special Thanks
 * =============================================================================
 * 
 * I would like to give a special thanks to my friend DarkSearinox92 for bringing me
 * the idea for this plugin, and relentlessly helping me test the many iterations
 * need for a stable release (and also for helping me test many of my other plugins).
 * 
 * Thank you!
 * 
 * =============================================================================
 * How it Works
 * =============================================================================
 * 
 * There's a long list of parameters used to configure the plugin operations. 
 * However, most of them are used to tell the plugin which scenes should keep
 * the original help window, and which should adopt the plugin's dynamic window.
 * For each scene, you can also select which windows will allow the smart help
 * window to show. When it is disabled for a window and that window becomes active,
 * the help window is hidden from the player as long as he remains in the current
 * disabled window.
 * 
 * The section below describes the plugin's parameters and additional advanced 
 * settings used to reposition the remaining windows.
 * 
 * -----------------------------------------------------------------------------
 * Battle Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Battle
 * ------------
 *  This defines if the smart help windows are enabled or not during battle. If
 *  this is set to false all the remaining parameters under the "Battle Scene"
 *  are made irrelevant, as they will all be ignored.
 * 
 * Window_BattleSkill
 * ------------------
 *  This defines if the smart help window is allowed in the skill selection window
 *  during battle. If Scene_Battle is enabled and this parameter is disabled, the
 *  help window will be hidden from the player, and pressing the key to turn it
 *  visible won't change its state.
 * 
 * Window_BattleItem
 * -----------------
 *  This defines if the smart help windows are enabled or not at the Item selection 
 *  window. If Scene_Battle is enabled and this parameter is disabled, the
 *  help window will be hidden from the player, and pressing the key to turn it
 *  visible won't change its state.
 * 
 * Battle Custom Windows
 * -------------------
 *  This can be used to change all remaining windows appearances to account for 
 *  the absence of the help window. Almost all settings are eval expressions,
 *  which are all evaluated at scene level. This settings are automatically 
 *  disabled if Scene_Battle is disabled. Otherwise, use the parameter "Custom
 *  Window Positioning" to enable or disable all customizations.
 *  
 *  I've tested the plugin with the engine defaults and also with YEP Engine
 *  menu changes. You can find below my recommended settings for both cases.
 * 
 *  Default Scene Recommendations (MV):
 *      Skill X = 0
 *      Skill Y = 0
 *      Skill Width = Graphics.boxWidth
 *      Skill Height = this._statusWindow.y
 *      Skill Cols = 2
 *      Item X = 0
 *      Item Y = 0
 *      Item Width = Graphics.boxWidth
 *      Item Height = this._statusWindow.y
 *      Item Cols = 2
 * 
 *  If YEP_BattleEngineCore menu AND the Lower Windows parameter is enabled (if the parameter is
 *  disabled, stick to the default settings):
 *      Skill X = 0
 *      Skill Y = Graphics.boxHeight - this._statusWindow.fittingHeight(Yanfly.Param.BECWindowRows)
 *      Skill Width = Graphics.boxWidth
 *      Skill Height = this._statusWindow.fittingHeight(Yanfly.Param.BECWindowRows)
 *      Skill Cols = 2
 *      Item X = 0
 *      Item Y = Graphics.boxHeight - this._statusWindow.fittingHeight(Yanfly.Param.BECWindowRows)
 *      Item Width = Graphics.boxWidth
 *      Item Height = this._statusWindow.fittingHeight(Yanfly.Param.BECWindowRows)
 *      Item Cols = 2
 * 
 *  Default Scene Recommendations (MZ):
 *      Skill X = 0
 *      Skill Y = Graphics.boxHeight - this.calcWindowHeight(4, true)
 *      Skill Width = Graphics.boxWidth
 *      Skill Height = this.calcWindowHeight(4, true)
 *      Skill Cols = 2
 *      Item X = 0
 *      Item Y = Graphics.boxHeight - this.calcWindowHeight(4, true)
 *      Item Width = Graphics.boxWidth
 *      Item Height = this.calcWindowHeight(4, true)
 *      Item Cols = 2
 * 
 * In all scenarios, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Skill (this._skillWindow)
 *      2. Item (this._itemWindow)
 * 
 * -----------------------------------------------------------------------------
 * Item Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Item
 * ----------
 *  This defines if the smart help windows are enabled or not during the item
 *  menu scene. If this is set to false all the remaining parameters under the
 *  "Item Scene" group are made irrelevant, as they will all be ignored.
 * 
 * Item Window_ItemCategory
 * ------------------------
 *  This defines if the smart help window is allowed in the item category selection
 *  window while on the item menu scene. If Scene_Item is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 *  By default, the engine does not provide any place to add a description for
 *  item category names. So enabling this feature would be pointless. However, 
 *  you can use the global parameters "Item Category Help" and "Item Category
 *  Texts" to set their description if you want.
 * 
 * Item Window_ItemList
 * --------------------
 *  This defines if the smart help window is allowed in the item selection list
 *  window while on the item menu scene. If Scene_Item is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Item Custom Windows
 * -------------------
 *  This can be used to change all remaining windows appearances to account for 
 *  the absence of the help window. Almost all settings are eval expressions,
 *  which are all evaluated at scene level. This settings are automatically 
 *  disabled if Scene_Item is disabled. Otherwise, use the parameter "Custom
 *  Window Positioning" to enable or disable all customizations.
 *  
 *  I've tested the plugin with the engine defaults and also with YEP Engine
 *  menu changes. You can find below my recommended settings for both cases.
 *  Parameters not listed for the default scene settings can be ignored, as
 *  they refer exclusively to YEP_ItemCore menu version.
 * 
 *  Default Scene Recommendations (MV):
 *      Item Category X = 0
 *      Item Category Y = 0
 *      Item Category Width = Graphics.boxWidth
 *      Item Category Rows = 1
 *      Item List X = 0
 *      Item List Y = this._categoryWindow.y + this._categoryWindow.height
 *      Item List Width = Graphics.boxWidth
 *      Item List Height = Graphics.boxHeight - this._categoryWindow.height
 *      Item List Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Category (this._categoryWindow)
 *      2. Item List (this._itemWindow)
 * 
 *  If YEP_ItemCore menu is enabled:
 *      Item Category X = 0
 *      Item Category Y = 0
 *      Item Category Width = 240
 *      Item Category Rows = 1
 *      Item List X = 0
 *      Item List Y = this._categoryWindow.y + this._categoryWindow.height
 *      Item List Width = Graphics.boxWidth / 2
 *      Item List Height = Graphics.boxHeight - this._categoryWindow.height
 *      Item List Cols = 2
 *      Status X = this_categoryWindow.width
 *      Status Y = 0
 *      Status Width = Graphics.boxWidth - this._categoryWindow.width
 *      Status Height = this._categoryWindow.height
 *      Info X = this._itemWindow.width
 *      Info Y = this._itemWindow.y
 *      Info Width = Graphics.boxWidth - this._itemWindow.width
 *      Info Height = this._itemWindow.height
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Category (this._categoryWindow)
 *      2. Item List (this._itemWindow)
 *      3. Status (this._statusWindow)
 *      4. Info (this._infoWindow)
 * 
 *  Default Scene Recommendations (MZ):
 *      Item Category X = 0
 *      Item Category Y = this._cancelButton.y + this._cancelButton.height
 *      Item Category Width = Graphics.boxWidth
 *      Item Category Rows = 1
 *      Item List X = 0
 *      Item List Y = this._categoryWindow.y + this._categoryWindow.height
 *      Item List Width = Graphics.boxWidth
 *      Item List Height = Graphics.boxHeight - this._itemWindow.y
 *      Item List Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Category (this._categoryWindow)
 *      2. Item List (this._itemWindow)
 * 
 * -----------------------------------------------------------------------------
 * Skill Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Skill
 * ----------
 *  This defines if the smart help windows are enabled or not during the skill
 *  menu scene. If this is set to false all the remaining parameters under the
 *  "Skill Scene" group are made irrelevant, as they will all be ignored.
 * 
 * Skill Window_SkillType
 * ----------------------
 *  This defines if the smart help window is allowed in the skill type selection
 *  window while on the skill menu scene. If Scene_Skill is enabled and this parameter is
 *  disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 *  By default, the engine does not provide any place to add a description for
 *  skill types. So enabling this feature would be pointless. However, you can use 
 *  the global parameters "Skill Type Help" and "Skill Type Texts" to set their 
 *  description if you want.
 * 
 * Skill Window_SkillList
 * ----------------------
 *  This defines if the smart help window is allowed in the skill selection list
 *  window while on the skill menu scene. If Scene_Skill is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Skill Custom Windows
 * -------------------
 *  This can be used to change all remaining windows appearances to account for 
 *  the absence of the help window. Almost all settings are eval expressions,
 *  which are all evaluated at scene level. This settings are automatically 
 *  disabled if Scene_Skill is disabled. Otherwise, use the parameter "Custom
 *  Window Positioning" to enable or disable all customizations.
 *  
 *  I've tested the plugin with the engine defaults and also with YEP Engine
 *  menu changes. You can find below my recommended settings for both cases.
 * 
 *  Default Scene Recommendations (MV):
 *      Skill Type X = 0
 *      Skill Type Y = 0
 *      Skill Type Width = 240
 *      Skill Type Rows = 4
 *      Skill Status X = this._skillTypeWindow.width
 *      Skill Status Y = 0
 *      Skill Status Width = Graphics.boxWidth - this._skillTypeWindow.width
 *      Skill Status Height = this._skillTypeWindow.height
 *      Skill List X = 0
 *      Skill List Y = this._statusWindow.y + this._statusWindow.height
 *      Skill List Width = Graphics.boxWidth
 *      Skill List Height = Graphics.boxHeight - this._statusWindow.y - this._statusWindow.Height
 *      Skill List Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Type (this._skillTypeWindow)
 *      2. Status (this._statusWindow)
 *      3. Skill List (this._itemWindow)
 * 
 *  Default Scene Recommendations (MZ):
 *      Skill Type X = 0
 *      Skill Type Y = this._cancelButton.y + this._cancelButton.height
 *      Skill Type Width = 240
 *      Skill Type Rows = 4
 *      Skill Status X = this._skillTypeWindow.width
 *      Skill Status Y = this._skillTypeWindow.y
 *      Skill Status Width = Graphics.boxWidth - this._skillTypeWindow.width
 *      Skill Status Height = this._skillTypeWindow.height
 *      Skill List X = 0
 *      Skill List Y = this._statusWindow.y + this._statusWindow.height
 *      Skill List Width = Graphics.boxWidth
 *      Skill List Height = Graphics.boxHeight - this._statusWindow.y - this._statusWindow.Height
 *      Skill List Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Type (this._skillTypeWindow)
 *      2. Status (this._statusWindow)
 *      3. Skill List (this._itemWindow)
 * 
 * -----------------------------------------------------------------------------
 * Equip Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Equip
 * ----------
 *  This defines if the smart help windows are enabled or not during the equip
 *  menu scene. If this is set to false all the remaining parameters under the
 *  "Equip Scene" group are made irrelevant, as they will all be ignored.
 * 
 * Equip Window_EquipCommand
 * -------------------------
 *  This defines if the smart help window is allowed in the equip command selection
 *  window while on the equip menu scene. If Scene_Equip is enabled and this parameter is
 *  disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 *  By default, the engine does not provide any place to add a description for
 *  equip commands. So enabling this feature would be pointless. However, you can use 
 *  the global parameters "Equip Command Help" and "Equip Command Texts" to set their 
 *  description if you want.
 * 
 * Equip Window_EquipItem
 * ----------------------
 *  This defines if the smart help window is allowed in the equip selection list
 *  window while on the equip menu scene. If Scene_Equip is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Equip Custom Windows
 * -------------------
 *  This can be used to change all remaining windows appearances to account for 
 *  the absence of the help window. Almost all settings are eval expressions,
 *  which are all evaluated at scene level. This settings are automatically 
 *  disabled if Scene_Equip is disabled. Otherwise, use the parameter "Custom
 *  Window Positioning" to enable or disable all customizations.
 *  
 *  I've tested the plugin with the engine defaults and also with YEP Engine
 *  menu changes. You can find below my recommended settings for both cases.
 *  Parameters not listed for the default scene settings can be ignored, as
 *  they refer exclusively to YEP_EquipCore menu version.
 * 
 *  Default Scene Recommendations (MV):
 *      Command X = this._statusWindow.width
 *      Command Y = 0
 *      Command Width = Graphics.boxWidth - this._statusWindow.width
 *      Command Cols = 3
 *      Command Rows = 1
 *      Status X = 0
 *      Status Y = 0
 *      Status Width = 312
 *      Status Height = this._statusWindow.fittingHeight(7)
 *      Slot X = this._statusWindow.width
 *      Slot Y = this._commandWindow.y + this._commandWindow.height
 *      Slot Width = Graphics.boxWidth - this._statusWindow.width
 *      Slot Height = this._statusWindow.height - this._commandWindow.height
 *      Item X = 0
 *      Item Y = this._statusWindow.y + this._statusWindow.height
 *      Item Width = Graphics.boxWidth
 *      Item Height = Graphics.boxHeight - this._statusWindow.y - this._statusWindow.height
 *      Item Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Status (this._statusWindow)
 *      2. Command (this._commandWindow)
 *      3. Slot (this._slotWindow)
 *      4. Items (this._itemWindow)
 * 
 *  If YEP_EquipCore menu is enabled:
 *      Command X = 0
 *      Command Y = 0
 *      Command Width = 240
 *      Command Cols = 1
 *      Command Rows = 4
 *      Status X = this._commandWindow.width
 *      Status Y = 0
 *      Status Width = Graphics.boxWidth - this._commandWindow.width
 *      Status Height = this._statusWindow.fittingHeight(4)
 *      Slot X = 0
 *      Slot Y = this._commandWindow.y + this._commandWindow.height
 *      Slot Width = Graphics.boxWidth / 2
 *      Slot Height = Graphics.boxHeight - this._commandWindow.y - this._commandWindow.height
 *      Item X = 0
 *      Item Y = this._slotWindow.y
 *      Item Width = Graphics.boxWidth / 2
 *      Item Height = Graphics.boxHeight - this._slotWindow.y
 *      Item Cols = 2
 *      Status Compare X = this._itemWindow.width
 *      Status Compare Y = this._itemWindow.y
 *      Status Compare Width = Graphics.boxWidth - this._itemWindow.width
 *      Status Compare Height = Graphics.boxHeight - this._itemWindow.y
 *      Info X = this._itemWindow.width
 *      Info Y = this._itemWindow.y
 *      Info Width = Graphics.boxWidth - this._itemWindow.width
 *      Info Height = this._itemWindow.height
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Status (this._statusWindow)
 *      2. Command (this._commandWindow)
 *      3. Slot (this._slotWindow)
 *      4. Items (this._itemWindow)
 *      5. Status Compare (this._compareWindow)
 *      6. Info (this._infoWindow)
 * 
 *  Default Scene Recommendations (MZ):
 *      Command X = this._statusWindow.width
 *      Command Y = this._statusWindow.y
 *      Command Width = Graphics.boxWidth - this._statusWindow.width
 *      Command Cols = 3
 *      Command Rows = 1
 *      Status X = 0
 *      Status Y = this._cancelButton.y + this._cancelButton.height
 *      Status Width = 312
 *      Status Height = Graphics.boxHeight - this.buttonAreaHeight()
 *      Slot X = this._statusWindow.width
 *      Slot Y = this._commandWindow.y + this._commandWindow.height
 *      Slot Width = Graphics.boxWidth - this._statusWindow.width
 *      Slot Height = this._statusWindow.height - this._commandWindow.height
 *      Item X = this._statusWindow.width
 *      Item Y = this._commandWindow.y + this._commandWindow.height
 *      Item Width = Graphics.boxWidth - this._statusWindow.width
 *      Item Height = this._statusWindow.height - this._commandWindow.height
 *      Item Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Status (this._statusWindow)
 *      2. Command (this._commandWindow)
 *      3. Slot (this._slotWindow)
 *      4. Items (this._itemWindow)
 * 
 * -----------------------------------------------------------------------------
 * Shop Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Shop
 * ----------
 *  This defines if the smart help windows are enabled or not during the shop
 *  menu scene. If this is set to false all the remaining parameters under the
 *  "Shop Scene" group are made irrelevant, as they will all be ignored.
 * 
 * Window_ShopBuy
 * --------------
 *  This defines if the smart help window is allowed in the item to buy selection list
 *  window while on the shop menu scene. If Scene_Shop is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Window_ShopSell
 * ---------------
 *  This defines if the smart help window is allowed in the item to sell selection list
 *  window while on the shop menu scene. If Scene_Shop is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Shop Window_ItemCategory
 * ------------------------
 *  This defines if the smart help window is allowed in the item category selection
 *  window while on the shop menu scene. If Scene_Shop is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 *  By default, the engine does not provide any place to add a description for
 *  item category names. So enabling this feature would be pointless. However, 
 *  you can use the global parameters "Item Category Help" and "Item Category
 *  Texts" to set their description if you want.
 * 
 * Shop Custom Windows
 * -------------------
 *  This can be used to change all remaining windows appearances to account for 
 *  the absence of the help window. Almost all settings are eval expressions,
 *  which are all evaluated at scene level. This settings are automatically 
 *  disabled if Scene_Shop is disabled. Otherwise, use the parameter "Custom
 *  Window Positioning" to enable or disable all customizations.
 *  
 *  I've tested the plugin with the engine defaults and also with YEP Engine
 *  menu changes. You can find below my recommended settings for both cases.
 *  Parameters not listed for the default scene settings can be ignored, as
 *  they refer exclusively to YEP_ShopMenuCore menu version.
 * 
 *  Default Scene Recommendations (MV):
 *      Gold X = Graphics.boxWidth - 140
 *      Gold Y = 0
 *      Gold Width = 240
 *      Command X = 0
 *      Command Y = 0
 *      Command Width = this._goldWindow.x
 *      Command Cols = 3
 *      Command Rows = 1
 *      Dummy X = 0
 *      Dummy Y = this._commandWindow.y + this._commandWindow.height
 *      Dummy Width = Graphics.boxWidth
 *      Dummy Height = Graphics.boxHeight - this._commandWindow.y - this._commandWindow.height
 *      Number X = 0
 *      Number Y = this._dummyWindow.y
 *      Number Width = 456
 *      Number Height = Graphics.boxHeight - this._dummyWindow.y
 *      Status X = this._numberWindow.width
 *      Status Y = this._dummyWindow.y
 *      Status Width = Graphics.boxWidth - this._numberWindow.width
 *      Status Height = this._numberWindow.height
 *      Buy X = 0
 *      Buy Y = this._dummyWindow.y
 *      Buy Width = 456
 *      Buy Height = this._numberWindow.height
 *      Buy Cols = 1
 *      Category X = 0
 *      Category Y = this._commandWindow.y + this._commandWindow.height
 *      Category Width = Graphics.boxWidth
 *      Category Cols = 4
 *      Category Rows = 1
 *      Sell X = 0
 *      Sell Y = this._categoryWindow.y + this._categoryWindow.height
 *      Sell Width = Graphics.boxWidth
 *      Sell Height = Graphics.boxHeight - this._categoryWindow.y - this._categoryWindow.height
 *      Sell Cols = 1
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Gold (this._goldWindow)
 *      2. Command (this._commandWindow)
 *      3. Item Category (this._categoryWindow)
 *      4. Dummy (this._dummyWindow)
 *      5. Number (this._numberWindow)
 *      6. Status (this._statusWindow)
 *      7. Buy (this._buyWindow)
 *      8. Sell (this._sellWindow)
 * 
 *  If YEP_ShopMenuCore menu is enabled:
 *      Gold X = this._buyWindow.width
 *      Gold Y = Graphics.boxHeight - this._goldWindow.height
 *      Gold Width = Graphics.boxWidth - this._buyWindow.width
 *      Command X = 0
 *      Command Y = 0
 *      Command Width = 240
 *      Command Cols = 1
 *      Command Rows = 4
 *      Dummy X = 0
 *      Dummy Y = this._commandWindow.y + this._commandWindow.height
 *      Dummy Width = Math.ceil(eval(Yanfly.Param.ShopListWidth))
 *      Dummy Height = Graphics.boxHeight - this._commandWindow.y - this._commandWindow.height
 *      Number X = 0
 *      Number Y = this._dummyWindow.y
 *      Number Width = this._dummyWindow.width
 *      Number Height = Graphics.boxHeight - this._dummyWindow.y
 *      Status X = this._numberWindow.width
 *      Status Y = this._dummyWindow.y
 *      Status Width = Graphics.boxWidth - this._dummyWindow.width
 *      Status Height = this._dummyWindow.height - this._goldWindow.height
 *      Buy X = 0
 *      Buy Y = this._dummyWindow.y
 *      Buy Width = Math.ceil(eval(Yanfly.Param.ShopListWidth))
 *      Buy Height = Graphics.boxHeight - this._dummyWindow.y
 *      Buy Cols = 2
 *      Category X = 0
 *      Category Y = this._commandWindow.y
 *      Category Width = 240
 *      Category Cols = 1
 *      Category Rows = 4
 *      Sell X = 0
 *      Sell Y = this._dummyWindow.y
 *      Sell Width = this._dummyWindow.width
 *      Sell Height = this._dummyWindow.height
 *      Sell Cols = 2
 *      Info X = this._commandWindow.width
 *      Info Y = this._commandWindow.y
 *      Info Width = Graphics.boxWidth - this._commandWindow.width
 *      Info Height = this._commandWindow.height
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Command (this._commandWindow)
 *      2. Info (this._infoWindow)
 *      3. Dummy (this._dummyWindow)
 *      4. Number (this._numberWindow)
 *      5. Buy (this._buyWindow)
 *      6. Shop Category (this._categoryWindow)
 *      7. Sell (this._sellWindow)
 *      8. Gold (this._goldWindow)
 *      9. Status (this._statusWindow)
 * 
 *  Default Scene Recommendations (MZ):
 *      Gold X = Graphics.boxWidth - 240
 *      Gold Y = this._cancelButton.y + this._cancelButton.height
 *      Gold Width = 240
 *      Command X = 0
 *      Command Y = this._goldWindow.y
 *      Command Width = this._goldWindow.x
 *      Command Cols = 3
 *      Command Rows = 1
 *      Dummy X = 0
 *      Dummy Y = this._commandWindow.y + this._commandWindow.height
 *      Dummy Width = Graphics.boxWidth
 *      Dummy Height = Graphics.boxHeight - this._commandWindow.y - this._commandWindow.height
 *      Number X = 0
 *      Number Y = this._dummyWindow.y
 *      Number Width = Graphics.boxWidth - this.statusWidth()
 *      Number Height = Graphics.boxHeight - this._dummyWindow.y
 *      Status X = Graphics.boxWidth - this.statusWidth()
 *      Status Y = this._dummyWindow.y
 *      Status Width = this.statusWidth()
 *      Status Height = this._dummyWindow.height
 *      Buy X = 0
 *      Buy Y = this._dummyWindow.y
 *      Buy Width = Graphics.boxWidth - this.statusWidth()
 *      Buy Height = this._dummyWindow.height
 *      Buy Cols = 2
 *      Category X = 0
 *      Category Y = this._dummyWindow.y
 *      Category Width = Graphics.boxWidth
 *      Category Cols = 4
 *      Category Rows = 1
 *      Sell X = 0
 *      Sell Y = this._categoryWindow.y + this._categoryWindow.height
 *      Sell Width = Graphics.boxWidth
 *      Sell Height = Graphics.boxHeight - this._categoryWindow.y - this._categoryWindow.height
 *      Sell Cols = 2
 * 
 * In this scenario, windows are set in the following order (so avoid trying to use 
 * as reference settings that have not been defined yet):
 *      1. Gold (this._goldWindow)
 *      2. Command (this._commandWindow)
 *      3. Item Category (this._categoryWindow)
 *      4. Dummy (this._dummyWindow)
 *      5. Number (this._numberWindow)
 *      6. Status (this._statusWindow)
 *      7. Buy (this._buyWindow)
 *      8. Sell (this._sellWindow)
 * 
 * -----------------------------------------------------------------------------
 * Title Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Title
 * ----------
 *  This defines if the smart help windows are enabled or not during the title
 *  scene. If this is set to false all the remaining parameters under the "Title 
 *  Scene" group are made irrelevant, as they will all be ignored.
 *  This feature is compatible with TAA_MapTitleScene and YEP_CreditsPage. Other
 *  compatibilities are not guaranteed.
 * 
 * Window_TitleCommand
 * --------------
 *  This defines if the smart help window is allowed in the title command list
 *  window while on the title scene. If Scene_Title is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Title Command Texts
 * -------------------
 *  This array defines what text description to show in the help window when a
 *  title command is highlighted. It basically an array of key value pairs: for
 *  each title command name (as shown in the command window), it gives a description 
 *  text to be shown.
 * 
 * Title Custom Windows
 * -------------------
 *  Since the help window is not present by default in the title scene, there are
 *  no parameters to customize the Scene_Title window placement.
 * 
 * -----------------------------------------------------------------------------
 * Menu Scene
 * -----------------------------------------------------------------------------
 * 
 * Scene_Menu
 * ----------
 *  This defines if the smart help windows are enabled or not during the menu
 *  scene. If this is set to false all the remaining parameters under the "Menu 
 *  Scene" group are made irrelevant, as they will all be ignored.
 * 
 * Window_MenuCommand
 * --------------
 *  This defines if the smart help window is allowed in the menu command list
 *  window while on the menu scene. If Scene_Menu is enabled and this parameter 
 *  is disabled, the help window will be hidden from the player, and pressing the key
 *  to turn it visible won't change its state.
 * 
 * Menu Command Texts
 * -------------------
 *  This array defines what text description to show in the help window when a
 *  menu command is highlighted. It basically an array of key value pairs: for
 *  each menu command name (as shown in the command window), it gives a description 
 *  text to be shown.
 * 
 * Menu Custom Windows
 * -------------------
 *  Since the help window is not present by default in the menu scene, there are
 *  no parameters to customize the Scene_Menu window placement.
 * 
 * -----------------------------------------------------------------------------
 * Input Info Settings
 * -----------------------------------------------------------------------------
 * 
 * Each scene has its own input info settings, so you can determine how this window
 * works for each scene independently. However, parameters are always the same for all
 * the scenes. You can enable / disable it, change the window position and size, change
 * font settings, padding, opacity and window skin.
 * 
 * -----------------------------------------------------------------------------
 * Input
 * -----------------------------------------------------------------------------
 * 
 * Input Key
 * ---------
 *  This parameter determines which input key is used to show or hide the smart help
 *  windows. It provides a list of supported keys for the developer to select.
 * 
 *  WARNING: YEP_EquipCore has a hardcoded effect programmed for the 'tab' key. If
 *  you're also using this plugin and select the same key for Smart Help Windows
 *  input, it won't work as expected. If that's your case, select a different key to
 *  ensure compatibility. That's also valid for any other plugin that alters input 
 *  behavior for any given scene.
 * 
 * Default Setting
 * ---------------
 *  This tells the game if the Smart Help Windows are shown or hidden by default
 *  at game boot.
 *  
 * -----------------------------------------------------------------------------
 * Choices
 * -----------------------------------------------------------------------------
 * 
 * Starting on version 1.2.0, the plugin can also load help texts to be used with
 * the engine Show Choices command. This section includes a parameter to enable
 * or disable this feature by default (it can be altered with plugin commands), 
 * and a specific set of input info window settings to be used when the choices
 * window is displayed in a map.
 * 
 * -----------------------------------------------------------------------------
 * Global
 * -----------------------------------------------------------------------------
 * 
 * Display When Empty
 * ------------------
 *  If disabled, Smart Help Windows will be automatically hidden when the 
 *  highlighted item has no attached description. This can be strange for a 
 *  player (the window appearing for some items, and some not). Use it with
 *  caution.
 * 
 * Help Window Settings
 * --------------------
 *  This allows you to configure the help window as a separate entity from all
 *  the other windows (effective only on scenes where the plugin is enabled).
 *  It allows you to set properties like line height, font settings, padding, 
 *  window opacity and skin.
 * 
 *  This settings also allow you to draw a flat color rectangle behind the smart
 *  help window. You can use that to replace the window skin itself (to do that,
 *  set both standard and back opacity to 0), or to replace the window skin 
 *  background only (by setting only back opacity to 0). The color settings for
 *  the background rectangle is based on the html color code, and alpha is used
 *  to set opacity, being 0 completely transparent, and 100 completely opaque.
 * 
 * Help Preferred Position
 * -----------------------
 *  This allows you to choose what's the preferred method of placement for the
 *  help window. If set to 'Above', the plugin will try to always place the window
 *  above the selected item. If the help window falls out of screen the plugin will
 *  move it below the selection. If set to 'Below' the inverse occurs.
 * 
 * Item Category Help
 * ------------------
 *  If enabled, whenever a Window_ItemCategory (with its corresponding scene 
 *  parameter enabled) is active the plugin will load category descriptions from 
 *  Item Category Texts to be shown in the help windows (be it smart or not).
 * 
 * Item Category Texts
 * -------------------
 *  This array defines what text description to show in the help window when an
 *  item category is highlighted. It basically an array of key value pairs: for
 *  each item category name (as shown in the menu), it gives a description text
 *  to be shown.
 * 
 * Skill Type Help
 * ------------------
 *  If enabled, whenever a Window_SkillType (with its corresponding scene 
 *  parameter enabled) is active the plugin will load type descriptions from 
 *  Skill Type Texts to be shown in the help windows (be it smart or not).
 * 
 * Skill Type Texts
 * -------------------
 *  This array defines what text description to show in the help window when an
 *  skill type is highlighted. It basically an array of key value pairs: for
 *  each skill type name (as shown in the menu), it gives a description text
 *  to be shown.
 * 
 * Equip Command Help
 * ------------------
 *  If enabled, whenever a Window_EquipCommand (with its corresponding scene 
 *  parameter enabled) is active the plugin will load command descriptions from 
 *  Equip Command Texts to be shown in the help windows (be it smart or not).
 * 
 * Equip Command Texts
 * -------------------
 *  This array defines what text description to show in the help window when an
 *  equip command is highlighted. It basically an array of key value pairs: for
 *  each equip command name (as shown in the menu), it gives a description text
 *  to be shown.
 *  
 * -----------------------------------------------------------------------------
 * Toggle Button
 * -----------------------------------------------------------------------------
 * 
 * Version 1.2.0 introduced a feature to load a custom button for the player to
 * toggle the smart help window (with a click, or touch based for mobile 
 * deployments). The button must exist in the game's button set image. 
 * Compatibility with plugins that alter the default engine button behavior may
 * not be supported.
 * 
 * The button is shown only when the smart help window is available. Whenever
 * it is not, the button is hidden to not confuse the player.
 * 
 * Enable Button
 * -------------
 *  This defines if the button feature is enabled or not for the whole game
 *  (applicable to all scenes where the smart help window is active). If this
 *  is set to false, all the remaining parameters under "Toggle Button" are 
 *  ignored.
 * 
 * Button Index
 * ------------
 *  Determines the button index in the system's button set image. The index 
 *  starts at 0 for the first button, and large buttons may take two indexes.
 *  Pay attention to this when setting this property.
 * 
 * Button Width
 * ------------
 *  Determines the button width type for the toggle function. If set to 1, the
 *  plugin will assume it has to load a small button. If set to 2, it will assume
 *  a large button is to be used.
 * 
 * Button X and Y
 * --------------
 *  Determines the button position in the screen. All scenes will use the same
 *  button placement, as moving it around may confuse the player. Both parameters
 *  are evals.
 *  
 * ============================================================================
 * Compatibility Warnings
 * ============================================================================
 * 
 * A few compatibility issues have already been identified which requires specific
 * configurations and settings. If you're having issues while using one of the
 * plugins listed below, be sure to read and adjust your settings accordingly.
 * 
 * MOG_BattleHud
 *  TAA_SmartHelpWindows overwrite most of MOG's changes (when the plugins are
 *  properly ordered and SmartHelpWindows is enabled for the battle scene). However,
 *  the "Layout (Picture)" property is not, and is also not supported by 
 *  SmartHelpWindows. So for this plugin to work properly along with MOG_BattleHud
 *  either keep "Layout (Picture)" disable in MOG's plugin, or disable smart
 *  help windows at the battle scene.
 *  
 *  MOG's slide animations are not a problem in itself, but they can render an odd
 *  feeling when toggling smart help windows. My general advice is to use it with
 *  caution, or (even better) keep them disabled (set to 0).
 * 
 *  A third (and final) warning about using MOG's plugin alongside SmartHelpWindows is
 *  that Custom Window Positioning for the battle scene must be either disabled or set
 *  to the same dimensions configured in MOG's plugin. If that is not taken care of,
 *  skill and item selection can have unexpected behaviors (like, for example, having 
 *  a skill item width larger than the window width).
 * 
 * Yanfly's Engine
 *  Most of Yanfly's plugin that alter scene behavior have been tested and validated.
 *  If you're using this plugin to move the other windows to account for the help
 *  window general absence, just be sure to check previous help sections to learn
 *  how to better adjust settings.
 * 
 *  To ensure compatibility with YEP_X_MoreCurrencies, be sure to place the smart
 *  help plugin below more currencies.
 * 
 * VisuStella's Engine
 *  Since all plugins are obfuscated, I can't make them compatible. If you're trying
 *  to use SmartHelpWindows and any VisuStella's plugins that alter scenes, contact
 *  them to see what can be done.
 * 
 * ============================================================================
 * Script Calls
 * ============================================================================
 * 
 * $gameSystem.clearSmartHelpChoices()
 *  Clears any previously set smart help choice texts.
 * 
 * $gameSystem.enableSmartHelpChoices()
 *  Enables the smart help choices plugin feature.
 * 
 * $gameSystem.disableSmartHelpChoices()
 *  Disables the smart help choices plugin feature.
 * 
 * $gameSystem.setSmartHelpChoices(index, text)
 *  Sets the smart help choices text for the specified index. If no index is provided
 *  (undefined or null value) then the plugin will simply push the text to the next
 *  empty choice index.
 * 
 * ============================================================================
 * Plugin Commands (MV)
 * ============================================================================
 * 
 * SmartHelpChoices clear
 *  Clears any previously set smart help choice texts.
 * 
 * SmartHelpChoices Enable
 * SmartHelpChoices Disable
 *  Enable or disable the smart help choices plugin feature.
 * 
 * SmartHelpChoices index text
 *  Sets the smart help choices text for the specified index. The index must be 
 *  specified as a number greater than zero.
 *
 * ============================================================================
 * Plugin Commands (MZ)
 * ============================================================================
 * 
 * Clear Smart Help Choices
 *  Clears any previously set smart help choice texts.
 * 
 * Enable/Disable Smart Help Choice
 *  Enable or disable the smart help choices plugin feature.
 * 
 * Set Smart Help Choice
 *  Sets the smart help choices text for the specified index. The index must be 
 *  specified as a number greater than zero.
 * 
 * ============================================================================
 * Changelog
 * ============================================================================
 *
 * Version 0.5.0:
 *  - Beta Release
 * Version 0.6.0:
 *  - Added rectangle and rounded rectangle with flat colors background for the smart
 *    help windows;
 *  - Added custom window positioning for the battle scene;
 *  - Fixed a bug which would cause issues with the help window in the skills menu;
 *  - Added a help section with compatibility warnings;
 *  - Added a new info window (which you can enable or disable) to show an informational
 *    message to the player, so he knows when the smart help window is available;
 * Version 0.7.0:
 *  - Added MZ support;
 *  - Fixed a bug that could cause the shop scene to show a misplaced help window when
 *    entering the scene;
 *  - Fixed a bug that would cause descriptions with more than one line to not position
 *    the help window properly;
 *  - Fixed a bug that would cause the custom window positioning to be always enabled;
 *  - Added a parameter to determine the preferred help window position (above or below
 *    the selected item), which substitutes the previous behavior that would automatically
 *    place above when the item is below the half of the screen, and below when above;
 *  - Changed the info window parameters, so that each scene has its own settings;
 *  - Added new parameters to custom window positioning to allow changing the number of
 *    visible columns for items and skill listings;
 * Version 0.8.0:
 *  - Fixed a compatibility issue with YEP_X_TurnOrderDisplay;
 *  - Fixed a compatibility issue with wordwrap plugins (like YEP_MessageCore), which would
 *    cause part of the help window text to be cut off, even though the window was resized
 *    correctly;
 *  - Added a parameter to turn the Input Info Window always visible for each scene, if enabled;
 * Version 1.0.0:
 *  - Added a preload of custom window skins (both for help windows and input info windows);
 *  - Fixed an issue in MV that caused a highlighted selection to be wrongly sized in MV the
 *    first time a scene was loaded;
 *  - Added compatibility with TAA_SkillMastery custom level descriptions;
 *  - Fixed a bug that would make the smart help window disappear when changing active windows;
 *  - Fixed a bug that would cause text to be cut out due to the window not moving properly;
 * Version 1.1.0:
 *  - Fixed a compatibility issue with YEP_X_MoreCurrencies;
 * Version 1.2.0:
 *  - Added parameters to include a touch based toggle button for the smart help window;
 *  - Added the option to define a background image for the smart help window;
 *  - Added commands to add smart help to the choice window;
 *  - Included support for the title and main menu scenes;
 * 
 * ============================================================================
 * End of Help
 * ============================================================================
 * 
 * 
 * 
 * =================================================================================
 * Commands (MZ)
 * =================================================================================
 * 
 * @command clearChoices
 * @text Clear Smart Help Choices
 * @desc Clears any smart help choice texts previously loaded.
 * 
 * @command setChoice
 * @text Set Smart Help Choice
 * @desc Set a text to be shown in smart help when a particular choice is highlighted.
 * 
 * @arg index
 * @text Choice Index
 * @type number
 * @min 1
 * @max 99
 * @default 1
 * @desc Choice number for the corresponding text.
 * 
 * @arg text
 * @text Smart Help Text
 * @type note
 * @default
 * @desc Smart Help text to be displayed when the choice item is highlighted.
 * 
 * @command choiceStatus
 * @text Enable/Disable Smart Help Choice
 * @desc Used to enable or disable smart help for the choice window.
 * 
 * @arg command
 * @text Enabled
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Enable or disable the smart help window for the choice window.
 * 
 * =================================================================================
 * Parameters
 * =================================================================================
 * 
 * @param ---Scenes---
 * @default --------------------
 * 
 * @param ---Battle Scene---
 * @parent ---Scenes---
 * @default ---Battle Settings---
 * 
 * @param Scene_Battle
 * @parent ---Battle Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the battle scene.
 * 
 * @param Window_BattleSkill
 * @parent ---Battle Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the skill selection window.
 * 
 * @param Window_BattleItem
 * @parent ---Battle Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for item selection window.
 * 
 * @param Battle Custom Windows
 * @parent ---Battle Scene---
 * @type struct<ScBattleSettings>
 * @default {"Custom Window Positioning":"true","Skill X":"0","Skill Y":"0","Skill Width":"Graphics.boxWidth","Skill Height":"this._statusWindow.y","Item X":"0","Item Y":"0","Item Width":"Graphics.boxWidth","Item Height":"this._statusWindow.y","Cancel Button Y":"0"}
 * @desc Customize Battle Scene remaining windows when smart help is enabled.
 * 
 * @param Battle Input Window
 * @parent ---Battle Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"0","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the battle scene input info window.
 * 
 * @param ---Item Scene---
 * @parent ---Scenes---
 * @default ---Item Settings---
 * 
 * @param Scene_Item
 * @parent ---Item Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the item scene.
 * 
 * @param Item Window_ItemCategory
 * @parent ---Item Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the category window.
 * 
 * @param Item Window_ItemList
 * @parent ---Item Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the item list window.
 * 
 * @param Item Custom Windows
 * @parent ---Item Scene---
 * @type struct<ScItemSettings>
 * @default {"Custom Window Positioning":"true","Item Category X":"0","Item Category Y":"0","Item Category Width":"Graphics.boxWidth","Item Category Rows":"1","Item List X":"0","Item List Y":"this._categoryWindow.y + this._categoryWindow.height","Item List Width":"Graphics.boxWidth","Item List Height":"Graphics.boxHeight - this._categoryWindow.height","Status X":"this._categoryWindow.width","Status Y":"0","Status Width":"Graphics.boxWidth - this._categoryWindow.width","Status Height":"this._categoryWindow.height","Info X":"this._itemWindow.width","Info Y":"this._itemWindow.y","Info Width":"Graphics.boxWidth - this._itemWindow.width","Info Height":"this._itemWindow.height"}
 * @desc Customize Item Scene remaining windows when smart help is enabled.
 * 
 * @param Item Input Window
 * @parent ---Item Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the item scene input info window.
 * 
 * @param ---Skill Scene---
 * @parent ---Scenes---
 * @default ---Skill Settings---
 * 
 * @param Scene_Skill
 * @parent ---Skill Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the skill scene.
 * 
 * @param Skill Window_SkillType
 * @parent ---Skill Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the skill type window.
 * 
 * @param Skill Window_SkillList
 * @parent ---Skill Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the skill list window.
 * 
 * @param Skill Custom Windows
 * @parent ---Skill Scene---
 * @type struct<ScSkillSettings>
 * @default {"Custom Window Positioning":"true","Skill Type X":"0","Skill Type Y":"0","Skill Type Width":"240","Skill Type Rows":"4","Skill Status X":"this._skillTypeWindow.width","Skill Status Y":"0","Skill Status Width":"Graphics.boxWidth - this._skillTypeWindow.width","Skill Status Height":"this._skillTypeWindow.height","Skill List X":"0","Skill List Y":"this._statusWindow.y + this._statusWindow.height","Skill List Width":"Graphics.boxWidth","Skill List Height":"Graphics.boxHeight - this._statusWindow.y - this._statusWindow.height"}
 * @desc Customize Skill Scene remaining windows when smart help is enabled.
 * 
 * @param Skill Input Window
 * @parent ---Skill Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the skill scene input info window.
 * 
 * @param ---Equip Scene---
 * @parent ---Scenes---
 * @default ---Equip Settings---
 * 
 * @param Scene_Equip
 * @parent ---Equip Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the equipment scene.
 * 
 * @param Equip Window_EquipCommand
 * @parent ---Equip Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the equip command window.
 * 
 * @param Equip Window_EquipSlot
 * @parent ---Equip Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the equip slot window.
 * 
 * @param Equip Window_EquipItem
 * @parent ---Equip Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the equip item window.
 * 
 * @param Equip Custom Windows
 * @parent ---Equip Scene---
 * @type struct<ScEquipSettings>
 * @default {"Custom Window Positioning":"true","Command X":"this._statusWindow.width","Command Y":"0","Command Width":"Graphics.boxWidth - this._statusWindow.width","Command Cols":"3","Command Rows":"3","Status X":"0","Status Y":"0","Status Width":"312","Status Height":"this.fittingHeight(7)","Slot X":"this._statusWindow.width","Slot Y":"this._commandWindow.y + this._commandWindow.height","Slot Width":"Graphics.boxWidth - this._statusWindow.width","Slot Height":"this._statusWindow.height - this._commandWindow.height","Item X":"0","Item Y":"this._statusWindow.y + this._statusWindow.height","Item Width":"Graphics.boxWidth","Item Height":"Graphics.boxHeight - this._statusWindow.y - this._statusWindow.height","Status Compare X":"this._itemWindow.width","Status Compare Y":"this._itemWindow.y","Status Compare Width":"Graphics.boxWidth - this._itemWindow.width","Status Compare Height":"Graphics.boxHeight - this._itemWindow.y","Info X":"this._itemWindow.width","Info Y":"this._itemWindow.y","Info Width":"Graphics.boxWidth - this._itemWindow.width","Info Height":"this._itemWindow.height"}
 * @desc Customize Equip Scene remaining windows when smart help is enabled.
 * 
 * @param Equip Input Window
 * @parent ---Equip Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the equip scene input info window.
 * 
 * @param ---Shop Scene---
 * @parent ---Scenes---
 * @default ---Shop Settings---
 * 
 * @param Scene_Shop
 * @parent ---Shop Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the shop scene.
 * 
 * @param Window_ShopBuy
 * @parent ---Shop Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the buy items list window.
 * 
 * @param Window_ShopSell
 * @parent ---Shop Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if smart help windows are enabled for the sell items list window.
 * 
 * @param Shop Window_ItemCategory
 * @parent ---Shop Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the item category window.
 * 
 * @param Shop Custom Windows
 * @parent ---Shop Scene---
 * @type struct<ScShopSettings>
 * @default {"Custom Window Positioning":"true","Gold X":"Graphics.boxWidth - 240","Gold Y":"0","Gold Width":"240","Command X":"0","Command Y":"0","Command Width":"this._goldWindow.x","Command Cols":"3","Command Rows":"1","Dummy X":"0","Dummy Y":"this._commandWindow.y + this._commandWindow.height","Dummy Width":"Graphics.boxWidth","Dummy Height":"Graphics.boxHeight - this._commandWindow.y - this._commandWindow.height","Number X":"0","Number Y":"this._commandWindow.y + this._commandWindow.height","Number Width":"456","Number Height":"Graphics.boxHeight - this._dummyWindow.y","Status X":"this._numberWindow.width","Status Y":"this._dummyWindow.y","Status Width":"Graphics.boxWidth - this._numberWindow.width","Status Height":"this._numberWindow.height","Buy X":"0","Buy Y":"this._dummyWindow.y","Buy Width":"456","Buy Height":"this._numberWindow.height","Category X":"0","Category Y":"this._commandWindow.y + this._commandWindow.height","Category Width":"Graphics.boxWidth","Category Cols":"4","Category Rows":"1","Sell X":"0","Sell Y":"this._categoryWindow.y + this._categoryWindow.height","Sell Width":"Graphics.boxWidth","Sell Height":"Graphics.boxHeight - this._categoryWindow.y - this._categoryWindow.height","Info X":"this._commandWindow.width","Info Y":"this._commandWindow.y","Info Width":"Graphics.boxWidth - this._commandWindow.width","Info Height":"this._commandWindow.height"}
 * @desc Customize Shop Scene remaining windows when smart help is enabled.
 * 
 * @param Shop Input Window
 * @parent ---Shop Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the shop scene input info window.
 * 
 * @param ---Title Scene---
 * @parent ---Scenes---
 * @default --------------------
 * 
 * @param Scene_Title
 * @parent ---Title Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the title scene.
 * 
 * @param Title Window_TitleCommand
 * @parent ---Title Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the title command window.
 * 
 * @param Title Command Texts
 * @parent ---Title Scene---
 * @type struct<ItemCatTexts>[]
 * @default ["{\"Category Name\":\"New Game\",\"Help Text\":\"\\\"Start new game\\\"\"}","{\"Category Name\":\"Continue\",\"Help Text\":\"\\\"Load last saved game\\\"\"}","{\"Category Name\":\"Credits\",\"Help Text\":\"\\\"Display game credits\\\"\"}","{\"Category Name\":\"Options\",\"Help Text\":\"\\\"Edit game settings\\\"\"}"]
 * @desc List of title command help descriptions.
 * 
 * @param Title Input Window
 * @parent ---Title Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the equip scene input info window.
 * 
 * @param ---Menu Scene---
 * @parent ---Scenes---
 * @default --------------------
 * 
 * @param Scene_Menu
 * @parent ---Menu Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the menu scene.
 * 
 * @param Menu Window_MenuCommand
 * @parent ---Menu Scene---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Defines if smart help windows are enabled for the menu command window.
 * 
 * @param Menu Command Texts
 * @parent ---Menu Scene---
 * @type struct<ItemCatTexts>[]
 * @default ["{\"Category Name\":\"Item\",\"Help Text\":\"\\\"Browse current item list\\\"\"}","{\"Category Name\":\"Skills\",\"Help Text\":\"\\\"Browse actor available skills\\\"\"}","{\"Category Name\":\"Equip\",\"Help Text\":\"\\\"Change party equipment\\\"\"}","{\"Category Name\":\"Status\",\"Help Text\":\"\\\"Check actor status\\\"\"}","{\"Category Name\":\"Books\",\"Help Text\":\"\\\"Access party book library\\\"\"}","{\"Category Name\":\"Options\",\"Help Text\":\"\\\"Customize game options\\\"\"}","{\"Category Name\":\"Save\",\"Help Text\":\"\\\"Save current game\\\"\"}","{\"Category Name\":\"Exit Game\",\"Help Text\":\"\\\"Exit to main title\\\"\"}"]
 * @desc List of menu command help descriptions.
 * 
 * @param Menu Input Window
 * @parent ---Menu Scene---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the equip scene input info window.
 * 
 * @param ---Input---
 * @default --------------------
 * 
 * @param Input Key
 * @parent ---Input---
 * @type select
 * @option tab
 * @option a
 * @option b
 * @option c
 * @option d
 * @option e
 * @option f
 * @option g
 * @option h
 * @option i
 * @option j
 * @option k
 * @option l
 * @option m
 * @option n
 * @option o
 * @option p
 * @option q
 * @option r
 * @option s
 * @option t
 * @option u
 * @option v
 * @option w
 * @option x
 * @option y
 * @option z
 * @option backspace
 * @default tab
 * @desc Select which key should be used to turn on or off the help window.
 * 
 * @param Default Setting
 * @parent ---Input---
 * @type boolean
 * @on Enabled
 * @off Disable
 * @default false
 * @desc Defines if the help windows are enabled or disabled by default.
 * 
 * @param ---Choices---
 * @default --------------------
 * 
 * @param Choices Enabled
 * @parent ---Choices---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Determines if the smart help window is enabled by default for the choice window.
 * 
 * @param Choices Input Window
 * @parent ---Choices---
 * @type struct<InputInfoWindow>
 * @default {"Enabled":"true","Always Visible":"false","Text":"Press 'tab' to toggle help","X":"Graphics.boxWidth - 320","Y":"Graphics.boxHeight - 64","Width":"320","Height":"64","Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"0","Back Opacity":"0","Background Mode":"1","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"45"}
 * @desc Settings for the choice selection input info window when on map scene.
 * 
 * @param ---Global---
 * @default --------------------
 * 
 * @param Display When Empty
 * @parent ---Global---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Defines if the smart help window will be displayed if there is no text to show.
 * 
 * @param Help Window Settings
 * @parent ---Global---
 * @type struct<HelpWindow>
 * @default {"Line Height":"36","Font Size":"20","Font Face":"GameFont","Standard Padding":"18","Text Padding":"6","Standard Opacity":"255","Back Opacity":"192","Background Mode":"0","Window Skin":"Window","Rectangle Color":"#33284f","Rectangle Alpha":"60"}
 * @desc Window settings for the smart help window.
 * 
 * @param Help Preferred Position
 * @parent ---Global---
 * @type select
 * @option Above
 * @option Below
 * @default Above
 * @desc Defines which is the preferred help window position: above or below the selected item.
 * 
 * @param Item Category Help
 * @parent ---Global---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Enable or disable item category help texts.
 * 
 * @param Item Category Texts
 * @parent ---Global---
 * @type struct<ItemCatTexts>[]
 * @default ["{\"Category Name\":\"Item\",\"Help Text\":\"\\\"Consumable Items\\\"\"}","{\"Category Name\":\"Weapon\",\"Help Text\":\"\\\"Party weapons\\\"\"}","{\"Category Name\":\"Armor\",\"Help Text\":\"\\\"Party armors\\\"\"}","{\"Category Name\":\"Key Item\",\"Help Text\":\"\\\"Special key items\\\"\"}"]
 * @desc List of item category help descriptions.
 * 
 * @param Skill Type Help
 * @parent ---Global---
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Enable or disable skill type help texts.
 * 
 * @param Skill Type Texts
 * @parent ---Global---
 * @type struct<ItemCatTexts>[]
 * @default ["{\"Category Name\":\"Special\",\"Help Text\":\"\\\"Special physical attacks\\\"\"}","{\"Category Name\":\"Magic\",\"Help Text\":\"\\\"Magical attacks\\\"\"}"]
 * @desc List of skill types help descriptions.
 * 
 * @param Equip Command Help
 * @parent ---Global---
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Enable or disable equip command help texts.
 * 
 * @param Equip Command Texts
 * @parent ---Global---
 * @type struct<ItemCatTexts>[]
 * @default ["{\"Category Name\":\"Equip\",\"Help Text\":\"\\\"Change actor equipment settings\\\"\"}","{\"Category Name\":\"Optimize\",\"Help Text\":\"\\\"Automatically optimize actor equipment settings\\\"\"}","{\"Category Name\":\"Clear\",\"Help Text\":\"\\\"Remove all actor equipment\\\"\"}"]
 * @desc List of equip command help descriptions.
 * 
 * @param ---Toggle Button---
 * @parent ---Global---
 * @default --------------------
 * 
 * @param Enable Button
 * @parent ---Toggle Button---
 * @type boolean
 * @on Enable
 * @off Disable
 * @default false
 * @desc Determines if the toggle button is enabled for the game.
 * 
 * @param Button Index
 * @parent ---Toggle Button---
 * @type number
 * @min 0
 * @max 99
 * @default 6
 * @desc Button index on the button set image. Count twice for larger buttons.
 * 
 * @param Button Width
 * @parent ---Toggle Button---
 * @type number
 * @min 1
 * @max 2
 * @default 1
 * @desc Determines if the toggle button occupies one or two indexes in the button set.
 * 
 * @param Button X
 * @parent ---Toggle Button---
 * @type text
 * @default Graphics.boxWidth - 96
 * @desc X position where to place the toggle button onscreen.
 * 
 * @param Button Y
 * @parent ---Toggle Button---
 * @type text
 * @default Graphics.boxHeight - 64
 * @desc Y position where to place the toggle button onscreen.
 * 
 */
//=============================================================================
// Input Info Window Settings
//=============================================================================
/*~struct~InputInfoWindow:
 * 
 * @param Enabled
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default true
 * @desc Enable / disable input info window.
 * 
 * @param Always Visible
 * @type boolean
 * @on Enabled
 * @off Disabled
 * @default false
 * @desc Determines if the input window should be always visible, or visible only when the active window is smart help enabled.
 *
 * @param Text
 * @type text
 * @default Press 'tab' to toggle help
 * @desc Text to be displayed at the input info window.
 * 
 * @param X
 * @type text
 * @default Graphics.boxWidth - 320
 * @desc Input Info window X position.
 * 
 * @param Y
 * @type text
 * @default Graphics.boxHeight - 64
 * @desc Input Info window Y position.
 * 
 * @param Width
 * @type text
 * @default 320
 * @desc Input info window width.
 * 
 * @param Height
 * @type text
 * @default 64
 * @desc Input info window height.
 *
 * @param Line Height
 * @type number
 * @min 1
 * @max 99
 * @default 36
 * @desc Input info window line height.
 * 
 * @param Font Size
 * @type number
 * @min 1
 * @max 99
 * @default 20
 * @desc Input info window font size.
 * 
 * @param Font Face
 * @type combo
 * @option GameFont
 * @option Arial
 * @option Courier New
 * @option SimHei
 * @option Heiti TC
 * @option Dotum
 * @option AppleGothic
 * @default GameFont
 * @desc Input info window font face.
 * 
 * @param Standard Padding
 * @type number
 * @min 1
 * @max 99
 * @default 18
 * @desc Input info window standard padding.
 * 
 * @param Text Padding
 * @type number
 * @min 1
 * @max 99
 * @default 6
 * @desc Input info window text padding.
 * 
 * @param Standard Opacity
 * @type number
 * @min 0
 * @max 255
 * @default 0
 * @desc Input info window standard opacity.
 * 
 * @param Back Opacity
 * @type number
 * @min 0
 * @max 255
 * @default 0
 * @desc Input info window back opacity.
 * 
 * @param Background Mode
 * @type select
 * @option Window Only
 * @value 0
 * @option Rectangle
 * @value 1
 * @option Rounded Rectangle
 * @value 2
 * @default 0
 * @desc If Rectangle or Rounded Rectangle are selected, the plugin will draw the form behind the window skin.
 * 
 * @param Window Skin
 * @type file
 * @dir img/system
 * @require 1
 * @default Window
 * @desc Input info window skin, if background mode is set to Window.
 * 
 * @param Rectangle Color
 * @type text
 * @default #33284f
 * @desc If background mode is set to draw a form, this defines its color.
 * 
 * @param Rectangle Alpha
 * @type number
 * @min 0
 * @max 100
 * @default 60
 * @desc If background mode is set to draw a form, this defines its transparency. 0 - fully transparent; 100 - fully opaque.
 * 
 */
//=============================================================================
// Help Window Settings
//=============================================================================
/*~struct~HelpWindow:
 * 
 * @param Line Height
 * @type number
 * @min 1
 * @max 99
 * @default 36
 * @desc Smart help windows line height.
 * 
 * @param Font Size
 * @type number
 * @min 1
 * @max 99
 * @default 20
 * @desc Smart help windows font size.
 * 
 * @param Font Face
 * @type combo
 * @option GameFont
 * @option Arial
 * @option Courier New
 * @option SimHei
 * @option Heiti TC
 * @option Dotum
 * @option AppleGothic
 * @default GameFont
 * @desc Smart help windows font face.
 * 
 * @param Standard Padding
 * @type number
 * @min 1
 * @max 99
 * @default 18
 * @desc Smart help window standard padding.
 * 
 * @param Text Padding
 * @type number
 * @min 1
 * @max 99
 * @default 6
 * @desc Smart help window text padding.
 * 
 * @param Standard Opacity
 * @type number
 * @min 0
 * @max 255
 * @default 255
 * @desc Smart help window standard opacity.
 * 
 * @param Back Opacity
 * @type number
 * @min 0
 * @max 255
 * @default 192
 * @desc Smart help window back opacity.
 * 
 * @param Background Mode
 * @type select
 * @option Window Only
 * @value 0
 * @option Rectangle
 * @value 1
 * @option Rounded Rectangle
 * @value 2
 * @option Background Image
 * @value 3
 * @default 0
 * @desc If Rectangle or Rounded Rectangle are selected, the plugin will draw the form behind the window skin.
 * 
 * @param Window Skin
 * @type file
 * @dir img/system
 * @require 1
 * @default Window
 * @desc Smart help window skin, if background mode is set to Window.
 * 
 * @param Rectangle Color
 * @type text
 * @default #33284f
 * @desc If background mode is set to draw a form, this defines its color.
 * 
 * @param Rectangle Alpha
 * @type number
 * @min 0
 * @max 100
 * @default 60
 * @desc If background mode is set to draw a form, this defines its transparency. 0 - fully transparent; 100 - fully opaque.
 * 
 * @param Background Image
 * @type file
 * @dir img/pictures/
 * @require 1
 * @default
 * @desc Image used if background mode is set to Background Image. This is ignored otherwise.
 * 
 */
//
//=============================================================================
// Item Category Custom Texts
//=============================================================================
/*~struct~ItemCatTexts:
 * 
 * @param Category Name
 * @type text
 * @default
 * @desc Item name as set in the selection window.
 * 
 * @param Help Text
 * @type note
 * @default
 * @desc Help text for the correspondent item.
 * 
 */
//=============================================================================
// Scene_Battle Custom Settings
//=============================================================================
/*~struct~ScBattleSettings:
 * 
 * @param Custom Window Positioning
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Determines if window positioning in the scene will be customized to account for the help window absence.
 * 
 * @param Skill X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Skill List Window X position. This is an eval.
 * 
 * @param Skill Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Skill LIst Window Y position. This is an eval.
 *  
 * @param Skill Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Skill List Window width. This is an eval.
 *  
 * @param Skill Height
 * @type text
 * @default this._statusWindow.y
 * @desc If settings are enabled, this param determines the Skill List Window number of visible rows.
 * 
 * @param Skill Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Skill List Window number of visible columns.
 * 
 * @param Item X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item List Window X position. This is an eval.
 * 
 * @param Item Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item List Window Y position. This is an eval.
 *  
 * @param Item Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Item List Window width. This is an eval.
 *  
 * @param Item Height
 * @type text
 * @default this._statusWindow.y
 * @desc If settings are enabled, this param determines the Item List Window height. This is an eval.
 * 
 * @param Item Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Item List Window number of visible columns.
 * 
 * @param Cancel Button Y
 * @type text
 * @default 0
 * @desc On MZ, if settings are enabled, this param determines the cancel button Y position. This is an eval.
 * 
 */
//=============================================================================
// Scene_Item Custom Settings
//=============================================================================
/*~struct~ScItemSettings:
 * 
 * @param Custom Window Positioning
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Determines if window positioning in the scene will be customized to account for the help window absence.
 * 
 * @param Item Category X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item Category Window X position. This is an eval.
 * 
 * @param Item Category Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item Category Window Y position. This is an eval.
 *  
 * @param Item Category Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Item Category Window width. This is an eval.
 *  
 * @param Item Category Rows
 * @type number
 * @min 1
 * @max 99
 * @default 1
 * @desc If settings are enabled, this param determines the Item Category Window number of visible rows.
 * 
 * @param Item List X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item List Window X position. This is an eval.
 * 
 * @param Item List Y
 * @type text
 * @default this._categoryWindow.y + this._categoryWindow.height
 * @desc If settings are enabled, this param determines the Item List Window Y position. This is an eval.
 *  
 * @param Item List Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Item List Window width. This is an eval.
 *  
 * @param Item List Height
 * @type text
 * @default Graphics.boxHeight - this._categoryWindow.height
 * @desc If settings are enabled, this param determines the Item List Window height. This is an eval.
 * 
 * @param Item List Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Item List Window number of visible columns.
 * 
 * @param Status X
 * @type text
 * @default this._categoryWindow.width
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the status window X. This is an eval.
 * 
 * @param Status Y
 * @type text
 * @default 0
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the status window Y. This is an eval.
 * 
 * @param Status Width
 * @type text
 * @default Graphics.boxWidth - this._categoryWindow.width
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the status window width. This is an eval.
 * 
 * @param Status Height
 * @type text
 * @default this._categoryWindow.height
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the status window width. This is an eval.
 * 
 * @param Info X
 * @type text
 * @default this._itemWindow.width
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the info window X. This is an eval.
 * 
 * @param Info Y
 * @type text
 * @default this._itemWindow.y
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the info window Y. This is an eval.
 * 
 * @param Info Width
 * @type text
 * @default Graphics.boxWidth - this._itemWindow.width
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the info window width. This is an eval.
 * 
 * @param Info Height
 * @type text
 * @default this._itemWindow.height
 * @desc If settings are enabled (and YEP_ItemCore scene is also enabled), determines the info window width. This is an eval.
 * 
 */
//=============================================================================
// Scene_Skill Custom Settings
//=============================================================================
/*~struct~ScSkillSettings:
 * 
 * @param Custom Window Positioning
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Determines if window positioning in the scene will be customized to account for the help window absence.
 * 
 * @param Skill Type X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Skill Type Window X position. This is an eval.
 * 
 * @param Skill Type Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Skill Type Window Y position. This is an eval.
 *  
 * @param Skill Type Width
 * @type text
 * @default 240
 * @desc If settings are enabled, this param determines the Skill Type Window width. This is an eval.
 *  
 * @param Skill Type Rows
 * @type number
 * @min 1
 * @max 99
 * @default 4
 * @desc If settings are enabled, this param determines the Skill Type Window number of visible rows.
 * 
 * @param Skill Status X
 * @type text
 * @default this._skillTypeWindow.width
 * @desc If settings are enabled, this param determines the Skill Status Window X position. This is an eval.
 * 
 * @param Skill Status Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Skill Status Window Y position. This is an eval.
 *  
 * @param Skill Status Width
 * @type text
 * @default Graphics.boxWidth - this._skillTypeWindow.width
 * @desc If settings are enabled, this param determines the Skill Status Window width. This is an eval.
 *  
 * @param Skill Status Height
 * @type text
 * @default this._skillTypeWindow.height
 * @desc If settings are enabled, this param determines the Skill Status Window height. This is an eval.
 * 
 * @param Skill List X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Skill List window X. This is an eval.
 * 
 * @param Skill List Y
 * @type text
 * @default this._statusWindow.y + this._statusWindow.height
 * @desc If settings are enabled, this param determines the Skill List window Y. This is an eval.
 * 
 * @param Skill List Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Skill List window width. This is an eval.
 * 
 * @param Skill List Height
 * @type text
 * @default Graphics.boxHeight - this._statusWindow.y - this._statusWindow.height
 * @desc If settings are enabled, this param determines the Skill List window width. This is an eval.
 * 
 * @param Skill List Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Skill List Window number of visible columns.
 * 
 */
//=============================================================================
// Scene_Equip Custom Settings
//=============================================================================
/*~struct~ScEquipSettings:
 * 
 * @param Custom Window Positioning
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Determines if window positioning in the scene will be customized to account for the help window absence.
 * 
 * @param Command X
 * @type text
 * @default this._statusWindow.width
 * @desc If settings are enabled, this param determines the Command Window X position. This is an eval.
 * 
 * @param Command Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Command Window Y position. This is an eval.
 *  
 * @param Command Width
 * @type text
 * @default Graphics.boxWidth - this._statusWindow.width
 * @desc If settings are enabled, this param determines the Command Window width. This is an eval.
 *  
 * @param Command Cols
 * @type number
 * @min 1
 * @max 99
 * @default 3
 * @desc If settings are enabled, this param determines the Command Window number of visible columns.
 *  
 * @param Command Rows
 * @type number
 * @min 1
 * @max 99
 * @default 1
 * @desc If settings are enabled, this param determines the Command Window number of visible rows.
 * 
 * @param Status X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Status Window X position. This is an eval.
 * 
 * @param Status Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Status Window Y position. This is an eval.
 *  
 * @param Status Width
 * @type text
 * @default 312
 * @desc If settings are enabled, this param determines the Status Window width. This is an eval.
 *  
 * @param Status Height
 * @type text
 * @default this.fittingHeight(7)
 * @desc If settings are enabled, this param determines the Status Window number of visible rows.
 * 
 * @param Slot X
 * @type text
 * @default this._statusWindow.width
 * @desc If settings are enabled, this param determines the Slot window X. This is an eval.
 * 
 * @param Slot Y
 * @type text
 * @default this._commandWindow.y + this._commandWindow.height
 * @desc If settings are enabled, this param determines the Slot window Y. This is an eval.
 * 
 * @param Slot Width
 * @type text
 * @default Graphics.boxWidth - this._statusWindow.width
 * @desc If settings are enabled, this param determines the Slot window width. This is an eval.
 * 
 * @param Slot Height
 * @type text
 * @default this._statusWindow.height - this._commandWindow.height
 * @desc If settings are enabled, this param determines the Slot window width. This is an eval.
 * 
 * @param Item X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item List X. This is an eval.
 * 
 * @param Item Y
 * @type text
 * @default this._statusWindow.y + this._statusWindow.height
 * @desc If settings are enabled, this param determines the Item List Y. This is an eval.
 * 
 * @param Item Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Item List Width. This is an eval.
 * 
 * @param Item Height
 * @type text
 * @default Graphics.boxHeight - this._statusWindow.y - this._statusWindow.height
 * @desc If settings are enabled, this param determines the Item List Height. This is an eval.
 * 
 * @param Item Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Item List Window number of visible columns.
 * 
 * @param Status Compare X
 * @type text
 * @default this._itemWindow.width
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the stat compare window X. This is an eval.
 * 
 * @param Status Compare Y
 * @type text
 * @default this._itemWindow.y
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the stat compare window Y. This is an eval.
 * 
 * @param Status Compare Width
 * @type text
 * @default Graphics.boxWidth - this._itemWindow.width
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the stat compare window Width. This is an eval.
 * 
 * @param Status Compare Height
 * @type text
 * @default Graphics.boxHeight - this._itemWindow.y
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the stat compare window Height. This is an eval.
 * 
 * @param Info X
 * @type text
 * @default this._itemWindow.width
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the info window X. This is an eval.
 * 
 * @param Info Y
 * @type text
 * @default this._itemWindow.y
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the info window Y. This is an eval.
 * 
 * @param Info Width
 * @type text
 * @default Graphics.boxWidth - this._itemWindow.width
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the info window Width. This is an eval.
 * 
 * @param Info Height
 * @type text
 * @default this._itemWindow.height
 * @desc If settings are enabled (and YEP_EquipCore scene is also enabled), determines the info window Height. This is an eval.
 * 
 */
//=============================================================================
// Scene_Shop Custom Settings
//=============================================================================
/*~struct~ScShopSettings:
 * 
 * @param Custom Window Positioning
 * @type boolean
 * @on Enable
 * @off Disable
 * @default true
 * @desc Determines if window positioning in the scene will be customized to account for the help window absence.
 * 
 * @param Gold X
 * @type text
 * @default Graphics.boxWidth - 240
 * @desc If settings are enabled, this param determines the Gold Window X position. This is an eval.
 * 
 * @param Gold Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Gold Window Y position. This is an eval.
 *  
 * @param Gold Width
 * @type text
 * @default 240
 * @desc If settings are enabled, this param determines the Gold Window width. This is an eval.
 * 
 * @param Command X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Command Window X position. This is an eval.
 * 
 * @param Command Y
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Command Window Y position. This is an eval.
 *  
 * @param Command Width
 * @type text
 * @default this._goldWindow.x
 * @desc If settings are enabled, this param determines the Command Window width. This is an eval.
 *  
 * @param Command Cols
 * @type number
 * @min 1
 * @max 99
 * @default 3
 * @desc If settings are enabled, this param determines the Command Window number of columns. This is an eval.
 * 
 * @param Command Rows
 * @type number
 * @min 1
 * @max 99
 * @default 1
 * @desc If settings are enabled, this param determines the Command Window number of rows. This is an eval.
 * 
 * @param Dummy X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Dummy Window X. This is an eval.
 * 
 * @param Dummy Y
 * @type text
 * @default this._commandWindow.y + this._commandWindow.height
 * @desc If settings are enabled, this param determines the Dummy Window Y. This is an eval.
 * 
 * @param Dummy Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Dummy Window Width. This is an eval.
 * 
 * @param Dummy Height
 * @type text
 * @default Graphics.boxHeight - this._commandWindow.y - this._commandWindow.height
 * @desc If settings are enabled, this param determines the Dummy Window Height. This is an eval.
 * 
 * @param Number X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Number Window X. This is an eval.
 * 
 * @param Number Y
 * @type text
 * @default this._commandWindow.y + this._commandWindow.height
 * @desc If settings are enabled, this param determines the Number Window Y. This is an eval.
 * 
 * @param Number Width
 * @type text
 * @default 456
 * @desc If settings are enabled, this param determines the Number Window Width. This is an eval.
 * 
 * @param Number Height
 * @type text
 * @default Graphics.boxHeight - this._dummyWindow.y
 * @desc If settings are enabled, this param determines the Number Window Height. This is an eval.
 * 
 * @param Status X
 * @type text
 * @default this._numberWindow.width
 * @desc If settings are enabled, this param determines the Status Window X. This is an eval.
 * 
 * @param Status Y
 * @type text
 * @default this._dummyWindow.y
 * @desc If settings are enabled, this param determines the Status Window Y. This is an eval.
 * 
 * @param Status Width
 * @type text
 * @default Graphics.boxWidth - this._numberWindow.width
 * @desc If settings are enabled, this param determines the Status Window Width. This is an eval.
 * 
 * @param Status Height
 * @type text
 * @default this._numberWindow.height
 * @desc If settings are enabled, this param determines the Status Window Height. This is an eval.
 * 
 * @param Buy X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Buy window X. This is an eval.
 * 
 * @param Buy Y
 * @type text
 * @default this._dummyWindow.y
 * @desc If settings are enabled, this param determines the Buy window Y. This is an eval.
 * 
 * @param Buy Width
 * @type text
 * @default 456
 * @desc If settings are enabled, this param determines the Buy window width. This is an eval.
 * 
 * @param Buy Height
 * @type text
 * @default this._numberWindow.height
 * @desc If settings are enabled, this param determines the Buy window width. This is an eval.
 * 
 * @param Buy Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Buy Window number of visible columns.
 * 
 * @param Category X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Item Category X. This is an eval.
 * 
 * @param Category Y
 * @type text
 * @default this._dummyWindow.y
 * @desc If settings are enabled, this param determines the Item Category Y. This is an eval.
 * 
 * @param Category Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Item Category Width. This is an eval.
 * 
 * @param Category Cols
 * @type text
 * @default 4
 * @desc If settings are enabled, this param determines the Item Category Cols. This is an eval.
 * 
 * @param Category Rows
 * @type text
 * @default 1
 * @desc If settings are enabled, this param determines the Item Category Rows. This is an eval.
 * 
 * @param Sell X
 * @type text
 * @default 0
 * @desc If settings are enabled, this param determines the Sell window X. This is an eval.
 * 
 * @param Sell Y
 * @type text
 * @default this._dummyWindow.y
 * @desc If settings are enabled, this param determines the Sell window Y. This is an eval.
 * 
 * @param Sell Width
 * @type text
 * @default Graphics.boxWidth
 * @desc If settings are enabled, this param determines the Sell window Width. This is an eval.
 * 
 * @param Sell Height
 * @type text
 * @default Graphics.boxHeight - this._dummyWindow.y
 * @desc If settings are enabled, this param determines the Sell window Height. This is an eval.
 * 
 * @param Sell Cols
 * @type number
 * @min 1
 * @max 99
 * @default 2
 * @desc If settings are enabled, this param determines the Sell Window number of visible columns.
 * 
 * @param Info X
 * @type text
 * @default this._commandWindow.width
 * @desc If settings are enabled (and YEP_ShopMenuCore scene is also enabled), determines the info window X. This is an eval.
 * 
 * @param Info Y
 * @type text
 * @default this._commandWindow.y
 * @desc If settings are enabled (and YEP_ShopMenuCore scene is also enabled), determines the info window Y. This is an eval.
 * 
 * @param Info Width
 * @type text
 * @default Graphics.boxWidth - this._commandWindow.width
 * @desc If settings are enabled (and YEP_ShopMenuCore scene is also enabled), determines the info window Width. This is an eval.
 * 
 * @param Info Height
 * @type text
 * @default this._commandWindow.height
 * @desc If settings are enabled (and YEP_ShopMenuCore scene is also enabled), determines the info window Height. This is an eval.
 * 
 */
//=============================================================================
// Local Functions
//=============================================================================

TAA.shw.functions = TAA.shw.functions || {};
TAA.shw.functions.parse = function(param){
    var result = {};
    try{
        result = JSON.parse(param);
    } catch(e){
        console.error("TAA_SmartHelpWindows: Failure to parse parameter");
        result['Custom Window Positioning'] = false;
    }
    if(result['Custom Window Positioning']){
        result['Custom Window Positioning'] = result['Custom Window Positioning'] === 'true';
    }
    return result;
};

TAA.shw.functions.parseArray = function(param){
    var result = {};
    var tmp = [];
    try{
        tmp = JSON.parse(param);
        for(var i=0; i<tmp.length; i++){
            var obj = {};
            var p = JSON.parse(tmp[i]);
            var key = p['Category Name'];
            obj[key] = JSON.parse(p['Help Text']);
            result[key] = obj[key];
        }
    } catch(e){
        console.error("TAA_SmartHelpWindows: Failure to parse array parameter");
    }
    return result;
};

TAA.shw.functions.parseHelpSettings = function(param){
    var result = {};
    try{
        var tmp = JSON.parse(param);
        result.LineHeight = isNaN(tmp['Line Height']) ? 36 : parseInt(tmp['Line Height']);
        result.FontSize = isNaN(tmp['Font Size']) ? 20 : parseInt(tmp['Font Size']);
        result.FontFace = tmp['Font Face'] || "GameFont";
        result.StandardPadding = isNaN(tmp['Standard Padding']) ? 18 : parseInt(tmp['Standard Padding']);
        result.TextPadding = isNaN(tmp['Text Padding']) ? 6 : parseInt(tmp['Text Padding']);
        result.StandardOpacity = isNaN(tmp['Standard Opacity']) ? 255 : parseInt(tmp['Standard Opacity']);
        result.BackOpacity = isNaN(tmp['Back Opacity']) ? 192 : parseInt(tmp['Back Opacity']);
        result.WindowSkin = tmp['Window Skin'];
        result.BackgroundMode = isNaN(tmp['Background Mode']) ? 0 : parseInt(tmp['Background Mode']);
        result.RectangleColor = this.getCSSColorFromString(tmp['Rectangle Color']);
        result.RectangleAlpha = isNaN(tmp['Rectangle Alpha']) ? 0.6 : parseFloat((parseInt(tmp['Rectangle Alpha']) / 100));
        if(tmp['Background Image']) result.BackgroundImg = tmp['Background Image'];
    } catch(e){
        console.error("TAA_SmartHelpWindows - Failed to parse help window settings.");
        result = {
            LineHeight:36,
            FontSize:20,
            FontFace:"GameFont",
            StandardPadding:18,
            TextPadding:6,
            StandardOpacity:255,
            BackOpacity:192,
            WindowSkin:"Window",
            BackgroundMode:0,
            RectangleColor:"#33284f",
            RectangleAlpha:0.6
        };
    }
    return result;
};

TAA.shw.functions.parseInputInfoSettings = function(param){
    var result = {};
    try{
        var tmp = JSON.parse(param);
        result.Enabled = tmp['Enabled'] === 'true';
        result.AlwaysVisible = tmp['Always Visible'] === 'true';
        result.Text = tmp['Text'];
        result.X = tmp['X'];
        result.Y = tmp['Y'];
        result.Width = tmp['Width'];
        result.Height = tmp['Height'];
        result.LineHeight = isNaN(tmp['Line Height']) ? 36 : parseInt(tmp['Line Height']);
        result.FontSize = isNaN(tmp['Font Size']) ? 20 : parseInt(tmp['Font Size']);
        result.FontFace = tmp['Font Face'] || "GameFont";
        result.StandardPadding = isNaN(tmp['Standard Padding']) ? 18 : parseInt(tmp['Standard Padding']);
        result.TextPadding = isNaN(tmp['Text Padding']) ? 6 : parseInt(tmp['Text Padding']);
        result.StandardOpacity = isNaN(tmp['Standard Opacity']) ? 0 : parseInt(tmp['Standard Opacity']);
        result.BackOpacity = isNaN(tmp['Back Opacity']) ? 0 : parseInt(tmp['Back Opacity']);
        result.WindowSkin = tmp['Window Skin'];
        result.BackgroundMode = isNaN(tmp['Background Mode']) ? 0 : parseInt(tmp['Background Mode']);
        result.RectangleColor = this.getCSSColorFromString(tmp['Rectangle Color']);
        result.RectangleAlpha = isNaN(tmp['Rectangle Alpha']) ? 0.6 : parseFloat((parseInt(tmp['Rectangle Alpha']) / 100));
    } catch(e){
        console.error("TAA_SmartHelpWindows - Failed to parse input info window settings.");
        result = {
            Enabled:true,
            AlwaysVisible:false,
            X:"Graphics.boxWidth - 320",
            Y:"Graphics.boxHeight - 64",
            Width:"320",
            Height:"64",
            LineHeight:36,
            FontSize:20,
            FontFace:"GameFont",
            StandardPadding:18,
            TextPadding:6,
            StandardOpacity:0,
            BackOpacity:0,
            WindowSkin:"Window",
            BackgroundMode:2,
            RectangleColor:"#33284f",
            RectangleAlpha:0.6
        };
    }
    return result;
};

TAA.shw.functions.getCSSColorFromString = function(color){
    if(color.match(/^\s*#?([0-9a-f]{1,6})\s*$/i)){
        var colorCode = new String(RegExp.$1);
        var trailingZeros = 6 - colorCode.length;
        while(trailingZeros > 0) {
            colorCode = '0' + colorCode;
            trailingZeros--;
        }

        return '#'+colorCode;
    }
    return '#33284f';
};

TAA.shw.functions.parseCoordinates = function(coord){
    var result = 0;
    try{
        result = eval(coord);
    } catch(e){
        console.error("TAA_SmartHelpWindows - Failed to parse coordinate: " + coord);
        result = 0;
    }
    return result;
};

//=============================================================================
// Parameters
//=============================================================================

TAA.shw.Parameters = TAA.shw.Parameters || {};
var Parameters = PluginManager.parameters(TAA.shw.PluginName);

TAA.shw.Parameters.Scenes = {};
TAA.shw.Parameters.Scenes['Scene_Battle'] = Parameters['Scene_Battle'] === 'true';
TAA.shw.Parameters.Scenes['Scene_Item'] = Parameters['Scene_Item'] === 'true';
TAA.shw.Parameters.Scenes['Scene_Skill'] = Parameters['Scene_Skill'] === 'true';
TAA.shw.Parameters.Scenes['Scene_Equip'] = Parameters['Scene_Equip'] === 'true';
TAA.shw.Parameters.Scenes['Scene_Shop'] = Parameters['Scene_Shop'] === 'true';
TAA.shw.Parameters.Scenes['Scene_Title'] = Parameters['Scene_Title'] === 'true';
TAA.shw.Parameters.Scenes['Scene_Menu'] = Parameters['Scene_Menu'] === 'true';

TAA.shw.Parameters.Windows = {};
TAA.shw.Parameters.Windows['Scene_Battle'] = {};
TAA.shw.Parameters.Windows['Scene_Battle']['Window_BattleSkill'] = Parameters['Window_BattleSkill'] === 'true';
TAA.shw.Parameters.Windows['Scene_Battle']['Window_BattleItem'] = Parameters['Window_BattleItem'] === 'true';

TAA.shw.Parameters.Windows['Scene_Item'] = {};
TAA.shw.Parameters.Windows['Scene_Item']['Window_ItemCategory'] = Parameters['Item Window_ItemCategory'] === 'true';
TAA.shw.Parameters.Windows['Scene_Item']['Window_ItemList'] = Parameters['Item Window_ItemList'] === 'true';

TAA.shw.Parameters.Windows['Scene_Skill'] = {};
TAA.shw.Parameters.Windows['Scene_Skill']['Window_SkillType'] = Parameters['Skill Window_SkillType'] === 'true';
TAA.shw.Parameters.Windows['Scene_Skill']['Window_SkillList'] = Parameters['Skill Window_SkillList'] === 'true';

TAA.shw.Parameters.Windows['Scene_Equip'] = {}
TAA.shw.Parameters.Windows['Scene_Equip']['Window_EquipCommand'] = Parameters['Equip Window_EquipCommand'] === 'true';
TAA.shw.Parameters.Windows['Scene_Equip']['Window_EquipSlot'] = Parameters['Equip Window_EquipSlot'] === 'true';
TAA.shw.Parameters.Windows['Scene_Equip']['Window_EquipItem'] = Parameters['Equip Window_EquipItem'] === 'true';

TAA.shw.Parameters.Windows['Scene_Shop'] = {};
TAA.shw.Parameters.Windows['Scene_Shop']['Window_ShopBuy'] = Parameters['Window_ShopBuy'] === 'true';
TAA.shw.Parameters.Windows['Scene_Shop']['Window_ShopSell'] = Parameters['Window_ShopSell'] === 'true';
TAA.shw.Parameters.Windows['Scene_Shop']['Window_ItemCategory'] = Parameters['Shop Window_ItemCategory'] === 'true';
if(Imported.YEP_ShopMenuCore){
    TAA.shw.Parameters.Windows['Scene_Shop']['Window_ShopCategory'] = TAA.shw.Parameters.Windows['Scene_Shop']['Window_ItemCategory'];
}

TAA.shw.Parameters.Windows['Scene_Title'] = {};
TAA.shw.Parameters.Windows['Scene_Title']['Window_TitleCommand'] = Parameters['Title Window_TitleCommand'] === 'true';

TAA.shw.Parameters.Windows['Scene_Menu'] = {};
TAA.shw.Parameters.Windows['Scene_Menu']['Window_MenuCommand'] = Parameters['Menu Window_MenuCommand'] === 'true';

TAA.shw.Parameters.Custom = {};
TAA.shw.Parameters.Custom['Scene_Battle'] = TAA.shw.functions.parse(Parameters['Battle Custom Windows']);
TAA.shw.Parameters.Custom['Scene_Item'] = TAA.shw.functions.parse(Parameters['Item Custom Windows']);
TAA.shw.Parameters.Custom['Scene_Skill'] = TAA.shw.functions.parse(Parameters['Skill Custom Windows']);
TAA.shw.Parameters.Custom['Scene_Equip'] = TAA.shw.functions.parse(Parameters['Equip Custom Windows']);
TAA.shw.Parameters.Custom['Scene_Shop'] = TAA.shw.functions.parse(Parameters['Shop Custom Windows']);

TAA.shw.Parameters.InputWindow = {};
TAA.shw.Parameters.InputWindow['Scene_Battle'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Battle Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Item'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Item Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Skill'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Skill Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Equip'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Equip Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Shop'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Shop Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Map'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Choices Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Title'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Title Input Window']);
TAA.shw.Parameters.InputWindow['Scene_Menu'] = TAA.shw.functions.parseInputInfoSettings(Parameters['Menu Input Window']);

TAA.shw.Parameters.Input = {};
TAA.shw.Parameters.Input.Key = Parameters['Input Key'];
TAA.shw.Parameters.Input.Default = Parameters['Default Setting'] === 'true';

TAA.shw.Parameters.Choices = {};
TAA.shw.Parameters.Choices.Enabled = Parameters['Choices Enabled'] === 'true';

TAA.shw.Parameters.Global = {};
TAA.shw.Parameters.Global.EmptyEnabled = Parameters['Display When Empty'] === 'true';
TAA.shw.Parameters.Global.HelpWindow = TAA.shw.functions.parseHelpSettings(Parameters['Help Window Settings']);
TAA.shw.Parameters.Global.HelpPosition = Parameters['Help Preferred Position'] || "Above";
TAA.shw.Parameters.Global.ItemCatHelp = {};
TAA.shw.Parameters.Global.ItemCatHelp.Enabled = Parameters['Item Category Help'] === 'true';
TAA.shw.Parameters.Global.ItemCatHelp.Texts = TAA.shw.functions.parseArray(Parameters['Item Category Texts']);
TAA.shw.Parameters.Global.SkillTypeHelp = {};
TAA.shw.Parameters.Global.SkillTypeHelp.Enabled = Parameters['Skill Type Help'] === 'true';
TAA.shw.Parameters.Global.SkillTypeHelp.Texts = TAA.shw.functions.parseArray(Parameters['Skill Type Texts']);
TAA.shw.Parameters.Global.EquipCommandHelp = {};
TAA.shw.Parameters.Global.EquipCommandHelp.Enabled = Parameters['Equip Command Help'] === 'true';
TAA.shw.Parameters.Global.EquipCommandHelp.Texts = TAA.shw.functions.parseArray(Parameters['Equip Command Texts']);
TAA.shw.Parameters.Global.TitleCommandHelp = {};
TAA.shw.Parameters.Global.TitleCommandHelp.Enabled = Parameters['Title Window_TitleCommand'] === 'true';
TAA.shw.Parameters.Global.TitleCommandHelp.Texts = TAA.shw.functions.parseArray(Parameters['Title Command Texts']);
TAA.shw.Parameters.Global.MenuCommandHelp = {};
TAA.shw.Parameters.Global.MenuCommandHelp.Enabled = Parameters['Menu Window_MenuCommand'] === 'true';
TAA.shw.Parameters.Global.MenuCommandHelp.Texts = TAA.shw.functions.parseArray(Parameters['Menu Command Texts']);

TAA.shw.Parameters.ToggleButton = TAA.shw.Parameters.ToggleButton || {};
TAA.shw.Parameters.ToggleButton.Enable = Parameters['Enable Button'] === 'true';
TAA.shw.Parameters.ToggleButton.Index = !isNaN(Parameters['Button Index']) ? parseInt(Parameters['Button Index']) : 0;
TAA.shw.Parameters.ToggleButton.Width = !isNaN(Parameters['Button Width']) ? parseInt(Parameters['Button Width']) : 1;
TAA.shw.Parameters.ToggleButton.X = Parameters['Button X'];
TAA.shw.Parameters.ToggleButton.Y = Parameters['Button Y'];

//=============================================================================
// Input
//=============================================================================

Input.keyMapper[8] = "backspace";
Input.keyMapper[9] = 'tab';
Input.letters = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'];
for(var i=0; i<Input.letters.length-1; i++){
    Input.keyMapper[i+65] = Input.letters[i];
}

//=============================================================================
// SmartHelpCache
//=============================================================================

if(!SmartHelpCache){
    function SmartHelpCache() {
        throw new Error('This is a static class');
    };
}

/**
 * Initializes Texture Manager
 * 
 * @static
 * @method initialize
 */
SmartHelpCache.initialize = function(){
    if(TAA.shw.Parameters.Global.HelpWindow.BackgroundMode === 3 && TAA.shw.Parameters.Global.HelpWindow.BackgroundImg){
        this._shwBackground = ImageManager.loadPicture(TAA.shw.Parameters.Global.HelpWindow.BackgroundImg);
    }
    else{
        this._shwBackground = null;
    }

    this._initialized = true;
};

SmartHelpCache.getBackgroundImg = function(){
    return this._shwBackground;
};

//=============================================================================
// Button Customizations
//=============================================================================

if(Utils.RPGMAKER_NAME === 'MV'){
    function Sprite_CustomButton(){
        this.initialize.apply(this, arguments);
    };

    Sprite_CustomButton.prototype = Object.create(Sprite_Button.prototype);
    Sprite_CustomButton.prototype.constructor = Sprite_CustomButton;

    Sprite_CustomButton.prototype.initialize = function(buttonType){
        Sprite_Button.prototype.initialize.call(this);
        this._buttonType = buttonType;
        this.setupFrames();
    };

    Sprite_CustomButton.prototype.setupFrames = function(){
        const data = this.buttonData();
        if(!data) return;
        const x = data.x * this.blockWidth();
        const width = data.w * this.blockWidth();
        const height = this.blockHeight();
        this.loadButtonImage();
        this.setColdFrame(x, 0, width, height);
        this.setHotFrame(x, height, width, height);
        this.updateFrame();
        this.updateOpacity();
    };

    Sprite_CustomButton.prototype.blockWidth = function(){
        return 48;
    };

    Sprite_CustomButton.prototype.blockHeight = function(){
        return 48;
    };

    Sprite_CustomButton.prototype.loadButtonImage = function(){
        this.bitmap = ImageManager.loadSystem('ButtonSet');
    };

    Sprite_CustomButton.prototype.buttonData = function(){
        const buttonTable = {
            pagedown: { x: 0, w: 1 },
            down: { x: 1, w: 1 },
            up: { x: 2, w: 1 },
            pageup: { x: 3, w: 1 },
            ok: { x: 4, w: 2 },
            toggleHelp: { x: TAA.shw.Parameters.ToggleButton.Index, w: TAA.shw.Parameters.ToggleButton.Width }
        };
        return buttonTable[this._buttonType];
    }; 

    Sprite_CustomButton.prototype.updateOpacity = function() {
        this.opacity = this._pressed ? 255 : 192;
    };
}
else{
    TAA.shw.alias.Sprite_Button = TAA.shw.alias.Sprite_Button || {};
    TAA.shw.alias.Sprite_Button.buttonData = Sprite_Button.prototype.buttonData;
    Sprite_Button.prototype.buttonData = function(){
        if(this._buttonType === 'toggleHelp')
            return { x: TAA.shw.Parameters.ToggleButton.Index, w: TAA.shw.Parameters.ToggleButton.Width };
        return TAA.shw.alias.Sprite_Button.buttonData.call(this);
    }; 
}

//=============================================================================
// Game_System
//=============================================================================

TAA.shw.alias.Game_System = TAA.shw.alias.Game_System || {};
TAA.shw.alias.Game_System.initialize = Game_System.prototype.initialize;
Game_System.prototype.initialize = function(){
    TAA.shw.alias.Game_System.initialize.call(this);
    if(!SmartHelpCache._initialized)
        SmartHelpCache.initialize();
    this._smartHelpWindow = TAA.shw.Parameters.Scenes;
    this._smartHelpWindow['Scene_Map'] = true;
    this._smartHelpWindowKey = TAA.shw.Parameters.Input.Key;
    this._smartHelpWindowVisibility = TAA.shw.Parameters.Input.Default === true;
    this._smartHelpWindowHideWhenEmpty = TAA.shw.Parameters.Global.EmptyEnabled !== true;
    this._smartHelpChoices = [];
    this._smartHelpEnabledForChoices = TAA.shw.Parameters.Choices.Enabled === true;
};

Game_System.prototype.isSmartHelpEnabledForScene = function(sceneName){
    return this._smartHelpWindow[sceneName] === true;
};

Game_System.prototype.isSmartHelpEnabledForChoices = function(){
    return this._smartHelpEnabledForChoices;
};

Game_System.prototype.getSmartHelpWindowKey = function(){
    return this._smartHelpWindowKey;
};

Game_System.prototype.isSmartHelpWindowVisible = function(){
    return this._smartHelpWindowVisibility;
};

Game_System.prototype.smartHelpWindowMakeVisible = function(){
    this._smartHelpWindowVisibility = true;
};

Game_System.prototype.smartHelpWindowHide = function(){
    this._smartHelpWindowVisibility = false;
};

Game_System.prototype.shouldHideSmartHelpWhenEmpty = function(){
    return this._smartHelpWindowHideWhenEmpty;
};

Game_System.prototype.hideSmartHelpWhenEmpty = function(){
    this._smartHelpWindowHideWhenEmpty = true;
    if(SceneManager._scene._helpWindow)
        SceneManager._scene._helpWindow.setHideWhenEmpty();
};

Game_System.prototype.showSmartHelpWhenEmpty = function(){
    this._smartHelpWindowHideWhenEmpty = false;
    if(SceneManager._scene._helpWindow)
        SceneManager._scene._helpWindow.setShowWhenEmpty();
};

Game_System.prototype.disableSmartHelpForChoices = function(){
    this._smartHelpEnabledForChoices = false;
};

Game_System.prototype.enableSmartHelpForChoices = function(){
    this._smartHelpEnabledForChoices = true;
};

Game_System.prototype.clearSmartHelpChoices = function(){
    this._smartHelpChoices = [];
};

Game_System.prototype.setSmartHelpChoices = function(index, text){
    if(isNaN(index)) this._smartHelpChoices.push(text);
    else this._smartHelpChoices[index] = text;
};

Game_System.prototype.getSmartHelpChoiceText = function(index){
    if(isNaN(index) || this._smartHelpChoices[index] === undefined || this._smartHelpChoices[index] === null) return "";
    return this._smartHelpChoices[index];
};

Game_System.prototype.isSmartHelpEnabledForWindow = function(window){
    if(window === 'Window_ChoiceList') return this.isSmartHelpEnabledForChoices();
    var scene = SceneManager._scene.constructor.name;
    if(!TAA.shw.Parameters.Windows[scene])
        return false;
    return TAA.shw.Parameters.Windows[scene][window] === true;
};

//=============================================================================
// Game_Interpreter (MV)
//=============================================================================

TAA.shw.alias.Game_Interpreter = TAA.shw.alias.Game_Interpreter || {};
TAA.shw.alias.Game_Interpreter.pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function(command, args){
    TAA.shw.alias.Game_Interpreter.pluginCommand.call(this, command, args);
    if(command.toLowerCase() === 'smarthelpchoices'){
        if(args[0].toLowerCase() === 'clear'){
            $gameSystem.clearSmartHelpChoices();
        }
        else if(args[0].toLowerCase() === 'enable'){
            $gameSystem.enableSmartHelpForChoices();
        }
        else if(args[0].toLowerCase() === 'disable'){
            $gameSystem.disableSmartHelpForChoices();
        }
        else if(!isNaN(args[0])){
            let index = parseInt(args[0]) - 1;
            if(index === undefined || index < 0) return;
            let text = args[1] || "";
            for(var i=2; i<args.length; i++){
                text += " " + args[i];
            }
            $gameSystem.setSmartHelpChoices(index, text);
        }
    }
};

//=============================================================================
// Plugin Commands (MZ)
//=============================================================================

if(Utils.RPGMAKER_NAME === 'MZ'){
    PluginManager.registerCommand(TAA.shw.PluginName, 'clearChoices', args => {
        $gameSystem.clearSmartHelpChoices();
    });

    PluginManager.registerCommand(TAA.shw.PluginName, 'setChoice', args => {
        let index = !isNaN(args.index) ? parseInt(args.index) : undefined;
        let text = args.text;
        $gameSystem.setSmartHelpChoices(index, text);
    });

    PluginManager.registerCommand(TAA.shw.PluginName, 'choiceStatus', args => {
        let state = args.command === 'true';
        if(state) $gameSystem.enableSmartHelpForChoices();
        else $gameSystem.disableSmartHelpForChoices();
    });
}

//=============================================================================
// Window_InputInfo
//=============================================================================

function Window_InputInfo() {
    this.initialize.apply(this, arguments);
};

Window_InputInfo.prototype = Object.create(Window_Base.prototype);
Window_InputInfo.prototype.constructor = Window_InputInfo;

Window_InputInfo.prototype.initialize = function(helpWindow){
    if(Utils.RPGMAKER_NAME === 'MZ')
        Window_Base.prototype.initialize.call(this, new Rectangle(0, 0, Graphics.boxWidth, Graphics.boxHeight));
    else
        Window_Base.prototype.initialize.call(this, 0, 0, Graphics.boxWidth, Graphics.boxHeight);
    this.updateWindowPlacement();
    this._helpWindow = helpWindow;
    this.standardOpacity();
    this.refresh();
    this.deactivate();
};

Window_InputInfo.prototype.updateWindowPlacement = function(){
    var sceneName = SceneManager._scene.constructor.name;
    if(!sceneName) return;
    var params = TAA.shw.Parameters.InputWindow[sceneName];
    if(!params.Enabled) return;
    try{
        var x = eval(params.X);
        var y = eval(params.Y);
        var w = eval(params.Width);
        var h = eval(params.Height);
        if(isNaN(x)) x = Graphics.boxWidth - 320;
        if(isNaN(y)) y = Graphics.boxHeight - 64;
        if(isNaN(w)) w = 320;
        if(isNaN(h)) h = 64;
    } catch(e){
        console.error("TAA_SmartHelpWindows - Failed to parse input info window dimensions. Sticking to default values.");
        var x = Graphics.boxWidth - 320;
        var y = Graphics.boxHeight - 64;
        var w = 320;
        var h = 64;
    }
    this.x = x;
    this.y = y;
    this.width = w;
    this.height = h;
    this._text = params ? params.Text : "";
};

Window_InputInfo.prototype.lineHeight = function(){
    var sceneName = SceneManager._scene.constructor.name;
    return TAA.shw.Parameters.InputWindow[sceneName].LineHeight;
};

Window_InputInfo.prototype.standardFontSize = function(){
    var sceneName = SceneManager._scene.constructor.name;
    return TAA.shw.Parameters.InputWindow[sceneName].FontSize;
};

Window_InputInfo.prototype.standardFontFace = function(){
    var sceneName = SceneManager._scene.constructor.name;
    return TAA.shw.Parameters.InputWindow[sceneName].FontFace;
};

Window_InputInfo.prototype.standardPadding = function(){
    var sceneName = SceneManager._scene.constructor.name;
    return TAA.shw.Parameters.InputWindow[sceneName].StandardPadding;
};

Window_InputInfo.prototype.textPadding = function(){
    var sceneName = SceneManager._scene.constructor.name;
    return TAA.shw.Parameters.InputWindow[sceneName].TextPadding;
};

Window_InputInfo.prototype.standardOpacity = function(){
    var sceneName = SceneManager._scene.constructor.name;
    this.opacity = TAA.shw.Parameters.InputWindow[sceneName].StandardOpacity;
    return this.opacity;
};

Window_InputInfo.prototype.standardBackOpacity = function(){
    var sceneName = SceneManager._scene.constructor.name;
    return TAA.shw.Parameters.InputWindow[sceneName].BackOpacity;
};

Window_InputInfo.prototype.loadWindowskin = function(){
    var sceneName = SceneManager._scene.constructor.name;
    var windowskin = TAA.shw.Parameters.InputWindow[sceneName].WindowSkin;
    this._windowskin = ImageManager.loadSystem(windowskin);
};

Window_InputInfo.prototype.windowWidth = function(){
    return this.width;
};

Window_InputInfo.prototype.windowHeight = function(){
    return this.height;
};

Window_InputInfo.prototype.hide = function(){
    this.visible = false;
};

Window_InputInfo.prototype.show = function(){
    this.visible = true;
};

Window_InputInfo.prototype.refresh = function(){
    this.contents.clear();
    this.drawTextEx(this._text, this.textPadding(), 0);
};

Window_InputInfo.prototype.resetFontSettings = function(){
    this.contents.fontFace = this.standardFontFace();
    this.contents.fontSize = this.standardFontSize();
    this.resetTextColor();
};

//=============================================================================
// Window_Selectable
//=============================================================================

TAA.shw.alias.Window_Selectable = TAA.shw.alias.Window_Selectable || {};
TAA.shw.alias.Window_Selectable.initialize = Window_Selectable.prototype.initialize;
Window_Selectable.prototype.initialize = function(x, y, width, height){
    TAA.shw.alias.Window_Selectable.initialize.call(this, x, y, width, height);
    this._smartHelpWindowEnabled = false;
};

Window_Selectable.prototype.sceneHelpWindow = function(){
    return SceneManager._scene._helpWindow;
};

Window_Selectable.prototype.enableSmartHelpWindow = function(){
    this._smartHelpWindowEnabled = true;
};

Window_Selectable.prototype.disableSmartHelpWindow = function(){
    this._smartHelpWindowEnabled = false;
};

Window_Selectable.prototype.isSmartHelpWindowEnabled = function(){
    return this._smartHelpWindowEnabled;
};

Window_Selectable.prototype.smartHelpUpdate = function(){
    if(!this.isSmartHelpWindowEnabled())
        return;
    this.refresh();
    const rect = this.itemRect(this.index());
    const x = rect.x + this.standardPadding() + this.x;
    const y = rect.y + this.y;
    this.sceneHelpWindow().moveSmartWindow(x, y, rect.width, rect.height, this.lineHeight());
};

Window_Selectable.prototype.isSmartHelpEnabledForScene = function(){
    return $gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name);
};

Window_Selectable.prototype.initSmartHelpWindow = function(){
    if(this.isSmartHelpEnabledForScene()){
        SceneManager._scene.setCurrentActiveWindow(this.constructor.name);
        if($gameSystem.isSmartHelpEnabledForWindow(this.constructor.name)){
            this.updateHelp();
            this.sceneHelpWindow().smartHelpShow();
        }
        else{
            this.sceneHelpWindow().smartHelpHide();
        }
        if(!$gameSystem.isSmartHelpWindowVisible()){
            this.hideHelpWindow();
        }
    }
};

TAA.shw.alias.Window_Selectable.activate = Window_Selectable.prototype.activate;
Window_Selectable.prototype.activate = function(){
    TAA.shw.alias.Window_Selectable.activate.call(this);
    SceneManager._scene.setCurrentActiveWindow(this.constructor.name);
    if(this.sceneHelpWindow() && this.sceneHelpWindow().constructor.name === 'Window_Help' && $gameSystem.isSmartHelpEnabledForWindow(this.constructor.name)){
        if($gameSystem.isSmartHelpEnabledForWindow(this.constructor.name)){
            this.updateHelp();
            this.sceneHelpWindow().smartHelpShow();
        }
        else {
            this.sceneHelpWindow().smartHelpHide();
        }
    }
    else if(SceneManager._scene.isHelpWindowVisible() && $gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        SceneManager._scene.smartHelpHide();
    }
    if($gameSystem.isSmartHelpEnabledForWindow(this.constructor.name)){
        SceneManager._scene.smartInputInfoWindowShow();
    }
    else{
        SceneManager._scene.smartInputInfoWindowHide();
    }
};

TAA.shw.alias.Window_Selectable.updateHelp = Window_Selectable.prototype.updateHelp;
Window_Selectable.prototype.updateHelp = function() {
    if(this._helpWindow){
        TAA.shw.alias.Window_Selectable.updateHelp.call(this);
    }
};

TAA.shw.alias.Window_Selectable.standardPadding = Window_Selectable.prototype.standardPadding;
Window_Selectable.prototype.standardPadding = function(){
    if(TAA.shw.alias.Window_Selectable.standardPadding)
        return TAA.shw.alias.Window_Selectable.standardPadding.call(this);
    else
        return 18;
};

TAA.shw.alias.Window_Selectable.maxCols = Window_Selectable.prototype.maxCols;
Window_Selectable.prototype.maxCols = function() {
    if(this.numCols && this.numCols > 0)
        return this.numCols;
    return TAA.shw.alias.Window_Selectable.maxCols.call(this);
};

//=============================================================================
// Window_Help
//=============================================================================

TAA.shw.alias.Window_Help = TAA.shw.alias.Window_Help || {};
TAA.shw.alias.Window_Help.initialize = Window_Help.prototype.initialize;
if(Utils.RPGMAKER_NAME === 'MV'){
    Window_Help.prototype.initialize = function(numLines){
        TAA.shw.alias.Window_Help.initialize.call(this, numLines);
        this._minWidth = 100;
        this._maxWidth = Graphics.boxWidth;
        this._minHeight = 100;
        this._maxHeight = Graphics.boxHeight;
        this._textState = {};
        this._dynamicVisibility = true;
        this.setStandardOpacity();
    };
}
else{
    Window_Help.prototype.initialize = function(rect){
        TAA.shw.alias.Window_Help.initialize.call(this, rect);
        this._minWidth = 100;
        this._maxWidth = Graphics.boxWidth;
        this._minHeight = 100;
        this._maxHeight = Graphics.boxHeight;
        this._textState = {};
        this._dynamicVisibility = true;
        this.setStandardOpacity();
    };
}

TAA.shw.alias.Window_Help.lineHeight = Window_Help.prototype.lineHeight;
Window_Help.prototype.lineHeight = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.LineHeight;
    }
    else{
        return TAA.shw.alias.Window_Help.lineHeight.call(this);
    }
};

TAA.shw.alias.Window_Help.standardFontSize = Window_Help.prototype.standardFontSize;
Window_Help.prototype.standardFontSize = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.FontSize;
    }
    else{
        return TAA.shw.alias.Window_Help.standardFontSize.call(this);
    }
};

TAA.shw.alias.Window_Help.standardFontFace = Window_Help.prototype.standardFontFace;
Window_Help.prototype.standardFontFace = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.FontFace;
    }
    else{
        return TAA.shw.alias.Window_Help.standardFontFace.call(this);
    }
};

TAA.shw.alias.Window_Help.standardPadding = Window_Help.prototype.standardPadding;
Window_Help.prototype.standardPadding = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.StandardPadding;
    }
    else{
        return TAA.shw.alias.Window_Help.standardPadding.call(this);
    }
};

TAA.shw.alias.Window_Help.textPadding = Window_Help.prototype.textPadding;
Window_Help.prototype.textPadding = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.TextPadding;
    }
    else{
        return TAA.shw.alias.Window_Help.textPadding.call(this);
    }
};

TAA.shw.alias.Window_Help.standardOpacity = Window_Help.prototype.standardOpacity;
Window_Help.prototype.standardOpacity = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.StandardOpacity;
    }
    else{
        return TAA.shw.alias.Window_Help.standardOpacity.call(this);
    }
};

TAA.shw.alias.Window_Help.setStandardOpacity = Window_Help.prototype.setStandardOpacity;
Window_Help.prototype.setStandardOpacity = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        this.opacity = TAA.shw.Parameters.Global.HelpWindow.StandardOpacity;
    }
    else if(TAA.shw.alias.Window_Help.setStandardOpacity){
        TAA.shw.alias.Window_Help.setStandardOpacity.call(this);
    }
};

TAA.shw.alias.Window_Help.standardBackOpacity = Window_Help.prototype.standardBackOpacity;
Window_Help.prototype.standardBackOpacity = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        return TAA.shw.Parameters.Global.HelpWindow.BackOpacity;
    }
    else{
        return TAA.shw.alias.Window_Help.standardBackOpacity.call(this);
    }
};

TAA.shw.alias.Window_Help.loadWindowskin = Window_Help.prototype.loadWindowskin;
Window_Help.prototype.loadWindowskin = function(){
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name)){
        this._windowskin = ImageManager.loadSystem(TAA.shw.Parameters.Global.HelpWindow.WindowSkin);
    }
    else{
        TAA.shw.alias.Window_Help.loadWindowskin.call(this);
    }
};

Window_Help.prototype.windowWidth = function(){
    return this.width;
};

Window_Help.prototype.windowHeight = function(){
    return this.height;
};

Window_Help.prototype.setHideWhenEmpty = function(){
    this._hideWhenEmpty = true;
};

Window_Help.prototype.setShowWhenEmpty = function(){
    this._hideWhenEmpty = false;
};

Window_Help.prototype.calcFullTextHeight = function(textState, all){
    var lastFontSize = this.contents.fontSize;
    var textHeight = 0;
    var lines = textState.text.slice(textState.index).split('\n');
    var maxLines = all ? lines.length : 1;
    var lineCount = 0;
    var canvas = this.contents.canvas.getContext("2d");
    var globalMaxFontSize = this.contents.fontSize;
    var width = this._minWidth;
    var maxAllowedWidth = this._maxWidth - textState.x;

    for (var i = 0; i < maxLines; i++) {
        var maxFontSize = this.contents.fontSize;
        var regExp = /\x1b(?:\{|\}|fs\[[0-9]+\])/g;
        for (;;) {
            var array = regExp.exec(lines[i]);
            if (array) {
                if (array[0] === '\x1b{') {
                    this.makeFontBigger();
                }
                if (array[0] === '\x1b}') {
                    this.makeFontSmaller();
                }
                if (array[0].match(/\x1bfs\[([0-9]+)\]/gi)){
                    this.contents.fontSize = parseInt(RegExp.$1);
                }
                if (maxFontSize < this.contents.fontSize) {
                    maxFontSize = this.contents.fontSize;
                }
            } else {
                break;
            }
        }
        canvas.font = maxFontSize + "px " + this.contents.fontFace;
        var tmpText = lines[i];

        if(maxFontSize > globalMaxFontSize)
            globalMaxFontSize = maxFontSize;
        
        // Had to reverse engineer to find out why VisuMZ Message Core WordWrap was 
        // messing with my functions. A few console.log later, found what seems to be
        // a placeholder for blank spaces. For my height estimative to work, that 
        // needs to go back to its original character
        if(Imported.VisuMZ_1_MessageCore === true)
            tmpText = lines[i].replace(/WrapBreak\[0\]/gi, ' ');

        var textSize = canvas.measureText(tmpText).width + 2 * (this.textPadding() + this.standardPadding());
        if(textSize > maxAllowedWidth && textSize < this._maxWidth){
            maxAllowedWidth = textSize;
        }
        var count = Math.ceil(canvas.measureText(tmpText).width / (maxAllowedWidth - 2 * this.standardPadding()));
        width = Math.max(width, Math.min(maxAllowedWidth, textSize));
        textHeight += ((maxFontSize + 8) * Math.max(count, 1));
        lineCount += Math.max(count, 1);
    }

    this.contents.fontSize = lastFontSize;
    var mh = Math.max(this._maxHeight - textState.y, textState.y);
    textState.estimatedHeight = Math.min(textHeight + 2 * this.standardPadding(), mh);
    textState.estimatedWidth = width;
    textState.lineCount = lineCount;
    textState.maxFontSize = globalMaxFontSize+8;
    return textState;
};

Window_Help.prototype.setupTextState = function(text, x, y){
    var textState = {};
    x = isNaN(x) ? 0 : x;
    y = isNaN(y) ? 0 : y;
    textState = { 
        index: 0,
        x: x,
        y: y,
        left: x };
            
    textState.originalText = text;
    textState.text = this.convertEscapeCharacters(text);
    textState = this.calcFullTextHeight(textState, true);
    
    this.resetFontSettings();
    return textState;
};

TAA.shw.alias.Window_Help.setItem = Window_Help.prototype.setItem;
Window_Help.prototype.setItem = function(item, actorId){
    if(DataManager.isSkill(item) && TAA.skm && !isNaN(actorId) && actorId > 0){
        var level = $gameSystem.getSkillLevel(item.id, actorId);
        this.setText(item ? item.leveledDescription(level) : '');
    }
    else{
        TAA.shw.alias.Window_Help.setItem.call(this, item);
    }
    this._textState = {};
};

Window_Help.prototype.relocateX = function(x, itemWidth){
    if(x > (Graphics.boxWidth / 2)){
        this._maxWidth = x + itemWidth;
        x = 0;
    }
    else{
        this._maxWidth = Graphics.boxWidth;
    }
    return x;
};

Window_Help.prototype.relocateY = function(y, itemHeight, lineHeight){
    var isUp = TAA.shw.Parameters.Global.HelpPosition === 'Above' ? true : false;
    var txtH = this._textState.estimatedHeight;
    var tmpY = y + lineHeight * 1.5;
    if(isUp){
        var tmpY2 = tmpY - itemHeight - this._textState.estimatedHeight;
        if(tmpY2 > 0){
            y = tmpY2;
        }
        else{
            y = tmpY;
        }
    }
    else{
        if(tmpY + txtH < Graphics.boxHeight){
            y = tmpY;
        }
        else{
            y = tmpY - itemHeight - this._textState.estimatedHeight;
        }
    }
    
    return y;
};

Window_Help.prototype.isSmartHelpWindowVisible = function(){
    return this.visible && this._dynamicVisibility;
};

Window_Help.prototype.moveSmartWindow = function(x, y, itemWidth, itemHeight, lineHeight){
    x = this.relocateX(x, itemWidth);
    this._textState = this.setupTextState(this._text, x, y);
    y = this.relocateY(y, itemHeight, lineHeight);
    var width = this._textState.estimatedWidth;
    var height = this._textState.estimatedHeight;
    if((this._maxWidth < Graphics.boxWidth && width < this._maxWidth) || (width + x) > Graphics.boxWidth){
        x = this._maxWidth - width;
    }
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    SceneManager._scene.smartHelpDrawBackgroundRect(this.x, this.y, this.width, this.height);
    this.refresh();
    if((this._textState.text === "" || this._textState.text === undefined) && this.shouldHideSmartHelpWhenEmpty() && this.visible){
        TAA.shw.alias.Window_Help.hide.call(this);
        SceneManager._scene.smartHelpBackgroundHide();
    }
    else if(!this.shouldHideSmartHelpWhenEmpty() && !this.visible && this._dynamicVisibility && $gameSystem.isSmartHelpWindowVisible()){
        TAA.shw.alias.Window_Help.show.call(this);
        SceneManager._scene.smartHelpBackgroundShow();
    }
    SceneManager._scene.smartHelpBackgroundMove(this.x, this.y, this.width, this.height);
};

Window_Help.prototype.smartHelpHide = function(){
    this._dynamicVisibility = false;
    this.updateVisibility();
};

Window_Help.prototype.smartHelpShow = function(){
    this._dynamicVisibility = true;
    this.updateVisibility();
};

Window_Help.prototype.shouldHideSmartHelpWhenEmpty = function(){
    return (this._textState.text === "" || this._textState.text === undefined) && $gameSystem.shouldHideSmartHelpWhenEmpty();
};

Window_Help.prototype.updateVisibility = function(){
    if(this._dynamicVisibility && !this.visible && !this.shouldHideSmartHelpWhenEmpty())
        this.show();
    else if((!this._dynamicVisibility && this.visible) || this.shouldHideSmartHelpWhenEmpty()){
        this.hide();
    }
};

Window_Help.prototype.isSmartHelpEnabledForScene = function(){
    return $gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name);
};

TAA.shw.alias.Window_Help.show = Window_Help.prototype.show;
Window_Help.prototype.show = function(){
    if(this.isSmartHelpEnabledForScene()){
        if(!$gameSystem.isSmartHelpWindowVisible())
            return;
        if(this._dynamicVisibility && !this.shouldHideSmartHelpWhenEmpty()){
            SceneManager._scene.smartHelpBackgroundShow();
            TAA.shw.alias.Window_Help.show.call(this); 
        }
    }
    else{
        SceneManager._scene.smartHelpBackgroundShow();
        TAA.shw.alias.Window_Help.show.call(this);
    }
};

TAA.shw.alias.Window_Help.hide = Window_Help.prototype.hide;
Window_Help.prototype.hide = function(){
    TAA.shw.alias.Window_Help.hide.call(this);
    SceneManager._scene.smartHelpBackgroundHide();
};

TAA.shw.alias.Window_Help.refresh = Window_Help.prototype.refresh;
Window_Help.prototype.refresh = function() {
    if(!this.isSmartHelpEnabledForScene()){
        TAA.shw.alias.Window_Help.refresh.call(this);
    }
    else{
        this.contents.clear();
        this.contents.resize(this._textState.estimatedWidth - 2 * this.standardPadding(), this._textState.estimatedHeight);
        this.drawTextEx(this._text, this.textPadding(), 0);
    }
};

//=============================================================================
// Window_TurnOrderIcon
//=============================================================================

if(Imported.YEP_X_TurnOrderDisplay){
    TAA.shw.alias.Window_TurnOrderIcon = TAA.shw.alias.Window_TurnOrderIcon || {};
    TAA.shw.alias.Window_TurnOrderIcon.destinationY = Window_TurnOrderIcon.prototype.destinationY;
    Window_TurnOrderIcon.prototype.destinationY = function(){
        var valueBefore = Yanfly.Param.TODPositionY;
        var value = TAA.shw.alias.Window_TurnOrderIcon.destinationY.call(this);
        if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name) && this._helpWindow && this._helpWindow.visible){
            value = valueBefore;
            if (this.battler().isSelected()) {
                value -= this._selectHeight;
            }
        }
        return value;
    };
}

//=============================================================================
// Window_BattleSkill
//=============================================================================

TAA.shw.alias.Window_BattleSkill = TAA.shw.alias.Window_BattleSkill || {};
TAA.shw.alias.Window_BattleSkill.updateHelp = Window_BattleSkill.prototype.updateHelp;
Window_BattleSkill.prototype.updateHelp = function(){
    TAA.shw.alias.Window_BattleSkill.updateHelp.call(this);
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_BattleSkill.show = Window_BattleSkill.prototype.show;
Window_BattleSkill.prototype.show = function(){
    TAA.shw.alias.Window_BattleSkill.show.call(this);
};

//=============================================================================
// Window_BattleItem
//=============================================================================

TAA.shw.alias.Window_BattleItem = TAA.shw.alias.Window_BattleItem || {};
TAA.shw.alias.Window_BattleItem.updateHelp = Window_BattleItem.prototype.updateHelp;
Window_BattleItem.prototype.updateHelp = function(){
    TAA.shw.alias.Window_BattleItem.updateHelp.call(this);
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_BattleItem.show = Window_BattleItem.prototype.show;
Window_BattleItem.prototype.show = function(){
    TAA.shw.alias.Window_BattleItem.show.call(this);
};

//=============================================================================
// Window_ItemCategory
//=============================================================================

TAA.shw.alias.Window_ItemCategory = TAA.shw.alias.Window_ItemCategory || {};
TAA.shw.alias.Window_ItemCategory.updateHelp = Window_ItemCategory.prototype.updateHelp;
Window_ItemCategory.prototype.updateHelp = function(){
    TAA.shw.alias.Window_ItemCategory.updateHelp.call(this);
    if(TAA.shw.Parameters.Global.ItemCatHelp.Enabled && this.sceneHelpWindow()){
        var text = {};
        text.description = this.item();
        this.setHelpWindowItem(text);
    }
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_ItemCategory.numVisibleRows = Window_ItemCategory.prototype.numVisibleRows;
Window_ItemCategory.prototype.numVisibleRows = function(){
    if(this.numRows && this.numRows > 0)
        return this.numRows;
    return TAA.shw.alias.Window_ItemCategory.numVisibleRows.call(this);
};

Window_ItemCategory.prototype.item = function(){
    if(!TAA.shw.Parameters.Global.ItemCatHelp.Enabled)
        return "";
    var index = this.index();
    var obj = this._list[index];
    if(!obj || !obj.name || !TAA.shw.Parameters.Global.ItemCatHelp.Texts[obj.name])
        return "";
    return TAA.shw.Parameters.Global.ItemCatHelp.Texts[obj.name];
};

if(Utils.RPGMAKER_NAME === 'MZ'){
    TAA.shw.alias.Window_ItemCategory.windowWidth = Window_ItemCategory.prototype.windowWidth;
    Window_ItemCategory.prototype.windowWidth = function(){
        if($gameSystem.isSmartHelpEnabledForWindow(this.constructor.name)){
            return this.width;
        }
        else {
            return TAA.shw.alias.Window_ItemCategory.windowWidth.call(this);
        }
    };
}

//=============================================================================
// Window_ShopCategory
//=============================================================================

if(Imported.YEP_ShopMenuCore){
    TAA.shw.alias.Window_ShopCategory = TAA.shw.alias.Window_ShopCategory || {};
    TAA.shw.alias.Window_ShopCategory.updateHelp = Window_ShopCategory.prototype.updateHelp;
    Window_ShopCategory.prototype.updateHelp = function(){
        TAA.shw.alias.Window_ShopCategory.updateHelp.call(this);
        if(TAA.shw.Parameters.Global.ItemCatHelp.Enabled && this.sceneHelpWindow()){
            var text = {};
            text.description = this.item();
            this.setHelpWindowItem(text);
        }
        this.smartHelpUpdate();
    };

    TAA.shw.alias.Window_ShopCategory.numVisibleRows = Window_ShopCategory.prototype.numVisibleRows;
    Window_ShopCategory.prototype.numVisibleRows = function(){
        if(this.numRows && this.numRows > 0)
            return this.numRows;
        return TAA.shw.alias.Window_ShopCategory.numVisibleRows.call(this);
    };

    Window_ShopCategory.prototype.item = function(){
        if(!TAA.shw.Parameters.Global.ItemCatHelp.Enabled)
            return "";
        var index = this.index();
        var obj = this._list[index];
        if(!obj || !obj.name || !TAA.shw.Parameters.Global.ItemCatHelp.Texts[obj.name])
            return "";
        return TAA.shw.Parameters.Global.ItemCatHelp.Texts[obj.name];
    };
}
//=============================================================================
// Window_ItemList
//=============================================================================

TAA.shw.alias.Window_ItemList = TAA.shw.alias.Window_ItemList || {};
TAA.shw.alias.Window_ItemList.updateHelp = Window_ItemList.prototype.updateHelp;
Window_ItemList.prototype.updateHelp = function(){
    TAA.shw.alias.Window_ItemList.updateHelp.call(this);
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_ItemList.maxCols = Window_ItemList.prototype.maxCols;
Window_ItemList.prototype.maxCols = function() {
    if(this.numCols && this.numCols > 0)
        return this.numCols;
    return TAA.shw.alias.Window_ItemList.maxCols.call(this);
};

//=============================================================================
// Window_SkillType
//=============================================================================

TAA.shw.alias.Window_SkillType = TAA.shw.alias.Window_SkillType || {};
TAA.shw.alias.Window_SkillType.updateHelp = Window_SkillType.prototype.updateHelp;
Window_SkillType.prototype.updateHelp = function(){
    TAA.shw.alias.Window_SkillType.updateHelp.call(this);
    if(TAA.shw.Parameters.Global.SkillTypeHelp.Enabled && this.sceneHelpWindow()){
        var text = {};
        text.description = this.item();
        this.setHelpWindowItem(text);
    }
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_SkillType.numVisibleRows = Window_SkillType.prototype.numVisibleRows;
Window_SkillType.prototype.numVisibleRows = function(){
    if(this.numRows && this.numRows > 0)
        return this.numRows;
    return TAA.shw.alias.Window_SkillType.numVisibleRows.call(this);
};

Window_SkillType.prototype.item = function(){
    if(!TAA.shw.Parameters.Global.SkillTypeHelp.Enabled)
        return "";
    var index = this.index();
    var obj = this._list[index];
    if(!obj || !obj.name || !TAA.shw.Parameters.Global.SkillTypeHelp.Texts[obj.name])
        return "";
    return TAA.shw.Parameters.Global.SkillTypeHelp.Texts[obj.name];
};

//=============================================================================
// Window_SkillList
//=============================================================================

TAA.shw.alias.Window_SkillList = TAA.shw.alias.Window_SkillList || {};
TAA.shw.alias.Window_SkillList.updateHelp = Window_SkillList.prototype.updateHelp;
Window_SkillList.prototype.updateHelp = function(){
    TAA.shw.alias.Window_SkillList.updateHelp.call(this);
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_SkillList.maxCols = Window_SkillList.prototype.maxCols;
Window_SkillList.prototype.maxCols = function() {
    if(this.numCols && this.numCols > 0)
        return this.numCols;
    return TAA.shw.alias.Window_SkillList.maxCols.call(this);
};

Window_SkillList.prototype.setHelpWindowItem = function(item) {
    if (this._helpWindow) {
        this._helpWindow.setItem(item, this._actor._actorId);
    }
};

//=============================================================================
// Window_EquipStatus
//=============================================================================

TAA.shw.alias.Window_EquipStatus = TAA.shw.alias.Window_EquipStatus || {};
TAA.shw.alias.Window_EquipStatus.windowHeight = Window_EquipStatus.prototype.windowHeight;
Window_EquipStatus.prototype.windowHeight = function() {
    if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name) && this.height)
        return this.height;
    else
        return TAA.shw.alias.Window_EquipStatus.windowHeight.call(this);
};

if(Utils.RPGMAKER_NAME === 'MZ'){
    TAA.shw.alias.Window_EquipStatus.contentsHeight = Window_EquipStatus.prototype.contentsHeight;
    Window_EquipStatus.prototype.contentsHeight = function() {
        if($gameSystem.isSmartHelpEnabledForScene(SceneManager._scene.constructor.name) && this.height)
            return this.height;
        else
            return TAA.shw.alias.Window_EquipStatus.contentsHeight.call(this);
    };
}

//=============================================================================
// Window_EquipCommand
//=============================================================================

TAA.shw.alias.Window_EquipCommand = TAA.shw.alias.Window_EquipCommand || {};
TAA.shw.alias.Window_EquipCommand.updateHelp = Window_EquipCommand.prototype.updateHelp;
Window_EquipCommand.prototype.updateHelp = function(){
    TAA.shw.alias.Window_EquipCommand.updateHelp.call(this);
    if(TAA.shw.Parameters.Global.EquipCommandHelp.Enabled && this.sceneHelpWindow()){
        var text = {};
        text.description = this.item();
        this.setHelpWindowItem(text);
    }
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_EquipCommand.numVisibleRows = Window_EquipCommand.prototype.numVisibleRows;
Window_EquipCommand.prototype.numVisibleRows = function(){
    if(this.numRows && this.numRows > 0)
        return this.numRows;
    return TAA.shw.alias.Window_EquipCommand.numVisibleRows.call(this);
};

TAA.shw.alias.Window_EquipCommand.maxCols = Window_EquipCommand.prototype.maxCols;
Window_EquipCommand.prototype.maxCols = function(){
    if(this.numCols && this.numCols > 0)
        return this.numCols;
    return TAA.shw.alias.Window_EquipCommand.maxCols.call(this);
};

Window_EquipCommand.prototype.item = function(){
    if(!TAA.shw.Parameters.Global.EquipCommandHelp.Enabled)
        return "";
    var index = this.index();
    var obj = this._list[index];
    if(!obj || !obj.name || !TAA.shw.Parameters.Global.EquipCommandHelp.Texts[obj.name])
        return "";
    return TAA.shw.Parameters.Global.EquipCommandHelp.Texts[obj.name];
};

//=============================================================================
// Window_EquipSlot
//=============================================================================

TAA.shw.alias.Window_EquipSlot = TAA.shw.alias.Window_EquipSlot || {};
TAA.shw.alias.Window_EquipSlot.updateHelp = Window_EquipSlot.prototype.updateHelp;
Window_EquipSlot.prototype.updateHelp = function(){
    TAA.shw.alias.Window_EquipSlot.updateHelp.call(this);
    this.smartHelpUpdate();
};

//=============================================================================
// Window_EquipItem
//=============================================================================

TAA.shw.alias.Window_EquipItem = TAA.shw.alias.Window_EquipItem || {};
TAA.shw.alias.Window_EquipItem.updateHelp = Window_EquipItem.prototype.updateHelp;
Window_EquipItem.prototype.updateHelp = function(){
    TAA.shw.alias.Window_EquipItem.updateHelp.call(this);
    this.smartHelpUpdate();
};

//=============================================================================
// Window_ShopCommand
//=============================================================================

TAA.shw.alias.Window_ShopCommand = TAA.shw.alias.Window_ShopCommand || {};
TAA.shw.alias.Window_ShopCommand.maxCols = Window_ShopCommand.prototype.maxCols;
Window_ShopCommand.prototype.maxCols = function(){
    if(this.numCols && this.numCols > 0)
        return this.numCols;
    return TAA.shw.alias.Window_ShopCommand.maxCols.call(this);
};

TAA.shw.alias.Window_ShopCommand.numVisibleRows = Window_ShopCommand.prototype.numVisibleRows;
Window_ShopCommand.prototype.numVisibleRows = function(){
    if(this.numRows && this.numRows > 0)
        return this.numRows;
    return TAA.shw.alias.Window_ShopCommand.numVisibleRows.call(this);
};

//=============================================================================
// Window_ShopBuy
//=============================================================================

TAA.shw.alias.Window_ShopBuy = TAA.shw.alias.Window_ShopBuy || {};
TAA.shw.alias.Window_ShopBuy.updateHelp = Window_ShopBuy.prototype.updateHelp;
Window_ShopBuy.prototype.updateHelp = function(){
    TAA.shw.alias.Window_ShopBuy.updateHelp.call(this);
    this.smartHelpUpdate();
};

//=============================================================================
// Window_ShopSell
//=============================================================================

TAA.shw.alias.Window_ShopSell = TAA.shw.alias.Window_ShopSell || {};
TAA.shw.alias.Window_ShopSell.updateHelp = Window_ShopSell.prototype.updateHelp;
Window_ShopSell.prototype.updateHelp = function(){
    TAA.shw.alias.Window_ShopSell.updateHelp.call(this);
    this.smartHelpUpdate();
};

//=============================================================================
// Window_ShopNumber
//=============================================================================

TAA.shw.alias.Window_ShopNumber = TAA.shw.alias.Window_ShopNumber || {};
TAA.shw.alias.Window_ShopNumber.drawTotalPrice = Window_ShopNumber.prototype.drawTotalPrice;
Window_ShopNumber.prototype.drawTotalPrice = function() {
    if(this._item === null) {
        this.contents.clear();
        return;
    }
    TAA.shw.alias.Window_ShopNumber.drawTotalPrice.call(this);
};

//=============================================================================
// Window_TitleCommand
//=============================================================================

Window_TitleCommand.prototype.updateHelp = function(){
    if(TAA.shw.Parameters.Global.TitleCommandHelp.Enabled && this.sceneHelpWindow()){
        var text = {};
        text.description = this.item();
        this.setHelpWindowItem(text);
    }
    this.smartHelpUpdate();
};

Window_TitleCommand.prototype.item = function(){
    if(!TAA.shw.Parameters.Global.TitleCommandHelp.Enabled)
        return "";
    var index = this.index();
    var obj = this._list[index];
    if(!obj || !obj.name || !TAA.shw.Parameters.Global.TitleCommandHelp.Texts[obj.name])
        return "";
    return TAA.shw.Parameters.Global.TitleCommandHelp.Texts[obj.name];
};

TAA.shw.alias.Window_TitleCommand = TAA.shw.alias.Window_TitleCommand || {};
TAA.shw.alias.Window_TitleCommand.update = Window_TitleCommand.prototype.update;
Window_TitleCommand.prototype.update = function(){
    TAA.shw.alias.Window_TitleCommand.update.call(this);
    this.updateHelp();
};

Window_TitleCommand.prototype.setHelpWindowItem = function(item) {
    if (this.sceneHelpWindow()) {
        this.sceneHelpWindow().setItem(item);
    }
};

//=============================================================================
// Window_MenuCommand
//=============================================================================

Window_MenuCommand.prototype.updateHelp = function(){
    if(TAA.shw.Parameters.Global.MenuCommandHelp.Enabled && this.sceneHelpWindow()){
        var text = {};
        text.description = this.item();
        this.setHelpWindowItem(text);
    }
    this.smartHelpUpdate();
};

Window_MenuCommand.prototype.item = function(){
    if(!TAA.shw.Parameters.Global.MenuCommandHelp.Enabled)
        return "";
    var index = this.index();
    var obj = this._list[index];
    if(!obj || !obj.name || !TAA.shw.Parameters.Global.MenuCommandHelp.Texts[obj.name])
        return "";
    return TAA.shw.Parameters.Global.MenuCommandHelp.Texts[obj.name];
};

TAA.shw.alias.Window_MenuCommand = TAA.shw.alias.Window_MenuCommand || {};
TAA.shw.alias.Window_MenuCommand.update = Window_MenuCommand.prototype.update;
Window_MenuCommand.prototype.update = function(){
    TAA.shw.alias.Window_MenuCommand.update.call(this);
    this.updateHelp();
};

Window_MenuCommand.prototype.setHelpWindowItem = function(item) {
    if (this.sceneHelpWindow()) {
        this.sceneHelpWindow().setItem(item);
    }
};


//=============================================================================
// Window_ChoiceList
//=============================================================================

Window_ChoiceList.prototype.isSmartHelpWindowEnabled = function(){
    return $gameSystem.isSmartHelpEnabledForChoices();
};

Window_ChoiceList.prototype.setHelpWindowReference = function(helpWindow){
    this._helpWindow = helpWindow;
};

Window_ChoiceList.prototype.setHelpWindowItem = function() {
    if (this._helpWindow) {
        let item = {};
        let index = this.index();
        if(Utils.RPGMAKER_NAME === 'MZ')
            index = this.index() + 1;
        item.description = $gameSystem.getSmartHelpChoiceText(index);
        this._helpWindow.setItem(item);
    }
};

TAA.shw.alias.Window_ChoiceList = TAA.shw.alias.Window_ChoiceList || {};
TAA.shw.alias.Window_ChoiceList.updateHelp = Window_ChoiceList.prototype.updateHelp;
Window_ChoiceList.prototype.updateHelp = function(){
    TAA.shw.alias.Window_ChoiceList.updateHelp.call(this);
    this.setHelpWindowItem();
    this.smartHelpUpdate();
};

TAA.shw.alias.Window_ChoiceList.start = Window_ChoiceList.prototype.start;
Window_ChoiceList.prototype.start = function(){
    TAA.shw.alias.Window_ChoiceList.start.call(this);
    SceneManager._scene.updateToggleShwButton();
};

TAA.shw.alias.Window_ChoiceList.callOkHandler = Window_ChoiceList.prototype.callOkHandler;
Window_ChoiceList.prototype.callOkHandler = function() {
    TAA.shw.alias.Window_ChoiceList.callOkHandler.call(this);
    SceneManager._scene.setCurrentActiveWindow("Window_Gold");
    this.updateHelp();
    SceneManager._scene.updateToggleShwButton();
};

TAA.shw.alias.Window_ChoiceList.callCancelHandler = Window_ChoiceList.prototype.callCancelHandler;
Window_ChoiceList.prototype.callCancelHandler = function() {
    TAA.shw.alias.Window_ChoiceList.callCancelHandler.call(this);
    SceneManager._scene.setCurrentActiveWindow("Window_Gold");
    this.updateHelp();
    SceneManager._scene.updateToggleShwButton();
};

//=============================================================================
// Scene_Base
//=============================================================================

TAA.shw.alias.Scene_Base = TAA.shw.alias.Scene_Base || {};
TAA.shw.alias.Scene_Base.initialize = Scene_Base.prototype.initialize;
Scene_Base.prototype.initialize = function() {
    TAA.shw.alias.Scene_Base.initialize.call(this);
    var inputParams = TAA.shw.Parameters.InputWindow[this.constructor.name];
    this._currentActiveWindow = undefined;
    this._helpWindowBackgroundMode = TAA.shw.Parameters.Global.HelpWindow.BackgroundMode;
    this._helpWindowRect = undefined;
    this._helpWindowBg = undefined;
    this._inputInfoWindowBackgroundMode = inputParams ? inputParams.BackgroundMode : 2;
    this._inputInfoWindowRect = undefined;
    this._inputInfoWindow = undefined;
    this._inputInfoWindowEnabled = inputParams ? inputParams.Enabled === true : false;
};

Scene_Base.prototype.isSmartWindowEnabledForActiveWindow = function(){
    if(!this._currentActiveWindow)
        return false;
    return $gameSystem.isSmartHelpEnabledForWindow(this._currentActiveWindow);
};

Scene_Base.prototype.setCurrentActiveWindow = function(windowName){
    this._currentActiveWindow = windowName;
};

Scene_Base.prototype.toggleSmartHelpVisibility = function(){
    if(this._helpWindow){
        if($gameSystem.isSmartHelpWindowVisible() && this.isSmartWindowEnabledForActiveWindow()){
            $gameSystem.smartHelpWindowHide();
            this._helpWindow.hide();
        }
        else if(this.isSmartWindowEnabledForActiveWindow()){
            $gameSystem.smartHelpWindowMakeVisible();
            this._helpWindow.show();
        }
    }
};

Scene_Base.prototype.isInputInfoWindowAlwaysVisible = function(){
    var scName = this.constructor.name;
    var alwaysVisible = TAA.shw.Parameters.InputWindow[scName] && TAA.shw.Parameters.InputWindow[scName].AlwaysVisible === true;
    return alwaysVisible;
};

TAA.shw.alias.Scene_Base.update = Scene_Base.prototype.update;
Scene_Base.prototype.update = function() {
    TAA.shw.alias.Scene_Base.update.call(this);
    if($gameSystem.isSmartHelpEnabledForWindow(this._currentActiveWindow)){
        this.smartInputInfoWindowShow();
        if($gameSystem.isSmartHelpWindowVisible())
            this.smartHelpShow();
        else
            this.smartHelpHide();
    }
    else {
        if(this.isInputInfoWindowAlwaysVisible())
            this.smartInputInfoWindowShow();
        else
            this.smartInputInfoWindowHide();
        this.smartHelpHide();
    }
};

Scene_Base.prototype.updateSmartHelp = function(){
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        if(Input.isTriggered($gameSystem.getSmartHelpWindowKey()))
            this.toggleSmartHelpVisibility();
    }
};

Scene_Base.prototype.initSmartHelpWindow = function(){
    if(this._helpWindow && $gameSystem.isSmartHelpEnabledForScene(this.constructor.name)){
        if(!$gameSystem.isSmartHelpWindowVisible() || !$gameSystem.isSmartHelpEnabledForWindow(this._currentActiveWindow)){
            this._helpWindow.hide();
        }
        if(this.isInputInfoWindowAlwaysVisible()){
            this.smartInputInfoWindowHide();
        }
        this.createToggleShwButton();
    }
};

Scene_Base.prototype.isHelpWindowVisible = function(){
    return this._helpWindow && this._helpWindow.visible;
};

Scene_Base.prototype.smartHelpHide = function(){
    if(this._helpWindow && $gameSystem.isSmartHelpEnabledForScene(this.constructor.name)){
        this._helpWindow.smartHelpHide();
    }
};

Scene_Base.prototype.smartHelpShow = function(){
    if(this._helpWindow && $gameSystem.isSmartHelpEnabledForScene(this.constructor.name))
        this._helpWindow.smartHelpShow();
};

Scene_Base.prototype.setShwWindowPosition = function(className, windowRef, windowObj){
    var obj = TAA.shw.Parameters.Custom[className];
    if(!obj || !obj['Custom Window Positioning'])
        return;
    var newX = windowObj.x;
    var newY = windowObj.y;
    var newWidth = windowObj.width;
    var newHeight = windowObj.height;
    try{
        newX = eval(obj[windowRef + ' X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - " + windowRef + " update to X position failed (" + obj[windowRef + ' X'] + ").");
        newX = windowObj.x;
    }
    try{
        newY = eval(obj[windowRef + ' Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - " + windowRef + " update to Y position failed (" + obj[windowRef + ' Y'] + ").");
        newY = windowObj.y;
    }
    try{
        newWidth = eval(obj[windowRef + ' Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - " + windowRef + " update to width failed (" + obj[windowRef + ' Width'] + ").");
        newWidth = windowObj.width;
    }
    try{
        newHeight = eval(obj[windowRef + ' Height']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - " + windowRef + " update to height failed (" + obj[windowRef + ' Height'] + ").");
        newHeight = windowObj.height;
    }
    windowObj.x = newX;
    windowObj.y = newY;
    windowObj.width = newWidth;
    windowObj.height = newHeight;
    if(!isNaN(obj[windowRef + ' Cols']))
        windowObj.numCols = parseInt(obj[windowRef + " Cols"]);
    if(windowObj.refresh) windowObj.refresh();
    if(Utils.RPGMAKER_NAME === 'MZ' && windowObj.refreshCursor) windowObj.refreshCursor();
    else if(Utils.RPGMAKER_NAME === 'MV' && windowObj.updateCursor) windowObj.updateCursor();
};

Scene_Base.prototype.smartHelpBackgroundEnabled = function(){
    return ($gameSystem.isSmartHelpEnabledForScene(this.constructor.name)) && this._helpWindowBackgroundMode > 0;
};

Scene_Base.prototype.smartInputInfoBackgroundEnabled = function(){
    return ($gameSystem.isSmartHelpEnabledForScene(this.constructor.name)) && this._inputInfoWindowBackgroundMode > 0;
};

Scene_Base.prototype.smartHelpDrawBackgroundRect = function(x, y, width, height){
    if(!this.smartHelpBackgroundEnabled() || ![1, 2].contains(this._helpWindowBackgroundMode)){
        return;
    }
    this._helpWindowRect.clear();
    this._helpWindowRect.beginFill(TAA.shw.Parameters.Global.HelpWindow.RectangleColor, TAA.shw.Parameters.Global.HelpWindow.RectangleAlpha);
    if(this._helpWindowBackgroundMode === 2)
        this._helpWindowRect.drawRoundedRect(x, y, width, height);
    else
        this._helpWindowRect.drawRect(x, y, width, height);
    this._helpWindowRect.endFill();
};

Scene_Base.prototype.smartHelpInitBackgroundRect = function(){
    if(this.smartHelpBackgroundEnabled() && [1, 2].contains(this._helpWindowBackgroundMode)){
        this._helpWindowRect = new PIXI.Graphics();
        this._helpWindowRect.visible = $gameSystem.isSmartHelpWindowVisible();
        var index = this.children.indexOf(this._helpWindow);
        if(index >= 0) this.addChildAt(this._helpWindowRect, index);
        else{
            console.error("TAA_SmartHelpWindows - Init Rect Background error! Help window child not found!");
        }
    }
};

Scene_Base.prototype.smartHelpInitBackgroundImage = function(){
    if(this.smartHelpBackgroundEnabled() && this._helpWindowBackgroundMode === 3){
        this._helpWindowBg = new TilingSprite();
        this._helpWindowBg.bitmap = SmartHelpCache.getBackgroundImg();
        this._helpWindowBg.visible = $gameSystem.isSmartHelpWindowVisible();
        var index = this.children.indexOf(this._helpWindow);
        if(index >= 0) this.addChildAt(this._helpWindowBg, index);
        else{
            console.error("TAA_SmartHelpWindows - Init Rect Background error! Help window child not found!");
        }
    }
};

Scene_Base.prototype.smartInputInfoWindowCreate = function(){
    if(this._inputInfoWindowEnabled){
        this._inputInfoWindow = new Window_InputInfo(this._helpWindow);
        this._inputInfoWindow.updateWindowPlacement();
        var index = this.children.indexOf(this._helpWindow);
        if(isNaN(index) || index < 0) return;
        this.addChildAt(this._inputInfoWindow, ++index);
        this.smartInputInfoBackgroundRect();
    }
};

Scene_Base.prototype.smartInputInfoBackgroundRect = function(){
    if(this.smartInputInfoBackgroundEnabled()){
        this._inputInfoWindowRect = new PIXI.Graphics();
        this._inputInfoWindowRect.visible = $gameSystem.isSmartHelpEnabledForScene(this.constructor.name);
        var index = this.children.indexOf(this._inputInfoWindow);
        if(isNaN(index) || index < 0) return;
        var params = TAA.shw.Parameters.InputWindow[this.constructor.name];
        this._inputInfoWindowRect.clear();
        this._inputInfoWindowRect.beginFill(params.RectangleColor, params.RectangleAlpha);
        if(this._inputInfoWindowBackgroundMode === 2)
            this._inputInfoWindowRect.drawRoundedRect(this._inputInfoWindow.x, this._inputInfoWindow.y, this._inputInfoWindow.width, this._inputInfoWindow.height);
        else
            this._inputInfoWindowRect.drawRect(this._inputInfoWindow.x, this._inputInfoWindow.y, this._inputInfoWindow.width, this._inputInfoWindow.height);
        this._inputInfoWindowRect.endFill();
        this.addChildAt(this._inputInfoWindowRect, index);
    }
};

Scene_Base.prototype.smartHelpBackgroundHide = function(){
    if(this.smartHelpBackgroundEnabled() && this._helpWindowRect)
        this._helpWindowRect.visible = false;
    else if(this.smartHelpBackgroundEnabled() && this._helpWindowBg)
        this._helpWindowBg.visible = false;
};

Scene_Base.prototype.smartHelpBackgroundShow = function(){
    if(this.smartHelpBackgroundEnabled() && this._helpWindowRect)
        this._helpWindowRect.visible = true;
    else if(this.smartHelpBackgroundEnabled() && this._helpWindowBg){
        this._helpWindowBg.visible = true;
    }
};

Scene_Base.prototype.smartHelpBackgroundMove = function(x, y, w, h){
    if(!this._helpWindowBg) return;
    this._helpWindowBg.move(x, y, this._helpWindowBg.bitmap.width, this._helpWindowBg.bitmap.height);
    let scaleX = w / this._helpWindowBg.bitmap.width;
    let scaleY = h / this._helpWindowBg.bitmap.height;
    this._helpWindowBg.setTransform(x, y, scaleX, scaleY);
};

Scene_Base.prototype.smartInputInfoWindowHide = function(){
    if(this._inputInfoWindow)
        this._inputInfoWindow.visible = false;
    if(this._inputInfoWindowRect)
        this._inputInfoWindowRect.visible = false;
};

Scene_Base.prototype.smartInputInfoWindowShow = function(){
    if(this._inputInfoWindow)
        this._inputInfoWindow.visible = true;
    if(this._inputInfoWindowRect)
        this._inputInfoWindowRect.visible = true;
};

Scene_Base.prototype.createToggleShwButton = function(){
    if(TAA.shw.Parameters.ToggleButton.Enable){
        if(Utils.RPGMAKER_NAME === 'MV')
            this._toggleShwButton = new Sprite_CustomButton('toggleHelp');
        else if(Utils.RPGMAKER_NAME === 'MZ')
            this._toggleShwButton = new Sprite_Button('toggleHelp');
        this._toggleShwButton.x = TAA.shw.functions.parseCoordinates(TAA.shw.Parameters.ToggleButton.X);
        this._toggleShwButton.y = TAA.shw.functions.parseCoordinates(TAA.shw.Parameters.ToggleButton.Y);
        this._toggleShwButton.setClickHandler(this.toggleSmartHelpVisibility.bind(this));
        this.updateToggleShwButton();
        this.addChild(this._toggleShwButton);
    }
};

Scene_Base.prototype.updateToggleShwButton = function() {
    if (this._toggleShwButton) {
        this._toggleShwButton.visible = this.isSmartWindowEnabledForActiveWindow();
    }
};

//=============================================================================
// Scene_Battle
//=============================================================================

TAA.shw.alias.Scene_Battle = TAA.shw.alias.Scene_Battle || {};
TAA.shw.alias.Scene_Battle.createAllWindows = Scene_Battle.prototype.createAllWindows;
Scene_Battle.prototype.createAllWindows = function(){
    TAA.shw.alias.Scene_Battle.createAllWindows.call(this);
    if($gameSystem.isSmartHelpEnabledForScene(this.constructor.name)){
        this.removeChild(this._helpWindow);
        var index = this.children.indexOf(this._windowLayer);
        if(isNaN(index)) index = 0;
        else index++;
        this.addChildAt(this._helpWindow, index);
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this.setShwCustomPositions();
        this._skillWindow.enableSmartHelpWindow();
        this._itemWindow.enableSmartHelpWindow();
    }
};

TAA.shw.alias.Scene_Battle.update = Scene_Battle.prototype.update;
Scene_Battle.prototype.update = function(){
    TAA.shw.alias.Scene_Battle.update.call(this);
    this.updateSmartHelp();
    this.updateToggleShwButton();
};

Scene_Battle.prototype.setShwCustomPositions = function(){
    if(TAA.shw.Parameters.Custom['Scene_Battle'] && TAA.shw.Parameters.Custom['Scene_Battle']['Custom Window Positioning']){
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Battle', 'Skill', this._skillWindow);
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Battle', 'Item', this._itemWindow);
    }
};

if(Utils.RPGMAKER_NAME === 'MZ'){
    TAA.shw.alias.Scene_Battle.createCancelButton = Scene_Battle.prototype.createCancelButton;
    Scene_Battle.prototype.createCancelButton = function() {
        TAA.shw.alias.Scene_Battle.createCancelButton.call(this);
        if(TAA.shw.Parameters.Custom['Scene_Battle'] && TAA.shw.Parameters.Custom['Scene_Battle']['Custom Window Positioning']){
            if(Utils.RPGMAKER_NAME === 'MZ'){
                try{
                    var y = eval(TAA.shw.Parameters.Custom['Scene_Battle']['Cancel Button Y']);
                } catch(e){
                    console.error("TAA_SmartHelpWindows - Failed to parse cancel button Y position (" + TAA.shw.Parameters.Custom['Scene_Battle']['Cancel Button Y'] + ")");
                    var y = 0;
                }
                this._cancelButton.y = y;
            }
        }
    };
}

if(Imported.MOG_BattleHud){
    TAA.shw.alias.Scene_Battle.onSkillCancel = Scene_Battle.prototype.onSkillCancel;
    Scene_Battle.prototype.onSkillCancel = function(){
        TAA.shw.alias.Scene_Battle.onSkillCancel.call(this);
        this._currentActiveWindow = 'Window_ChoiceList';
    };

    TAA.shw.alias.Scene_Battle.onItemCancel = Scene_Battle.prototype.onItemCancel;
    Scene_Battle.prototype.onItemCancel = function(){
        TAA.shw.alias.Scene_Battle.onItemCancel.call(this);
        this._currentActiveWindow = 'Window_ChoiceList';
    };
}

//=============================================================================
// Scene_Item
//=============================================================================

TAA.shw.alias.Scene_Item = TAA.shw.alias.Scene_Item || {};
TAA.shw.alias.Scene_Item.create = Scene_Item.prototype.create;
Scene_Item.prototype.create = function(){
    TAA.shw.alias.Scene_Item.create.call(this);
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        this.removeChild(this._helpWindow);
        var index = this.children.indexOf(this._windowLayer);
        if(isNaN(index)) index = 0;
        else index++;
        this.addChildAt(this._helpWindow, index);
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this._categoryWindow.enableSmartHelpWindow();
        this._itemWindow.enableSmartHelpWindow();
        this.initSmartHelpWindow();
        this.setShwCustomPositions();
        this._categoryWindow.initSmartHelpWindow();
        this._categoryWindow.smartHelpUpdate();
    }
};

TAA.shw.alias.Scene_Item.update = Scene_Item.prototype.update;
Scene_Item.prototype.update = function(){
    TAA.shw.alias.Scene_Item.update.call(this);
    this.updateSmartHelp();
    this.updateToggleShwButton();
};

Scene_Item.prototype.setShwCategoryPosition = function(){
    var obj = TAA.shw.Parameters.Custom['Scene_Item'];
    var newX = this._categoryWindow.x;
    var newY = this._categoryWindow.y;
    var newWidth = this._categoryWindow.width;
    var numRows = isNaN(obj['Item Category Rows']) ? 1 : parseInt(obj['Item Category Rows']);
    try{
        newX = eval(obj['Item Category X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - ItemCategory update to X position failed (" + obj['Item Category X'] + ").");
        newX = this._categoryWindow.x;
    }
    try{
        newY = eval(obj['Item Category Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - ItemCategory update to Y position failed (" + obj['Item Category Y'] + ").");
        newY = this._categoryWindow.y;
    }
    try{
        newWidth = eval(obj['Item Category Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - ItemCategory update to width failed (" + obj['Item Category Width'] + ").");
        newWidth = this._categoryWindow.width;
    }
    this._categoryWindow.x = newX;
    this._categoryWindow.y = newY;
    this._categoryWindow.width = newWidth;
    this._categoryWindow.numRows = numRows;
    this._categoryWindow.refresh();
    if(Utils.RPGMAKER_NAME === 'MZ' && this._categoryWindow.refreshCursor) this._categoryWindow.refreshCursor();
    else if(Utils.RPGMAKER_NAME === 'MV' && this._categoryWindow.updateCursor) this._categoryWindow.updateCursor();
};

Scene_Item.prototype.setShwCustomPositions = function(){
    if(TAA.shw.Parameters.Custom['Scene_Item'] && TAA.shw.Parameters.Custom['Scene_Item']['Custom Window Positioning']){
        this.setShwCategoryPosition();
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Item', 'Item List', this._itemWindow);
        if(Imported.YEP_ItemCore && Yanfly.Param.ItemSceneItem){
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Item', 'Status', this._statusWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Item', 'Info', this._infoWindow);
        }
    }
};

//=============================================================================
// Scene_Skill
//=============================================================================

TAA.shw.alias.Scene_Skill = TAA.shw.alias.Scene_Skill || {};
TAA.shw.alias.Scene_Skill.create = Scene_Skill.prototype.create;
Scene_Skill.prototype.create = function(){
    TAA.shw.alias.Scene_Skill.create.call(this);
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        this.removeChild(this._helpWindow);
        var index = this.children.indexOf(this._windowLayer);
        if(isNaN(index)) index = 0;
        else index++;
        this.addChildAt(this._helpWindow, index);
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this._skillTypeWindow.enableSmartHelpWindow();
        this._itemWindow.enableSmartHelpWindow();
        this.initSmartHelpWindow();
        this.setShwCustomPositions();
        this._skillTypeWindow.initSmartHelpWindow();
        this._skillTypeWindow.smartHelpUpdate();
    }
};

TAA.shw.alias.Scene_Skill.update = Scene_Skill.prototype.update;
Scene_Skill.prototype.update = function(){
    TAA.shw.alias.Scene_Skill.update.call(this);
    this.updateSmartHelp();
    this.updateToggleShwButton();
};

Scene_Skill.prototype.setShwTypePosition = function(){
    var obj = TAA.shw.Parameters.Custom['Scene_Skill'];
    var newX = this._skillTypeWindow.x;
    var newY = this._skillTypeWindow.y;
    var newWidth = this._skillTypeWindow.width;
    var numRows = isNaN(obj['Skill Type Rows']) ? 1 : parseInt(obj['Skill Type Rows']);
    try{
        newX = eval(obj['Skill Type X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - SkillType update to X position failed (" + obj['Skill Type X'] + ").");
        newX = this._skillTypeWindow.x;
    }
    try{
        newY = eval(obj['Skill Type Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - SkillType update to Y position failed (" + obj['Skill Type Y'] + ").");
        newY = this._skillTypeWindow.y;
    }
    try{
        newWidth = eval(obj['Skill Type Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - SkillType update to width failed (" + obj['Skill Type Width'] + ").");
        newWidth = this._skillTypeWindow.width;
    }
    this._skillTypeWindow.x = newX;
    this._skillTypeWindow.y = newY;
    this._skillTypeWindow.width = newWidth;
    this._skillTypeWindow.numRows = numRows;
    this._skillTypeWindow.refresh();
    if(Utils.RPGMAKER_NAME === 'MZ' && this._skillTypeWindow.refreshCursor) this._skillTypeWindow.refreshCursor();
    else if(Utils.RPGMAKER_NAME === 'MV' && this._skillTypeWindow.updateCursor) this._skillTypeWindow.updateCursor();
};

Scene_Skill.prototype.setShwCustomPositions = function(){
    if(TAA.shw.Parameters.Custom['Scene_Skill'] && TAA.shw.Parameters.Custom['Scene_Skill']['Custom Window Positioning']){
        this.setShwTypePosition();
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Skill', 'Skill Status', this._statusWindow);
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Skill', 'Skill List', this._itemWindow);
    }
};

//=============================================================================
// Scene_Equip
//=============================================================================

TAA.shw.alias.Scene_Equip = TAA.shw.alias.Scene_Equip || {};
TAA.shw.alias.Scene_Equip.create = Scene_Equip.prototype.create;
Scene_Equip.prototype.create = function(){
    TAA.shw.alias.Scene_Equip.create.call(this);
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        this.removeChild(this._helpWindow);
        var index = this.children.indexOf(this._windowLayer);
        if(isNaN(index)) index = 0;
        else index++;
        this.addChildAt(this._helpWindow, index);
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this._commandWindow.enableSmartHelpWindow();
        this._slotWindow.enableSmartHelpWindow();
        this._itemWindow.enableSmartHelpWindow();
        this.initSmartHelpWindow();
        this.setShwCustomPositions();
        this._commandWindow.initSmartHelpWindow();
        this._commandWindow.smartHelpUpdate();
    }
};

TAA.shw.alias.Scene_Equip.update = Scene_Equip.prototype.update;
Scene_Equip.prototype.update = function(){
    this.updateSmartHelp();
    this.updateToggleShwButton();
    TAA.shw.alias.Scene_Equip.update.call(this);
};

Scene_Equip.prototype.setShwCommandPosition = function(){
    var obj = TAA.shw.Parameters.Custom['Scene_Equip'];
    var newX = this._commandWindow.x;
    var newY = this._commandWindow.y;
    var newWidth = this._commandWindow.width;
    var numCols = isNaN(obj['Command Cols']) ? Imported.YEP_EquipCore ? 1 : 3 : parseInt(obj['Command Cols']);
    var numRows = isNaN(obj['Command Rows']) ? Imported.YEP_EquipCore ? 4 : 1 : parseInt(obj['Command Rows']);
    try{
        newX = eval(obj['Command X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - CommandWindow update to X position failed (" + obj['Command X'] + ").");
        newX = this._commandWindow.x;
    }
    try{
        newY = eval(obj['Command Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - CommandWindow update to Y position failed (" + obj['Command Y'] + ").");
        newY = this._commandWindow.y;
    }
    try{
        newWidth = eval(obj['Command Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - CommandWindow update to width failed (" + obj['Command Width'] + ").");
        newWidth = this._commandWindow.width;
    }
    this._commandWindow.x = newX;
    this._commandWindow.y = newY;
    this._commandWindow.width = newWidth;
    this._commandWindow.numRows = numRows;
    this._commandWindow.numCols = numCols;
    this._commandWindow.refresh();
    if(Utils.RPGMAKER_NAME === 'MZ' && this._commandWindow.refreshCursor) this._commandWindow.refreshCursor();
    else if(Utils.RPGMAKER_NAME === 'MV' && this._commandWindow.updateCursor) this._commandWindow.updateCursor();
};

Scene_Equip.prototype.setShwCustomPositions = function(){
    if(TAA.shw.Parameters.Custom['Scene_Equip'] && TAA.shw.Parameters.Custom['Scene_Equip']['Custom Window Positioning']){
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Equip', 'Status', this._statusWindow);
        this.setShwCommandPosition();
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Equip', 'Slot', this._slotWindow);
        Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Equip', 'Item', this._itemWindow);
        if(Imported.YEP_EquipCore && this._compareWindow){
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Equip', 'Status Compare', this._compareWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Equip', 'Info', this._infoWindow);
        }
    }
};

//=============================================================================
// Scene_Shop
//=============================================================================

TAA.shw.alias.Scene_Shop = TAA.shw.alias.Scene_Shop || {};
TAA.shw.alias.Scene_Shop.create = Scene_Shop.prototype.create;
Scene_Shop.prototype.create = function(){
    TAA.shw.alias.Scene_Shop.create.call(this);
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        this.removeChild(this._helpWindow);
        var index = this.children.indexOf(this._windowLayer);
        if(isNaN(index)) index = 0;
        else index++;
        this.addChildAt(this._helpWindow, index);
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this._buyWindow.enableSmartHelpWindow();
        this._sellWindow.enableSmartHelpWindow();
        this._categoryWindow.enableSmartHelpWindow();
        this.setCurrentActiveWindow(this._commandWindow.constructor.name);
        this.initSmartHelpWindow();
        this.setShwCustomPositions();
    }
};

TAA.shw.alias.Scene_Shop.update = Scene_Shop.prototype.update;
Scene_Shop.prototype.update = function(){
    TAA.shw.alias.Scene_Shop.update.call(this);
    this.updateSmartHelp();
    this.updateToggleShwButton();
};

TAA.shw.alias.Scene_Shop.onSellCancel = Scene_Shop.prototype.onSellCancel;
Scene_Shop.prototype.onSellCancel = function() {
    TAA.shw.alias.Scene_Shop.onSellCancel.call(this);
    if(['Window_ItemCategory', 'Window_ShopCategory'].contains(this._currentActiveWindow)){
        this._categoryWindow.updateHelp();
    }
};

Scene_Shop.prototype.setShwGoldPosition = function(){
    var obj = TAA.shw.Parameters.Custom['Scene_Shop'];
    var newX = this._goldWindow.x;
    var newY = this._goldWindow.y;
    var newWidth = this._goldWindow.width;
    try{
        newX = eval(obj['Gold X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - GoldWindow update to X position failed (" + obj['Gold X'] + ").");
        newX = this._goldWindow.x;
    }
    try{
        newY = eval(obj['Gold Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - GoldWindow update to Y position failed (" + obj['Gold Y'] + ").");
        newY = this._goldWindow.y;
    }
    try{
        newWidth = eval(obj['Gold Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - GoldWindow update to width failed (" + obj['Gold Width'] + ").");
        newWidth = this._goldWindow.width;
    }
    this._goldWindow.x = newX;
    this._goldWindow.y = newY;
    this._goldWindow.width = newWidth;
};

Scene_Shop.prototype.setShwCommandPosition = function(){
    var obj = TAA.shw.Parameters.Custom['Scene_Shop'];
    var newX = this._commandWindow.x;
    var newY = this._commandWindow.y;
    var newWidth = this._commandWindow.width;
    var numRows = isNaN(obj['Command Rows']) ? Imported.YEP_ShopMenuCore ? 4 : 1 : parseInt(obj['Command Rows']);
    var numCols = isNaN(obj['Command Cols']) ? Imported.YEP_ShopMenuCore ? 1 : 3 : parseInt(obj['Command Cols']);
    try{
        newX = eval(obj['Command X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - CommandWindow update to X position failed (" + obj['Command X'] + ").");
        newX = this._commandWindow.x;
    }
    try{
        newY = eval(obj['Command Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - CommandWindow update to Y position failed (" + obj['Command Y'] + ").");
        newY = this._commandWindow.y;
    }
    try{
        newWidth = eval(obj['Command Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - CommandWindow update to width failed (" + obj['Command Width'] + ").");
        newWidth = this._commandWindow.width;
    }
    this._commandWindow.x = newX;
    this._commandWindow.y = newY;
    this._commandWindow.width = newWidth;
    this._commandWindow.numRows = numRows;
    this._commandWindow.numCols = numCols;
    this._commandWindow.refresh();
    if(Utils.RPGMAKER_NAME === 'MZ' && this._commandWindow.refreshCursor) this._commandWindow.refreshCursor();
    else if(Utils.RPGMAKER_NAME === 'MV' && this._commandWindow.updateCursor) this._commandWindow.updateCursor();
};

Scene_Shop.prototype.setShwCategoryPosition = function(){
    var obj = TAA.shw.Parameters.Custom['Scene_Shop'];
    var newX = this._categoryWindow.x;
    var newY = this._categoryWindow.y;
    var newWidth = this._categoryWindow.width;
    var numRows = isNaN(obj['Category Rows']) ? Imported.YEP_ShopMenuCore ? 4 : 1 : parseInt(obj['Category Rows']);
    var numCols = isNaN(obj['Category Cols']) ? Imported.YEP_ShopMenuCore ? 1 : 4 : parseInt(obj['Category Cols']);
    try{
        newX = eval(obj['Category X']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - ItemCategory update to X position failed (" + obj['Category X'] + ").");
        newX = this._categoryWindow.x;
    }
    try{
        newY = eval(obj['Category Y']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - ItemCategory update to Y position failed (" + obj['Category Y'] + ").");
        newY = this._categoryWindow.y;
    }
    try{
        newWidth = eval(obj['Category Width']);
    } catch(e){
        console.error("TAA_SmartHelpWindows - ItemCategory update to width failed (" + obj['Category Width'] + ").");
        newWidth = this._categoryWindow.width;
    }
    this._categoryWindow.x = newX;
    this._categoryWindow.y = newY;
    this._categoryWindow.width = newWidth;
    this._categoryWindow.numCols = numCols;
    this._categoryWindow.numRows = numRows;
    this._categoryWindow.refresh();
    if(Utils.RPGMAKER_NAME === 'MZ' && this._categoryWindow.refreshCursor) this._categoryWindow.refreshCursor();
    else if(Utils.RPGMAKER_NAME === 'MV' && this._categoryWindow.updateCursor) this._categoryWindow.updateCursor();
};

Scene_Shop.prototype.setShwCustomPositions = function(){
    if(TAA.shw.Parameters.Custom['Scene_Shop'] && TAA.shw.Parameters.Custom['Scene_Shop']['Custom Window Positioning']){
        if(Imported.YEP_ShopMenuCore){
            this.setShwCommandPosition();
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Info', this._infoWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Dummy', this._dummyWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Number', this._numberWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Buy', this._buyWindow);
            this.setShwCategoryPosition();
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Sell', this._sellWindow);
            this.setShwGoldPosition();
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Status', this._statusWindow);
        }
        else{
            this.setShwGoldPosition();
            this.setShwCommandPosition();
            this.setShwCategoryPosition();
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Dummy', this._dummyWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Number', this._numberWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Status', this._statusWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Buy', this._buyWindow);
            Scene_Base.prototype.setShwWindowPosition.call(this, 'Scene_Shop', 'Sell', this._sellWindow);
        }
    }
};

//=============================================================================
// Scene_Map
//=============================================================================

TAA.shw.alias.Scene_Map = TAA.shw.alias.Scene_Map || {};
TAA.shw.alias.Scene_Map.createAllWindows = Scene_Map.prototype.createAllWindows;
Scene_Map.prototype.createAllWindows = function(){
    TAA.shw.alias.Scene_Map.createAllWindows.call(this);
    this.createHelpWindow();
    this._helpWindow.visible = false;
    this.smartHelpInitBackgroundRect();
    this.smartHelpInitBackgroundImage();
    this.smartInputInfoWindowCreate();
    this.initSmartHelpWindow();
    this._currentActiveWindow = "Window_Gold";
};

Scene_Map.prototype.createHelpWindow = function() {
    if(Utils.RPGMAKER_NAME === 'MV'){
        this._helpWindow = new Window_Help();
        this._choiceWindow = this._messageWindow._choiceWindow;
    }
    else{
        const rect = this.smartHelpWindowRect();
        this._helpWindow = new Window_Help(rect);
        this._choiceWindow = this._choiceListWindow;
    }
    this.addChild(this._helpWindow);
    this._choiceWindow.setHelpWindowReference(this._helpWindow);
};

TAA.shw.alias.Scene_Map.update = Scene_Map.prototype.update;
Scene_Map.prototype.update = function(){
    TAA.shw.alias.Scene_Map.update.call(this);
    this.updateSmartHelp();
    this.updateToggleShwButton();
};

Scene_Map.prototype.smartHelpWindowRect = function() {
    const wx = 0;
    const wy = 0;
    const ww = 100;
    const wh = 100;
    return new Rectangle(wx, wy, ww, wh);
};

if(Imported.YEP_CreditsPage && TAA.mts){
    TAA.shw.alias.Scene_Map.createCreditsWindow = Scene_Map.prototype.createCreditsWindow;
    Scene_Map.prototype.createCreditsWindow = function() {
        TAA.shw.alias.Scene_Map.createCreditsWindow.call(this);
        if(SceneManager._scene.constructor.name === 'Scene_Title')
            SceneManager._scene._commandWindow.activate();
    };
}

//=============================================================================
// Scene_Title
//=============================================================================

TAA.shw.alias.Scene_Title = TAA.shw.alias.Scene_Title || {};
TAA.shw.alias.Scene_Title.createCommandWindow = Scene_Title.prototype.createCommandWindow;
Scene_Title.prototype.createCommandWindow = function() {
    TAA.shw.alias.Scene_Title.createCommandWindow.call(this);
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        this.createHelpWindow();
        this._helpWindow.visible = false;
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this._commandWindow.enableSmartHelpWindow();
        this.initSmartHelpWindow();
        this._commandWindow.initSmartHelpWindow();
        this._commandWindow.smartHelpUpdate();
    }
};

Scene_Title.prototype.createHelpWindow = function() {
    if(Utils.RPGMAKER_NAME === 'MV')
        this._helpWindow = new Window_Help();
    else{
        const rect = this.smartHelpWindowRect();
        this._helpWindow = new Window_Help(rect);
    }
    this.addChild(this._helpWindow);
};

TAA.shw.alias.Scene_Title.update = Scene_Title.prototype.update;
Scene_Title.prototype.update = function(){
    this.updateSmartHelp();
    this.updateToggleShwButton();
    TAA.shw.alias.Scene_Title.update.call(this);
};

Scene_Title.prototype.smartHelpWindowRect = function() {
    const wx = 0;
    const wy = 0;
    const ww = 100;
    const wh = 100;
    return new Rectangle(wx, wy, ww, wh);
};

if(Imported.YEP_CreditsPage){
    TAA.shw.alias.Scene_Title.createCreditsWindow = Scene_Title.prototype.createCreditsWindow;
    Scene_Title.prototype.createCreditsWindow = function() {
        TAA.shw.alias.Scene_Title.createCreditsWindow.call(this);
        this._commandWindow.activate();
    };
}

//=============================================================================
// Scene_Menu
//=============================================================================

TAA.shw.alias.Scene_Menu = TAA.shw.alias.Scene_Menu || {};
TAA.shw.alias.Scene_Menu.createCommandWindow = Scene_Menu.prototype.createCommandWindow;
Scene_Menu.prototype.createCommandWindow = function() {
    TAA.shw.alias.Scene_Menu.createCommandWindow.call(this);
    if($gameSystem.isSmartHelpEnabledForScene([this.constructor.name])){
        this.createHelpWindow();
        this._helpWindow.visible = false;
        this.smartHelpInitBackgroundRect();
        this.smartHelpInitBackgroundImage();
        this.smartInputInfoWindowCreate();
        this._commandWindow.enableSmartHelpWindow();
        this.initSmartHelpWindow();
        this._commandWindow.initSmartHelpWindow();
        this._commandWindow.smartHelpUpdate();
    }
};

Scene_Menu.prototype.createHelpWindow = function() {
    if(Utils.RPGMAKER_NAME === 'MV')
        this._helpWindow = new Window_Help();
    else {
        const rect = this.helpWindowRect();
        this._helpWindow = new Window_Help(rect);
    }
    this.addChild(this._helpWindow);
};

TAA.shw.alias.Scene_Menu.update = Scene_Menu.prototype.update;
Scene_Menu.prototype.update = function(){
    this.updateSmartHelp();
    this.updateToggleShwButton();
    TAA.shw.alias.Scene_Menu.update.call(this);
};

//=============================================================================
// Scene_Boot
//=============================================================================

TAA.shw.alias.Scene_Boot = TAA.shw.alias.Scene_Boot || {};
if(Utils.RPGMAKER_NAME === 'MZ'){
    TAA.shw.alias.Scene_Boot.loadSystemImages = Scene_Boot.prototype.loadSystemImages;
    Scene_Boot.prototype.loadSystemImages = function(){
        TAA.shw.alias.Scene_Boot.loadSystemImages.call(this);
        this.preloadSmartHelpSkins();
    };
};

if(Utils.RPGMAKER_NAME === 'MV'){
    TAA.shw.alias.Scene_Boot.loadSystemWindowImage = Scene_Boot.prototype.loadSystemWindowImage;
    Scene_Boot.prototype.loadSystemWindowImage = function() {
        TAA.shw.alias.Scene_Boot.loadSystemWindowImage.call(this);
        this.preloadSmartHelpSkins();
    };
}

Scene_Boot.prototype.preloadSmartHelpSkins = function(){
    var loaded = [];
    var keys = Object.keys(TAA.shw.Parameters.InputWindow);
    for(var i=0; i<keys.length; i++){
        var obj = TAA.shw.Parameters.InputWindow[keys[i]];
        var skin = obj.WindowSkin;
        if(skin && skin !== "" && !loaded.contains(skin)){
            if(Utils.RPGMAKER_NAME === 'MZ'){
                ImageManager.loadBitmap('img/system/' + skin, undefined, false);
            }
            else {
                ImageManager.reserveSystem(skin);
            }
            loaded.push(skin);
        }
    }
    var skin = TAA.shw.Parameters.Global.HelpWindow.WindowSkin;
    if(skin && skin !== "" && !loaded.contains(skin)){
        if(Utils.RPGMAKER_NAME === 'MZ'){
            ImageManager.loadBitmap('img/system/' + skin, undefined, false);
        }
        else{
            ImageManager.reserveSystem(skin);
        }
    }
};