﻿//=============================================================================
// WindowBackImage.js
// ----------------------------------------------------------------------------
// (C)2017 Triacontane
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 2.0.2 2020/10/15 指定可能なウィンドウに戦闘画面のステータスウィンドウを追加
// 2.0.1 2020/08/22 カスタムメニュープラグインで作成したウィンドウ背景を変えられる機能を追加
// 2.0.0 2020/08/13 MZ対応版作成
// 1.3.0 2019/01/13 ウィンドウ背景の画像を複数表示できる機能を追加
//                  ウィンドウ背景を指定した場合も元のウィンドウフレームを表示したままにできる機能を追加
// 1.2.0 2018/11/29 ウィンドウ背景を有効にするかどうかを動的に制御するスイッチを追加
// 1.1.0 2017/11/19 拡大率を設定できる機能を追加
// 1.0.0 2017/11/18 初版
// ----------------------------------------------------------------------------
// [Blog]   : https://triacontane.blogspot.jp/
// [Twitter]: https://twitter.com/triacontane/
// [GitHub] : https://github.com/triacontane/
//=============================================================================

/*:
 * @plugindesc ウィンドウ背景画像指定プラグイン
 * @target MZ
 * @url https://github.com/triacontane/RPGMakerMV/tree/mz_master/WindowBackImage.js
 * @author トリアコンタン
 *
 * @param windowImageInfo
 * @text ウィンドウ画像情報
 * @desc 背景画像を差し替えるウィンドウの情報です。
 * @default
 * @type struct<WindowImages>[]
 *
 * @help WindowBackImage.js
 *
 * ウィンドウの背景を任意の画像に置き換えます。
 * 画像は複数指定可能で、それぞれに出現条件スイッチを指定できます。
 * 画像が表示された場合、元のウィンドウフレームを非表示するかどうかを
 * 選択できます。
 *
 * 背景画像はウィンドウのサイズにかかわらず、中央を原点に表示されます。
 * 拡大率と座標を補正することは可能ですがサイズが可変、不定のウィンドウに
 * 対して背景画像を指定することは推奨しません。
 *
 * プラグインによって追加されたウィンドウにも指定可能ですが
 * 正常に動作するとは限りません。
 *
 * SceneCustomMenu.jsで追加したウィンドウの背景を変えたい場合は
 * WindowClassの指定を同プラグインの『ウィンドウ識別子』を指定してください。
 *
 * このプラグインにはプラグインコマンドはありません。
 *
 * 利用規約：
 *  作者に無断で改変、再配布が可能で、利用形態（商用、18禁利用等）
 *  についても制限はありません。
 *  このプラグインはもうあなたのものです。
 */

/*~struct~WindowImages:
 *
 * @param WindowClass
 * @desc 専用の画像に差し替える対象のウィンドウです。一覧にない場合は直接入力してください。
 * @type select
 * @default
 * @option [general game] help window
  * @value Window_Help
  * @option [general game] money window
  * @value Window_Gold
  * @option [main menu] main command window
  * @value Window_MenuCommand
  * @option [main menu] actor status window
  * @value Window_MenuStatus
  * @option [item screen] item category window
  * @value Window_ItemCategory
  * @option [item screen] item list window
  * @value Window_ItemList
  * @option [item screen] actor selection window
  * @value Window_MenuActor
  * @option [skill screen] skill type window
  * @value Window_SkillType
  * @option [skill screen] status window
  * @value Window_SkillStatus
  * @option [skill screen] skill list window
  * @value Window_SkillList
  * @option [equipment screen] status window
  * @value Window_EquipStatus
  * @option [equipment screen] equipment command window
  * @value Window_EquipCommand
  * @option [Equipment screen] Equipment slot window
  * @value Window_EquipSlot
  * @option [Equipment screen] Equipment list window
  * @value Window_EquipItem
  * @option [status screen] status window
  * @value Window_Status
  * @option [option screen] option window
  * @value Window_Options
  * @option [save, load screen] file list window
  * @value Window_SavefileList
  * @option [shop screen] shop command window
  * @value Window_ShopCommand
  * @option [shop screen] purchase item window
  * @value Window_ShopBuy
  * @option [Shop Screen] Sale Item Window
  * @value Window_ShopSell
  * @option [shop screen] number input window
  * @value Window_ShopNumber
  * @option [shop screen] status window
  * @value Window_ShopStatus
  * @option [name input screen] name window
  * @value Window_NameEdit
  * @option [name input screen] name input window
  * @value Window_NameInput
  * @option [map screen] option window
  * @value Window_ChoiceList
  * @option [Map Screen] Value Input Window
  * @value Window_NumberInput
  * @option [map screen] item selection window
  * @value Window_EventItem
  * @option [map screen] message window
  * @value Window_Message
  * @option [map screen] scroll message window
  * @value Window_ScrollText
  * @option [map screen] map name window
  * @value Window_MapName
  * @option [battle screen] battle log window
  * @value Window_BattleLog
  * @option [battle screen] party command window
  * @value Window_PartyCommand
  * @option [battle screen] actor command window
  * @value Window_ActorCommand
  * @option [battle screen] battler status window
  * @value Window_BattleStatus
  * @option [battle screen] actor list window
  * @value Window_BattleActor
  * @option [Battle Screen] Enemy Character List Window
  * @value Window_BattleEnemy
  * @option [Battle screen] Skill list window
  * @value Window_BattleSkill
  * @option [battle screen] item list window
  * @value Window_BattleItem
  * @option [title screen] title window
  * @value Window_TitleCommand
  * @option [game end screen] end confirmation window
  * @value Window_GameEnd
  * @option [debug screen] variable selection window
  * @value Window_DebugRange
  * @option [debug screen] variable setting window
  * @value Window_DebugEdit
  * @option [action goal window plugin] action goal window
  * @value Window_Destination
  * @option [action goal window plugin] menu action goal window
  * @value Window_DestinationMenu
  * @option [in-game time introduction plugin] time window
  * @value Window_Chronus
  * @option [Official Gacha Plugin] Gacha Display Window
  * @value Window_Gacha
  * @option [official gacha plugin] command window
  * @value Window_GachaCommand
  * @option [Official Gacha Plugin] Acquisition Confirmation Window
  * @value Window_GachaGetCommand
  * @option [Official Gacha Plugin] Acquisition Information Window
  * @value Window_GachaGet
  * @option [Official Gacha Plugin] Cost Window
  * @value Window_Cost
  * @option [Novel game general plug-in] Novel option window
  * @value Window_NovelChoiceList
  * @option [Novel game general plug-in] Novel message window
  * @value Window_NovelMessage
  * @option [Novel game general plug-in] Novel title command window
  * @value Window_NovelTitleCommand
  * @option [Novel game general plug-in] Novel value input window
  * @value Window_NovelNumberInput
  * @option [Novel game general plug-in] pause menu window
  * @value Window_PauseMenu
  * @option [cross save plugin] password entry window
  * @value Window_PasswordInput
  * @option [cross save plugin] password window
  * @value Window_PasswordEdit
  * @option [Terminology Plugin] Term Category Window
  * @value Window_GlossaryCategory
  * @option [Terminology Plugin] Term list window
  * @value Window_GlossaryList
  * @option [Glossary Plugin] confirmation window
  * @value Window_GlossaryConfirm
  * @option [term dictionary plugin] collection rate window
  * @value Window_GlossaryComplete
  * @option [term dictionary plugin] term window
  * @value Window_Glossary
  * @option [sound test plugin] audio category window
  * @value Window_AudioCategory
  * @option [sound test plugin] audio list window
  * @value Window_AudioList
  * @option [sound test plugin] audio settings window
  * @value Window_AudioSetting
  * @option [numeric input screen plug-in] numeric input window
  * @value Window_NumberInput
  * @option [numeric input screen plug-in] numeric window
  * @value Window_NumberEdit
  * @option [face] face message
  * @value Window_FaceBox
  * @option [credits] credits
  * @value CGMZ_Window_Credits
  * @option [playtime] numeric window
  * @value Window_Gold.prototype
  * @option [playtime] numeric window
  * @value Window_PartyCommand_initialize
  * @option [playtime] numeric window
  * @value Window_Base
 *
 * @param ImageFile
 * @desc 差し替える画像のファイル名です。(img/pictureの中から選択します)　空を指定すると枠だけが非表示になります。
 * @default
 * @require 1
 * @dir img/pictures/
 * @type file
 *
 * @param OffsetX
 * @desc 表示X座標の補正値です。
 * @default 0
 * @type number
 * @min -2000
 * @max 2000
 *
 * @param OffsetY
 * @desc 表示Y座標の補正値です。
 * @default 0
 * @type number
 * @min -2000
 * @max 2000
 *
 * @param ScaleX
 * @desc X方向の拡大率(%指定)です。
 * @default 100
 * @type number
 * @min -2000
 * @max 2000
 *
 * @param ScaleY
 * @desc Y方向の拡大率(%指定)です。
 * @default 100
 * @type number
 * @min -2000
 * @max 2000
 *
 * @param WindowShow
 * @desc 画像が表示されているときでもウィンドウの元背景を表示したままにします。
 * @default false
 * @type boolean
 *
 * @param SwitchId
 * @desc 指定したスイッチがONのときのみウィンドウを差し替えます。
 * @default 0
 * @type switch
 */

(function() {
    'use strict';

    var getClassName = function(object) {
        return object.constructor.toString().replace(/function\s+(.*)\s*\([\s\S]*/m, '$1');
    };

    //=============================================================================
    // ローカル関数
    //  プラグインパラメータやプラグインコマンドパラメータの整形やチェックをします
    //=============================================================================
    /**
     * Create plugin parameter. param[paramName] ex. param.commandPrefix
     * @param pluginName plugin name(EncounterSwitchConditions)
     * @returns {Object} Created parameter
     */
    var createPluginParameter = function(pluginName) {
        var paramReplacer = function(key, value) {
            if (value === 'null') {
                return value;
            }
            if (value[0] === '"' && value[value.length - 1] === '"') {
                return value;
            }
            try {
                return JSON.parse(value);
            } catch (e) {
                return value;
            }
        };
        var parameter     = JSON.parse(JSON.stringify(PluginManager.parameters(pluginName), paramReplacer));
        PluginManager.setParameters(pluginName, parameter);
        return parameter;
    };

    var param = createPluginParameter('WindowBackImage');
    if (!param.windowImageInfo) {
        param.windowImageInfo = [];
    }

    //=============================================================================
    // Window
    //  専用の背景画像を設定します。
    //=============================================================================
    var _Window__createAllParts      = Window.prototype._createAllParts;
    Window.prototype._createAllParts = function() {
        _Window__createAllParts.apply(this, arguments);
        this._backImageDataList = this.initBackImageData();
        if (this._backImageDataList.length >= 0) {
            this._createBackImage();
        }
    };

    Window.prototype._setBackImageProperty = function(backImageData) {
        this._backImageDx        = parseInt(backImageData['OffsetX']) || 0;
        this._backImageDy        = parseInt(backImageData['OffsetY']) || 0;
        this._backSprite.scale.x = (parseInt(backImageData['ScaleX']) || 100) / 100;
        this._backSprite.scale.y = (parseInt(backImageData['ScaleY']) || 100) / 100;
    };

    /**
     * 背景画像を作成します。
     * @private
     */
    Window.prototype._createBackImage = function() {
        this._backSprite.visible  = false;
        this._frameSprite.visible = false;
        this.frameVisible = false;
        this._windowBackImageSprites    = [];
        this._backImageDataList.forEach(function(backImageData) {
            var bitmap     = ImageManager.loadPicture(backImageData['ImageFile']);
            var sprite     = new Sprite_WindowBackImage(bitmap);
            sprite.scale.x = (backImageData['ScaleX'] || 100) / 100;
            sprite.scale.y = (backImageData['ScaleY'] || 100) / 100;
            this._windowBackImageSprites.push(sprite);
            this._container.addChild(sprite);
        }, this);
    };

    Window.prototype.initBackImageData = function() {
        var className = getClassName(this);
        // for SceneCustomMenu.js
        if (this._data && this._data.Id) {
            className = this._data.Id;
        }
        return param.windowImageInfo.filter(function(data) {
            return data['WindowClass'] === className;
        }, this);
    };

    Window.prototype.getBackImageDataItem = function(index, propName) {
        return this._backImageDataList[index][propName];
    };

    var _Window__refreshAllParts      = Window.prototype._refreshAllParts;
    Window.prototype._refreshAllParts = function() {
        if (this._windowBackImageSprites) {
            this._refreshBackImage();
        }
        _Window__refreshAllParts.apply(this, arguments);
    };

    /**
     * 背景画像をリフレッシュします。
     * @private
     */
    Window.prototype._refreshBackImage = function() {
        this._windowBackImageSprites.forEach(function(sprite, index) {
            sprite.x = this.width / 2 + this.getBackImageDataItem(index, 'OffsetX');
            sprite.y = this.height / 2 + this.getBackImageDataItem(index, 'OffsetY');
        }, this);
    };

    var _Window_update      = Window.prototype.update;
    Window.prototype.update = function() {
        _Window_update.apply(this, arguments);
        if (!this._windowBackImageSprites) {
            return;
        }
        var defaultVisible = true;
        this._windowBackImageSprites.forEach(function(sprite, index) {
            var switchId = this.getBackImageDataItem(index, 'SwitchId');
            sprite.visible = !switchId || $gameSwitches.value(switchId);
            if (sprite.visible && !this.getBackImageDataItem(index, 'WindowShow')) {
                defaultVisible = false;
            }
        }, this);
        this._backSprite.visible  = defaultVisible;
        this._frameSprite.visible = defaultVisible;
        this.frameVisible = defaultVisible;
    };

    //=============================================================================
    // Sprite_WindowBackImage
    //  ウィンドウ背景画像のスプライトです。
    //=============================================================================
    function Sprite_WindowBackImage() {
        this.initialize.apply(this, arguments);
    }

    Sprite_WindowBackImage.prototype             = Object.create(Sprite.prototype);
    Sprite_WindowBackImage.prototype.constructor = Sprite_WindowBackImage;

    Sprite_WindowBackImage.prototype.initialize = function(bitmap) {
        Sprite.prototype.initialize.call(this);
        this.bitmap   = bitmap;
        this.anchor.x = 0.5;
        this.anchor.y = 0.5;
    };
})();

