﻿// XPStyle.js Ver.1.3.0
// MIT License (C) 2022 あわやまたな
// http://opensource.org/licenses/mit-license.php

/*:
* @target MZ
* @plugindesc Battle UI in the style of RPG Maker XP.
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/
* @help Ver.1.3.0
*【Cursor】
* If the parameter [Arrow Image] is not specified, it works with
* the conventional cursor specifications.
* If specified, it behaves like RPG Maker XP.
* Arrow images are created br specified
* in the parameter [Number of Frames] in the horizontal directy
* arranging the numbeion.
* If you set the number of frames to 1, you can create an arrow that
* does not animate.
*
* 【Note】
* <battlerFace:Filename,Index>
* By entering this in the enemy character's note in the database,
* you can display the enemy's face image in the battler help window.
* Create images in the face folder based on the same criteria as face images.
* Input example）<battlerFace:Monster,7>
*
* 【Supplement】
* Combining this with the launch plugin SimpleMsgSideViewMZ makes
* it more like that.
*
* @param partyCommand
* @text Party Command
* @desc Party command window settings.
* @type struct<partyCommand>
* @default {"windowPosition":"top","partyCommandMaxWidth":"0","partyCommandMinWidth":"0","itemTextAlign":"center"}
*
* @param actorCommand
* @text Actor Command
* @desc Actor command window settings.
* @type struct<actorCommand>
* @default {"windowPosition":"movable","actorCommandMaxWidth":"0","actorCommandMinWidth":"0","itemTextAlign":"center"}
*
* @param battleStatus
* @text Battle Status
* @desc Battle status command window settings.
* @type struct<battleStatus>
* @default {"backOpacity":"-1"}
*
* @param battleEnemy
* @text Battle Enemy
* @desc Battle enemy window settings.
* @type struct<battleEnemy>
* @default {"windowVisible":"true","backOpacity":"-1"}
*
* @param battlerHelp
* @text Battler Help
* @desc Battler Help command window settings.
* Shows the status of the selected battler.
* @type struct<battlerHelp>
* @default {"windowPosition":"top","actorHelp":"true","battlerFace":"false","enemyHelp":"true","enemyGauges":"{\"hp\":\"false\",\"mp\":\"false\",\"tp\":\"false\",\"time\":\"false\"}","enemyStateIcon":"false","outlineWidth":"3"}
*
* @param battleSkill
* @text Battle Skill/Item
* @desc Skill/item command window settings.
* @type struct<battleSkill>
* @default {"windowPosition":"center","backOpacity":"-1","helpLines":"2"}
*
* @param cursor
* @text Cursor
* @desc Cursor settings.
* @type struct<cursor>
* @default {"arrowImage":"","originY":"0","numberOfFrames":"2","speed":"8"}
*
*/

/*~struct~partyCommand:
*
* @param windowPosition
* @text Window Position
* @desc Specifies the position of the party command window.
* @type select
* @default top
* @option left
* @option upper-left
* @option center
* @option right
* @option upper-right
* @option top
* @option bottom
*
* @param partyCommandMaxWidth
* @text Maximum Width
* @desc Specifies the maximum width of the window.
* Disabled with 0.
* @type number
* @default 0
*
* @param partyCommandMinWidth
* @text Minimum Width
* @desc Specifies the minimum width of the window.
* Disabled with 0.
* @type number
* @default 0
*
* @param itemTextAlign
* @text Item Text Align
* @desc String origin.
* @type select
* @default center
* @option left
* @option center
* @option right
*
*/

/*~struct~actorCommand:
*
* @param windowPosition
* @text Window Position
* @desc Specifies the position of the window.
* @type select
* @default movable
* @option movable
* @option left
* @option upper-left
* @option right
* @option upper-right
* @option top
* @option bottom
*
* @param actorCommandMaxWidth
* @text Maximum Width
* @desc Specifies the maximum width of the window.
* Disabled with 0.
* @type number
* @default 0
*
* @param actorCommandMinWidth
* @text Minimum Width
* @desc Specifies the minimum width of the window.
* Disabled with 0.
* @type number
* @default 0
*
* @param itemTextAlign
* @text Item Text Align
* @desc String origin.
* @type select
* @default center
* @option left
* @option center
* @option right
*
*/

/*~struct~cursor:
*
* @param arrowImage
* @text Arrow Image
* @desc Sets the image of an arrow pointing at an actor or enemy.
* @type file
* @dir img/system
*
* @param originY
* @text Origin Y
* @desc Specifies the origin of the arrow image.
* @type select
* @default 0
* @option Top
* @option 0
* @option Center
* @option 0.5
*
* @param numberOfFrames
* @text Number of Frames
* @desc Specify the number of frames for the arrow image.
* @type number
* @default 2
* @min 1
*
* @param speed
* @text Speed
* @desc The number of frames before transitioning to the next frame.
* @type number
* @default 8
* @min 1
*
*/

/*~struct~battleStatus:
*
* @param backOpacity
* @text Opacity
* @desc Specifies opacity.
* Disabled with -1.
* @type number
* @default -1
* @max 255
* @min -1
*
*/

/*~struct~battleEnemy:
*
* @param windowVisible
* @text Window Visible
* @desc true：Touch-friendly like MZ.
* false：Reproduce the specifications of XP.
* @type boolean
* @default true
*
* @param backOpacity
* @text Opacity
* @desc Specifies opacity.
* Disabled with -1.
* @type number
* @default -1
* @max 255
* @min -1
*
*/

/*~struct~battlerHelp:
*
* @param windowPosition
* @text Window Position
* @desc Specifies the position of the window.
* @type select
* @default top
* @option top
* @option bottom
*
* @param actorHelp
* @text Actor Help
* @desc Display help for the actor.
* @type boolean
* @default true
*
* @param battlerFace
* @text Battler Face
* @desc Displays the battler's face image.
* @type boolean
* @default false
*
* @param enemyHelp
* @text Enemy Help
* @desc Displays help for the enemy.
* @type boolean
* @default true
*
* @param enemyGauges
* @text Enemy Gauges
* @desc Select the enemy gauge to display.
* @type struct<enemyGauges>
* @default 
*
* @param enemyStateIcon
* @text Enemy State Icon
* @desc Show state icons.
* @type boolean
* @default false
*
* @param outlineWidth
* @text Outline Width
* @desc Width of text border.
* @type number
* @default 3
*
*/

/*~struct~enemyGauges:
*
* @param hp
* @desc HP is open to the public from the beginning.
* @type boolean
* @default false
*
* @param mp
* @desc MP is open to the public from the beginning.
* @type boolean
* @default false
*
* @param tp
* @desc TP is open to the public from the beginning.
* @type boolean
* @default false
*
* @param time
* @desc Time is open to the public from the beginning.
* @type boolean
* @default false
*
*/

/*~struct~battleSkill:
*
* @param windowPosition
* @text Window Position
* @desc Specifies the position of the window.
* @type select
* @default center
* @option center
* @option bottom (same as MZ)
* @value bottom
* @option bottom with help window
* @value bottom_bottom
*
* @param backOpacity
* @text Opacity
* @desc Specifies opacity.
* Disabled with -1.
* @type number
* @default -1
* @max 255
* @min -1
*
* @param helpLines
* @text Help Window Number of Lines
* @desc Specifies the number of lines in the help window.
* @type number
* @default 2
*
*/
/*:ja
* @target MZ
* @plugindesc 画面配置をRPGツクールXP風にできます。
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/
* @help 【カーソル】
* パラメータ[矢印画像]が未指定の場合、従来のカーソルの仕様で動作します。
* 指定した場合はツクールXP風の挙動になります。
* 矢印画像はパラメータ[コマ数]で指定した数だけ横方向に並べて作成します。
* コマ数を1にするとアニメーションをしない矢印を作れます。
*
* 【メモ】
* <battlerFace:ファイル名,インデックス>
* をデータベースの敵キャラのメモに入力する事で、バトラーヘルプウィンドウに
* エネミーの顔画像を表示可能になります。
* 画像はfacesフォルダに、顔画像と同じ規格で作成して下さい。
* 入力例）<battlerFace:Monster,7>
*
* 【補足】
* ロンチプラグインのSimpleMsgSideViewMZと組み合わせるとより
* それっぽくなります。
*
* [更新履歴]
* 2022/03/28：Ver.1.0.0　公開
* 2022/03/31：Ver.1.1.0　ウィンドウの設定を追加。
* 2022/04/05：Ver.1.2.0　パラメータ内容を追加。エネミーが変身した際、バトラーヘルプの整列がおかしくなる不具合を修正。
* 2022/09/22：Ver.1.3.0　ヘルプウィンドウにキャラクターの顔画像が表示されない場合がある不具合を修正。
*
* @param partyCommand
* @text パーティコマンド
* @desc パーティコマンドウィンドウの設定です。
* @type struct<partyCommand>
* @default {"windowPosition":"top","partyCommandMaxWidth":"0","partyCommandMinWidth":"0","itemTextAlign":"center"}
*
* @param actorCommand
* @text アクターコマンド
* @desc アクターコマンドウィンドウの設定です。
* @type struct<actorCommand>
* @default {"windowPosition":"movable","actorCommandMaxWidth":"0","actorCommandMinWidth":"0","itemTextAlign":"center"}
*
* @param battleStatus
* @text バトルステータス
* @desc バトルステータスウィンドウの設定です。
* @type struct<battleStatus>
* @default {"backOpacity":"-1"}
*
* @param battleEnemy
* @text バトルエネミー
* @desc エネミーウィンドウの設定です。
* @type struct<battleEnemy>
* @default {"windowVisible":"true","backOpacity":"-1"}
*
* @param battlerHelp
* @text バトラーヘルプ
* @desc バトラーヘルプウィンドウの設定です。
* 選択したバトラーのステータスを表示します。
* @type struct<battlerHelp>
* @default {"windowPosition":"top","actorHelp":"true","battlerFace":"false","enemyHelp":"true","enemyGauges":"{\"hp\":\"false\",\"mp\":\"false\",\"tp\":\"false\",\"time\":\"false\"}","enemyStateIcon":"false","outlineWidth":"3"}
*
* @param battleSkill
* @text バトルスキル・アイテム
* @desc スキル、アイテムウィンドウの設定です。
* @type struct<battleSkill>
* @default {"windowPosition":"center","backOpacity":"-1","helpLines":"2"}
*
* @param cursor
* @text カーソル
* @desc カーソルの設定です。
* @type struct<cursor>
* @default {"arrowImage":"","originY":"0","numberOfFrames":"2","speed":"8"}
*
*/

/*~struct~partyCommand:ja
*
* @param windowPosition
* @text ウィンドウ位置
* @desc パーティコマンドウィンドウの位置を指定します。
* @type select
* @default top
* @option 左
* @value left
* @option 左上
* @value upper-left
* @option 中央
* @value center
* @option 右
* @value right
* @option 右上
* @value upper-right
* @option 上
* @value top
* @option 下
* @value bottom
*
* @param partyCommandMaxWidth
* @text パーティコマンド最大幅
* @desc パーティコマンドウィンドウの最大幅を指定します。
* 0で無効。
* @type number
* @default 0
*
* @param partyCommandMinWidth
* @text パーティコマンド最低幅
* @desc パーティコマンドウィンドウの最低幅を指定します。
* 0で無効。
* @type number
* @default 0
*
* @param itemTextAlign
* @text 要素整列
* @desc 文字列の原点
* @type select
* @default center
* @option 左
* @value left
* @option 中央
* @value center
* @option 右
* @value right
*
*/

/*~struct~actorCommand:ja
*
* @param windowPosition
* @text ウィンドウ位置
* @desc アクターコマンドウィンドウの位置を指定します。
* @type select
* @default movable
* @option 可動
* @value movable
* @option 左
* @value left
* @option 左上
* @value upper-left
* @option 右
* @value right
* @option 右上
* @value upper-right
* @option 上
* @value top
* @option 下
* @value bottom
*
* @param actorCommandMaxWidth
* @text アクターコマンド最大幅
* @desc アクターコマンドウィンドウの最大幅を指定します。
* 0で無効。
* @type number
* @default 0
*
* @param actorCommandMinWidth
* @text アクターコマンド最低幅
* @desc アクターコマンドウィンドウの最低幅を指定します。
* 0で無効。
* @type number
* @default 0
*
* @param itemTextAlign
* @text 要素整列
* @desc 文字列の原点
* @type select
* @default center
* @option 左
* @value left
* @option 中央
* @value center
* @option 右
* @value right
*
*/

/*~struct~cursor:ja
*
* @param arrowImage
* @text 矢印画像
* @desc アクターやエネミーを指し示す矢印の画像を設定します。
* @type file
* @dir img/system
*
* @param originY
* @text 原点Y
* @desc 矢印画像の原点を指定します。
* @type select
* @default 0
* @option 上
* @value 0
* @option 中央
* @value 0.5
*
* @param numberOfFrames
* @text コマ数
* @desc 矢印画像のコマ数を指定します。 
* @type number
* @default 2
* @min 1
*
* @param speed
* @text スピード
* @desc 次のコマに移行するまでのフレーム数。 
* @type number
* @default 8
* @min 1
*
*/

/*~struct~battleStatus:ja
*
* @param backOpacity
* @text 不透明度
* @desc 不透明度を指定します。
* -1で無効。
* @type number
* @default -1
* @max 255
* @min -1
*
*/

/*~struct~battleEnemy:ja
*
* @param windowVisible
* @text ウィンドウ可視化
* @desc true：MZと同じくタッチ操作に優しい。
* false：XPの仕様を再現。
* @type boolean
* @default true
*
* @param backOpacity
* @text 不透明度
* @desc 不透明度を指定します。
* -1で無効。
* @type number
* @default -1
* @max 255
* @min -1
*
*/

/*~struct~battlerHelp:ja
*
* @param windowPosition
* @text ウィンドウ位置
* @desc バトラーヘルプウィンドウの位置を指定します。
* @type select
* @default top
* @option 上
* @value top
* @option 下
* @value bottom
*
* @param actorHelp
* @text アクターヘルプ
* @desc アクターのヘルプを表示します。
* @type boolean
* @default true
*
* @param battlerFace
* @text バトラー顔画像
* @desc バトラーの顔画像を表示します。
* @type boolean
* @default false
*
* @param enemyHelp
* @text エネミーヘルプ
* @desc エネミーのヘルプを表示します。
* @type boolean
* @default true
*
* @param enemyGauges
* @text エネミーゲージ
* @desc エネミーの表示するゲージを選択します。
* @type struct<enemyGauges>
* @default 
*
* @param enemyStateIcon
* @text エネミーステートアイコン
* @desc ステートアイコンを表示します。
* @type boolean
* @default false
*
* @param outlineWidth
* @text アウトライン幅
* @desc 文字の縁取りの幅
* @type number
* @default 3
*
*/

/*~struct~enemyGauges:ja
*
* @param hp
* @desc HPを最初から開示します。
* @type boolean
* @default false
*
* @param mp
* @desc MPを最初から開示します。
* @type boolean
* @default false
*
* @param tp
* @desc TPを最初から開示します。
* @type boolean
* @default false
*
* @param time
* @desc Timeを最初から開示します。
* @type boolean
* @default false
*
*/

/*~struct~battleSkill:ja
*
* @param windowPosition
* @text ウィンドウ位置
* @desc スキル、アイテムウィンドウの位置を指定します。
* @type select
* @default center
* @option 中央
* @value center
* @option 下（MZと同じ）
* @value bottom
* @option ヘルプウィンドウも下
* @value bottom_bottom
*
* @param backOpacity
* @text 不透明度
* @desc 不透明度を指定します。
* -1で無効。
* @type number
* @default -1
* @max 255
* @min -1
*
* @param helpLines
* @text ヘルプ行数
* @desc ヘルプウィンドウの行数を指定します。
* @type number
* @default 2
*
*/

'use strict';

{

	const useMZ = Utils.RPGMAKER_NAME === "MZ";
	const pluginName = document.currentScript.src.match(/^.*\/(.*).js$/)[1];
	const parameter = PluginManager.parameters(pluginName);

	
	const partyCommand = JSON.parse(parameter["partyCommand"]);
	const actorCommand = JSON.parse(parameter["actorCommand"]);
	const battleStatus = JSON.parse(parameter["battleStatus"]);
	const battleEnemy = JSON.parse(parameter["battleEnemy"]);
	const battlerHelp = JSON.parse(parameter["battlerHelp"]);
	const battleSkill = JSON.parse(parameter["battleSkill"]);
	const enemyGauges = JSON.parse(battlerHelp["enemyGauges"]);
	
	const cursor = JSON.parse(parameter["cursor"]);
	//partyCommand
	const partyCommandPosition = partyCommand["windowPosition"];
	const partyCommandMinWidth = Number(partyCommand["partyCommandMinWidth"]);
	const partyCommandMaxWidth = Number(partyCommand["partyCommandMaxWidth"]);
	const partyCommandAlign = partyCommand["itemTextAlign"];
	//actorCommand
	const actorCommandPosition = actorCommand["windowPosition"];
	const actorCommandMinWidth = Number(actorCommand["actorCommandMinWidth"]);
	const actorCommandMaxWidth = Number(actorCommand["actorCommandMaxWidth"]);
	const actorCommandAlign = actorCommand["itemTextAlign"];
	//battleStatus
	const battleStatusBackOpacity = Number(battleStatus["backOpacity"]);
	//battleEnemy
	const battleEnemyBackOpacity = Number(battleEnemy["backOpacity"]);
	const enemyWindowVisible = battleEnemy["windowVisible"] === "true";
	//enemyGauges
	const visibleEnemyGauges = [];
	Object.keys(enemyGauges).forEach(index => {
		if (enemyGauges[index] === "true") {
			visibleEnemyGauges.push(index);
		}
	});
	//battlerHelp
	const battlerHelpPosition = battlerHelp["windowPosition"];
	const actorHelp = battlerHelp["actorHelp"] === "true";
	const battlerFace = battlerHelp["battlerFace"] === "true";
	const enemyHelp = battlerHelp["enemyHelp"] === "true";
	const enemyStateIcon = battlerHelp["enemyStateIcon"] === "true";
	const battlerNameOutlineWidth = Number(battlerHelp["outlineWidth"]);
	//battleSkill
	const battleSkillPosition = battleSkill["windowPosition"];
	const battleSkillBackOpacity = Number(battleSkill["backOpacity"]);
	const helpLines = Number(battleSkill["helpLines"]);
	//cursor
	const arrowImage = cursor["arrowImage"];
	const originY = Number(cursor["originY"]);
	const numberOfFrames = Number(cursor["numberOfFrames"]);
	const speed = Number(cursor["speed"]);
	const maxIndex = numberOfFrames - 1;

	//-----------------------------------------------------------------------------
	// Sprite_Battler
	
	if (arrowImage) Sprite_Battler.prototype.updateSelectionEffect = function() {};

	//-----------------------------------------------------------------------------
	// Window_PartyCommand

	Window_PartyCommand.prototype.itemTextAlign = function() {
		return partyCommandAlign;
	};

	const _Window_PartyCommand_maxCols = Window_PartyCommand.prototype.maxCols;
	Window_PartyCommand.prototype.maxCols = function() {
		return this._list && ["top", "bottom"].includes(partyCommandPosition) ? this._list.length : 1;
	};

	//-----------------------------------------------------------------------------
	// Window_ActorCommand

	Window_ActorCommand.prototype.itemTextAlign = function() {
		return actorCommandAlign;
	};

	const _Window_ActorCommand_setup = Window_ActorCommand.prototype.setup;
	Window_ActorCommand.prototype.setup = function(actor) {
		_Window_ActorCommand_setup.call(this, actor);
		if (actorCommandPosition === "movable") {
			this.moveToActor(actor);
		};
	};

	Window_ActorCommand.prototype.moveToActor = function(actor) {
		const memberPos = $gameParty.battleMembers().indexOf(actor);
		if (memberPos === -1) return;
		const statusWindow = SceneManager._scene._statusWindow;
		const itemRect = statusWindow.itemRect(memberPos);
		this.x = statusWindow._clientArea.x + itemRect.x + (itemRect.width/2) - (this.width/2);
		const maxX = Graphics.boxWidth - this.width;
		if (this.x > maxX) {
			this.x = maxX;
		} else if (this.x < 0) {
			this.x = 0;
		}
	};

	const _Window_ActorCommand_maxCols = Window_ActorCommand.prototype.maxCols;
	Window_ActorCommand.prototype.maxCols = function() {
		return ["top", "bottom"].includes(actorCommandPosition) ? (this._list ? this._list.length : 0) : _Window_ActorCommand_maxCols.call(this);
	};

	//-----------------------------------------------------------------------------
	// Window_BattleSkill

	if (battleSkillBackOpacity > -1) {
		Window_BattleSkill.prototype.updateBackOpacity = function() {
			this.backOpacity = battleSkillBackOpacity;
		};
	}

	//-----------------------------------------------------------------------------
	// Window_BattleItem

	if (battleSkillBackOpacity > -1) {
		Window_BattleItem.prototype.updateBackOpacity = function() {
			this.backOpacity = battleSkillBackOpacity;
		};
	}

	//-----------------------------------------------------------------------------
	// Window_BattleEnemy

	if (battleEnemyBackOpacity > -1) {
		Window_BattleEnemy.prototype.updateBackOpacity = function() {
			this.backOpacity = battleEnemyBackOpacity;
		};
	}

	const _Window_BattleEnemy_show = Window_BattleEnemy.prototype.show;
	Window_BattleEnemy.prototype.show = function() {
		this.showHelpWindow();
		_Window_BattleEnemy_show.call(this);
	};

	const _Window_BattleEnemy_hide = Window_BattleEnemy.prototype.hide;
	Window_BattleEnemy.prototype.hide = function() {
		this.hideHelpWindow();
		_Window_BattleEnemy_hide.call(this);
	};

	const _Window_BattleEnemy_select = Window_BattleEnemy.prototype.select;
	Window_BattleEnemy.prototype.select = function(index) {
		_Window_BattleEnemy_select.call(this, index);
		this.ensureCursorVisible(false);
		const enemy = this.enemy();
		if (arrowImage && enemy) {
			const sprite = SceneManager._scene._spriteset.findTargetSprite(enemy);
			const pos = sprite.getGlobalPosition();
			SceneManager._scene._arrowXp.move(pos.x, pos.y - sprite.height/2);
		}
	};

	Window_BattleEnemy.prototype.updateHelp = function() {
		this.setHelpWindowBattlerStatus(this.enemy(this._index));
	};
		
	Window_BattleEnemy.prototype.setHelpWindowBattlerStatus = function(enemy) {
		if (this._helpWindow) {
			this._helpWindow.drawItem(enemy);
		}
	};

	const _Window_BattleEnemy_isHorizontal = Window_BattleEnemy.prototype.isHorizontal;
	Window_BattleEnemy.prototype.isHorizontal = function() {
		return enemyWindowVisible ? _Window_BattleEnemy_isHorizontal.call(this) : true;
	};

	const _Window_BattleEnemy_isHoverEnabled = Window_BattleEnemy.prototype.isHoverEnabled;
	Window_BattleEnemy.prototype.isHoverEnabled = function() {
		return enemyWindowVisible ? _Window_BattleEnemy_isHoverEnabled.call(this) : false;
	};

	const _Window_BattleEnemy_cursorDown = Window_BattleEnemy.prototype.cursorDown;
	Window_BattleEnemy.prototype.cursorDown = function(wrap) {
		if (enemyWindowVisible) {
			_Window_BattleEnemy_cursorDown.call(this, wrap);
		}
	};

	const _Window_BattleEnemy_cursorUp = Window_BattleEnemy.prototype.cursorUp
	Window_BattleEnemy.prototype.cursorUp = function(wrap) {
		if (enemyWindowVisible) {
			_Window_BattleEnemy_cursorUp.call(this, wrap);
		}
	};

	const _Window_BattleEnemy_onTouchSelect = Window_BattleEnemy.prototype.onTouchSelect;
	Window_BattleEnemy.prototype.onTouchSelect = function(trigger) {
		if (enemyWindowVisible) {
			_Window_BattleEnemy_onTouchSelect.call(this, trigger);
		}
	};

	const _Window_BattleEnemy_onTouchOk = Window_BattleEnemy.prototype.onTouchOk;
	Window_BattleEnemy.prototype.onTouchOk = function() {
		if (enemyWindowVisible) {
			_Window_BattleEnemy_onTouchOk.call(this);
		}
	};

	//-----------------------------------------------------------------------------
	// Window_BattleStatus

	if (battleStatusBackOpacity > -1) {
		Window_BattleStatus.prototype.updateBackOpacity = function() {
			this.backOpacity = battleStatusBackOpacity;
		};
	}

	//-----------------------------------------------------------------------------
	// Window_BattleActor

	const _Window_BattleActor_createBattleField = Window_BattleActor.prototype.createBattleField;
	Window_BattleActor.prototype.createBattleField = function() {
		if (arrowImage) return;
		_Window_BattleActor_createBattleField.call(this);
	};

	const _Window_BattleActor_drawItem = Window_BattleActor.prototype.drawItem;
	Window_BattleActor.prototype.drawItem = function(index) {
		if (!$gameSystem.isSideView() && arrowImage) return;
		_Window_BattleActor_drawItem.call(this, index);
	};

	const _Window_BattleActor_show = Window_BattleActor.prototype.show;
	Window_BattleActor.prototype.show = function() {
		this.showHelpWindow();
		_Window_BattleActor_show.call(this);
	};

	const _Window_BattleActor_hide = Window_BattleActor.prototype.hide;
	Window_BattleActor.prototype.hide = function() {
		this.hideHelpWindow();
		_Window_BattleActor_hide.call(this);
	};

	const _Window_BattleActor_select = Window_BattleActor.prototype.select;
	Window_BattleActor.prototype.select = function(index) {
		_Window_BattleActor_select.call(this, index);
		const actor = this.actor(index);
			if (arrowImage && actor) {
				let x = 0;
				let y = 0;
				
				if ($gameSystem.isSideView()) {
					const sprite = SceneManager._scene._spriteset.findTargetSprite(actor);
					const pos = sprite.getGlobalPosition();
					x = pos.x;
					y = pos.y - sprite.height/2;
				} else {
					const faceRect = this.faceRect(index);
					x = ((Graphics.width - Graphics.boxWidth) / 2) + this._clientArea.x + faceRect.x + (faceRect.width/2);
					y = this.y + this.height/2;
				}
				SceneManager._scene._arrowXp.move(x, y);
			}
	};

	Window_BattleActor.prototype.updateHelp = function() {
		this.setHelpWindowBattlerStatus(this.actor(this._index));
	};
		
	Window_BattleActor.prototype.setHelpWindowBattlerStatus = function(actor) {
		if (this._helpWindow) {
			this._helpWindow.drawItem(actor);
		}
	};

	//-----------------------------------------------------------------------------
	// Window_BattlerHelp

	function Window_BattlerHelp() {
		this.initialize(...arguments);
	}

	Window_BattlerHelp.prototype = Object.create(Window_Help.prototype);
	Window_BattlerHelp.prototype.constructor = Window_BattlerHelp;

	Window_BattlerHelp.prototype.initialize = function(rect) {
		this._additionalSprites = {};
		this._battler = null;
		Window_Help.prototype.initialize.call(this, rect);
	};

	Window_BattlerHelp.prototype.drawActorFace = function(actor, x, y, width, height) {
		Window_StatusBase.prototype.drawActorFace.apply(this, arguments);
	};

	Window_BattlerHelp.prototype.hideAdditionalSprites = function() {
		Window_StatusBase.prototype.hideAdditionalSprites.call(this);
	};

	Window_BattlerHelp.prototype.drawItem = function(battler) {
		this._battler = battler
		this.clear();
		this.refresh();
		this.drawItemImage(battler);
		this.drawItemStatus(battler);
	};

	Window_BattlerHelp.prototype.faceRect = function() {
		const width = ImageManager.faceWidth;
		const height = this.innerHeight;
		const x = 0;
		return new Rectangle(x, 0, width, height);
	};

	Window_BattlerHelp.prototype.itemRectWithPadding = function() {
		const rect = this.innerRect;
		const x = 0;
		const y = 0;
		const width = rect.width;
		const height = rect.height;
		return new Rectangle(x, y, width, height);
	};

	Window_BattlerHelp.prototype.setFaceImage = function(battler) {
		const bitmap = ImageManager.loadFace(battler.faceName());
		bitmap.addLoadListener(this.onFaceLoad.bind(this, battler));
	};

	Window_BattlerHelp.prototype.onFaceLoad = function(battler) {
		if (this._battler === battler) {
			const rect = this.faceRect();
			this.drawActorFace(battler, rect.x, rect.y, rect.width, rect.height);
		}
	};

	Window_BattlerHelp.prototype.drawItemImage = function(battler) {
		if (!battlerFace) return;
		this.setFaceImage(battler);
	};

	let battlerHelpWidth = 0;
	Window_BattlerHelp.prototype.drawItemStatus = function(battler) {
		this.hideAdditionalSprites();
		const rect = this.itemRectWithPadding();
		rect.y = 0;
		const nameX = this.nameX(rect);
		const nameY = this.nameY(rect);
		const stateIconX = this.stateIconX(rect);
		const stateIconY = this.stateIconY(rect);
		const basicGaugesX = this.basicGaugesX(rect);
		const basicGaugesY = this.basicGaugesY(rect);
		battlerHelpWidth = 150;
		this.placeTimeGauge(battler, nameX, basicGaugesY);
		this.placeActorName(battler, nameX, nameY);
		if (battler.isActor() || enemyStateIcon) {
			this.placeStateIcon(battler, stateIconX, stateIconY);
		}
		this.placeBasicGauges(battler, basicGaugesX, basicGaugesY);
	};

	Window_BattlerHelp.prototype.nameX = function(rect) {
		const faceExists = this._battler.faceName() && battlerFace;
		const faceRect = this.faceRect();
		const spacing = 16
		const margin = faceExists ? rect.x + faceRect.x + faceRect.width + spacing : 0;
		return margin || this.padding;
	};

	Window_BattlerHelp.prototype.nameY = function(rect) {
		return rect.y;
	};

	Window_BattlerHelp.prototype.stateIconX = function(rect) {
		return rect.x + 156 + this.nameX(rect) + ImageManager.iconWidth/2;
	};

	Window_BattlerHelp.prototype.stateIconY = function(rect) {
		return (rect.y + rect.height)/2;
	};

	Window_BattlerHelp.prototype.basicGaugesX = function(rect) {
		return rect.width - this.gaugeAreaWidth() - this.padding;
	};

	Window_BattlerHelp.prototype.basicGaugesY = function(rect) {
		return rect.y;
	};

	Window_BattlerHelp.prototype.gaugeAreaWidth = function() {
		return 330;
	};

	Window_BattlerHelp.prototype.placeActorName = function(battler, x, y) {
		y +=  + this.innerRect.y/2;
		if (battler.isActor()) {
			Window_StatusBase.prototype.placeActorName.call(this, battler, x, y);
		} else if (battler.isEnemy()){
			this.placeEnemyName(battler, x, y);
		}
	};

	let battlerHelpAlign = "left";
	Window_BattlerHelp.prototype.placeEnemyName = function(enemy, x, y) {
		battlerHelpAlign = "left";
		if (this.isNoGauge() && !enemyStateIcon) {
			x = 0;
			battlerHelpWidth = this.innerWidth;
			battlerHelpAlign = "center";
		}
		const key = "enemy%1-name".format(enemy.index());
		const sprite = this.createInnerSprite(key, Sprite_Name);
		sprite.setup(enemy);
		sprite.move(x, y);
		sprite.show();
		
	};

	Window_BattlerHelp.prototype.isNoGauge = function() {
		let numberOfGauges = visibleEnemyGauges.length;
		if (!BattleManager.isTpb() && visibleEnemyGauges.includes("time")) {
			numberOfGauges--;
		}
		return !numberOfGauges;
	};

	Window_BattlerHelp.prototype.placeStateIcon = function(battler, x, y) {
		if (battler.isActor()) {
			Window_StatusBase.prototype.placeStateIcon.apply(this, arguments);
		} else if (battler.isEnemy()){
			this.placeEnemyStateIcon(...arguments);
		}
	};

	Window_BattlerHelp.prototype.placeGauge = function(battler, type, x, y) {
		if (battler.isActor()) {
			Window_StatusBase.prototype.placeGauge.apply(this, arguments);
		} else if (battler.isEnemy()){
			this.placeEnemyGauge(...arguments);
		}
	};

	Window_BattlerHelp.prototype.placeEnemyStateIcon = function(enemy, x, y) {
		const key = "enemy%1-stateIcon".format(enemy.index());
		const sprite = this.createInnerSprite(key, Sprite_StateIcon);
		sprite.setup(enemy);
		sprite.move(x, y);
		sprite.show();
	};

	Window_BattlerHelp.prototype.placeEnemyGauge = function(enemy, type, x, y) {
		const key = "enemy%1-gauge-%2".format(enemy.index(), type);
		const sprite = this.createInnerSprite(key, Sprite_Gauge);
		sprite.setup(enemy, type);
		sprite.move(x, y);
		sprite.show();
	};

	Window_BattlerHelp.prototype.createInnerSprite = function(key, spriteClass) {
		let newClass = null;
		if (spriteClass === Sprite_Name) {
			newClass = Sprite_BattlerHelpName;
		} else if (spriteClass === Sprite_Gauge) {
			newClass = Sprite_BattlerHelpGauge;
		}
		return Window_StatusBase.prototype.createInnerSprite.call(this, key, newClass || spriteClass);
	};

	Window_BattlerHelp.prototype.placeTimeGauge = function(battler, x, y) {
		if (battler.isActor() || visibleEnemyGauges.includes("time")) {
			Window_StatusBase.prototype.placeTimeGauge.apply(this, arguments);
		}
	};

	Window_BattlerHelp.prototype.placeBasicGauges = function(battler, x, y) {
		if (battler.isEnemy()) {
			this.placeBasicEnemyGauges(...arguments);
		} else {
			this.placeBasicActorGauges(...arguments);
		}
			
	};

	Window_BattlerHelp.prototype.placeBasicActorGauges = function(actor, x, y) {
		if ($dataSystem.optDisplayTp) {
			battlerHelpWidth = 108;
			this.placeGauge(actor, "hp", x, y);
			battlerHelpWidth = 96;
			this.placeGauge(actor, "mp", x + 123, y);
			this.placeGauge(actor, "tp", x + 234, y);
		} else {
			battlerHelpWidth = 201;
			this.placeGauge(actor, "hp", x, y);
			battlerHelpWidth = 114;
			this.placeGauge(actor, "mp", x + 216, y);
		}
	};

	Window_BattlerHelp.prototype.placeBasicEnemyGauges = function(enemy, x, y) {
		const visibleStatus = visibleEnemyGauges;
		const hp = visibleStatus.includes("hp");
		const mp = visibleStatus.includes("mp");
		const tp = visibleStatus.includes("tp") && $dataSystem.optDisplayTp;
		const numberOfGauges = [hp, mp, tp].filter(bool => bool).length;
		if (numberOfGauges === 3) {
			battlerHelpWidth = 108;
			this.placeGauge(enemy, visibleStatus[0], x, y);
			battlerHelpWidth = 96;
			this.placeGauge(enemy, visibleStatus[1], x + 123, y);
			this.placeGauge(enemy, visibleStatus[2], x + 234, y);
		} else if (numberOfGauges === 2) {
			battlerHelpWidth = 201;
			this.placeGauge(enemy, visibleStatus[0], x, y);
			battlerHelpWidth = 114;
			this.placeGauge(enemy, visibleStatus[1], x + 216, y);
		} else if (numberOfGauges === 1){
			battlerHelpWidth = 330;
			this.placeGauge(enemy, visibleStatus[0], x, y);
		}
	};

	//-----------------------------------------------------------------------------
	// Game_Enemy

	Game_Enemy.prototype.faceName = function() {
		const meta = this.enemy().meta["battlerFace"];
		return meta ? meta.split(',')[0] : "";
	};

	Game_Enemy.prototype.faceIndex = function() {
		const meta = this.enemy().meta["battlerFace"];
		return meta ? Number(meta.split(',')[1]) : 0;
	};

	const _Game_Enemy_setup = Game_Enemy.prototype.setup;
	Game_Enemy.prototype.setup = function(enemyId, x, y) {
		_Game_Enemy_setup.apply(this, arguments);
		if (battlerFace) {
			ImageManager.loadFace(this.faceName());
		}
	};

	//-----------------------------------------------------------------------------
	// Scene_Battle

	const _Scene_Battle_startPartyCommandSelection = Scene_Battle.prototype.startPartyCommandSelection;
	Scene_Battle.prototype.startPartyCommandSelection = function() {
		_Scene_Battle_startPartyCommandSelection.call(this);
		this._actorCommandWindow.open();
		this._actorCommandWindow.hide();
		this._partyCommandWindow.show();
	};

	const _Scene_Battle_startActorCommandSelection = Scene_Battle.prototype.startActorCommandSelection;
	Scene_Battle.prototype.startActorCommandSelection = function() {
		_Scene_Battle_startActorCommandSelection.call(this);
		this._partyCommandWindow.open();
		this._partyCommandWindow.hide();
		this._actorCommandWindow.show();
	};

	const _Scene_Battle_createDisplayObjects = Scene_Battle.prototype.createDisplayObjects;
	Scene_Battle.prototype.createDisplayObjects = function() {
		_Scene_Battle_createDisplayObjects.call(this);
		this.createArrowXp();
	};

	Scene_Battle.prototype.createArrowXp = function() {
		if (!arrowImage) return;
		this._arrowXp = new Sprite_ArrowXp();
		this.addChild(this._arrowXp);
		this.hideArrowXp();
	};

	const _Scene_Battle_createAllWindows = Scene_Battle.prototype.createAllWindows;
	Scene_Battle.prototype.createAllWindows = function() {
		_Scene_Battle_createAllWindows.call(this);
		this.sortWindows();
	};

	const _Scene_Battle_createHelpWindow  = Scene_Battle.prototype.createHelpWindow;
	Scene_Battle.prototype.createHelpWindow = function() {
		_Scene_Battle_createHelpWindow.call(this);
		this.createBattlerHelpWindow(this);
		if (battleSkillPosition === "bottom_bottom") {
			this._helpWindow.y = this._statusWindow.y - this._helpWindow.height;
		}
	};

	Scene_Battle.prototype.helpAreaHeight = function() {
		return this.calcWindowHeight(helpLines, false);
	};

	Scene_Battle.prototype.createBattlerHelpWindow = function() {
		const rect = this.battlerHelpWindowRect();
		this._battlerHelpWindow = new Window_BattlerHelp(rect);
		this._battlerHelpWindow.hide();
		this.addWindow(this._battlerHelpWindow);
	};

	Scene_Battle.prototype.battlerHelpWindowRect = function() {
		const wx = 0;
		const ww = Graphics.boxWidth;
		const wh = this.battlerHelpAreaHeight();
		const wy = battlerHelpPosition === "top" ? this.helpAreaTop() : this._statusWindow.y - wh;
		return new Rectangle(wx, wy, ww, wh);
	};

	Scene_Battle.prototype.battlerHelpAreaHeight = function() {
		return this.calcWindowHeight(1, false);
	};

	Scene_Battle.prototype.statusWindowRect = function() {
		const ww = Graphics.boxWidth;
		const wh = this.windowAreaHeight();
		const wx = 0;
		const wy = Graphics.boxHeight - wh;
		return new Rectangle(wx, wy, ww, wh);
	};

	Scene_Battle.prototype.partyCommandWindowRect = function() {
		let ww = Graphics.boxWidth;
		let wx = 0;
		const pos = partyCommandPosition;
		const LorR = pos.includes("left") || pos.includes("right");
		const center = pos === "center";
		const upper = pos === "top" || pos.includes("upper");
		if (LorR || center) {
			const statusWindow = this._statusWindow;
			ww = 192;
			if (partyCommandMaxWidth && ww > partyCommandMaxWidth) {
				ww = partyCommandMaxWidth;
			} else if (partyCommandMinWidth && ww < partyCommandMinWidth) {
				ww = partyCommandMinWidth;
			}
			if (pos.includes("right")) {
				wx = Graphics.boxWidth - ww;
			} else if (center) {
				wx = Math.floor((Graphics.boxWidth - ww)/2);
			}
		}
		const lines = LorR ? 4 : (center ? 2 : 1);
		const wh = this.calcWindowHeight(lines, true);
		const wy = upper ? 0 : center ? Math.floor((Graphics.boxHeight - wh)/2) : this._statusWindow.y - wh;
		return new Rectangle(wx, wy, ww, wh);
	};

	const _Scene_Battle_createActorCommandWindow = Scene_Battle.prototype.createActorCommandWindow;
	Scene_Battle.prototype.createActorCommandWindow = function() {
		_Scene_Battle_createActorCommandWindow.call(this);
		const commandWindow = this._actorCommandWindow;
		const pos = actorCommandPosition;
		if (pos.includes("right")) {
			commandWindow.x = Graphics.boxWidth - commandWindow.width;
		} else {
			commandWindow.x = 0;
		}
		if (["top", "upper-right", "upper-left"].includes(pos)) {
			commandWindow.y = 0;
		} else if (pos === "bottom") {
			commandWindow.y = this._statusWindow.y - commandWindow.height;
		} else {
			commandWindow.y = Graphics.boxHeight - commandWindow.height - this.statusWindowRect().height;
		}
	};

	Scene_Battle.prototype.actorCommandWindowRect = function() {
		const statusWindow = this._statusWindow;
		let ww = actorCommandPosition === "movable" ? statusWindow.width/4 + statusWindow.colSpacing() / 2 + statusWindow.itemPadding() : 192;
		const wh = ["top", "bottom"].includes(actorCommandPosition) ? this.calcWindowHeight(1, true) : this.windowAreaHeight();
		const wx = 0;
		const wy = 0;
		if (actorCommandMaxWidth && ww > actorCommandMaxWidth) {
			ww = actorCommandMaxWidth;
		} else if (actorCommandMinWidth && ww < actorCommandMinWidth) {
			ww = actorCommandMinWidth;
		}
		if (["top", "bottom"].includes(actorCommandPosition)) {
			ww = Graphics.boxWidth;
		}
		return new Rectangle(wx, wy, ww, wh);
	};

	Scene_Battle.prototype.updateStatusWindowPosition = function() {};

	const _Scene_Battle_skillWindowRect = Scene_Battle.prototype.skillWindowRect;
	Scene_Battle.prototype.skillWindowRect = function() {
		if (battleSkillPosition !== "center") {
			return _Scene_Battle_skillWindowRect.call(this);
		}
		const wx = 0;
		const wh = this.calcWindowHeight(6, true);
		const wy = this.statusWindowRect().y - wh;
		const ww = Graphics.boxWidth;
		return new Rectangle(wx, wy, ww, wh);
	};

	const _Scene_Battle_createActorWindow = Scene_Battle.prototype.createActorWindow;
	Scene_Battle.prototype.createActorWindow = function() {
		_Scene_Battle_createActorWindow.call(this);
		if (actorHelp) {
			this._actorWindow.setHelpWindow(this._battlerHelpWindow);
		}
	};

	const _Scene_Battle_createEnemyWindow = Scene_Battle.prototype.createEnemyWindow;
	Scene_Battle.prototype.createEnemyWindow = function() {
		_Scene_Battle_createEnemyWindow.call(this);
		if (enemyHelp) {
			this._enemyWindow.setHelpWindow(this._battlerHelpWindow);
		}
	};

	Scene_Battle.prototype.sortWindows = function() {
		const windows = [];
		const arrow = !$gameSystem.isSideView() && arrowImage;
		windows.push(arrow ? this._actorWindow : this._statusWindow);
		windows.push(arrow ? this._statusWindow : this._actorWindow);
		if (battleSkillPosition !== "center") {
			windows.push(this._skillWindow);
			windows.push(this._itemWindow);
		}
		if (enemyWindowVisible) {
			windows.push(this._enemyWindow);
		}
		const aboveWindow = this._messageWindow;
		windows.forEach(_window => {
			this._windowLayer.removeChild(_window);
			this._windowLayer.addChildAt(_window, this._windowLayer.children.indexOf(aboveWindow));
		});
	};

	//以下ウィンドウの表示切替
	const _Scene_Battle_commandSkill = Scene_Battle.prototype.commandSkill;
	Scene_Battle.prototype.commandSkill = function() {
		_Scene_Battle_commandSkill.call(this);
		this._statusWindow.show();
	};

	const _Scene_Battle_commandItem = Scene_Battle.prototype.commandItem;
	Scene_Battle.prototype.commandItem = function() {
		_Scene_Battle_commandItem.call(this);
		this._statusWindow.show();
	};

	const _Scene_Battle_startActorSelection = Scene_Battle.prototype.startActorSelection;
	Scene_Battle.prototype.startActorSelection = function() {
		this._actorCommandWindow.hide();
		this._itemWindow.hide();
		this._skillWindow.hide();
		_Scene_Battle_startActorSelection.call(this);
		this.showArrowXp();
	};

	const _Scene_Battle_onActorOk = Scene_Battle.prototype.onActorOk;
	Scene_Battle.prototype.onActorOk = function() {
		_Scene_Battle_onActorOk.call(this);
		this.hideArrowXp();
	};

	const _Scene_Battle_onActorCancel = Scene_Battle.prototype.onActorCancel;
	Scene_Battle.prototype.onActorCancel = function() {
		_Scene_Battle_onActorCancel.call(this);
		this.hideArrowXp();
	};

	const _Scene_Battle_startEnemySelection = Scene_Battle.prototype.startEnemySelection;
	Scene_Battle.prototype.startEnemySelection = function() {
		_Scene_Battle_startEnemySelection.call(this);
		this._statusWindow.show();
		this._itemWindow.hide();
		this._skillWindow.hide();
		this._actorCommandWindow.hide();
		this.showArrowXp();
	};

	const _Scene_Battle_onEnemyOk = Scene_Battle.prototype.onEnemyOk;
	Scene_Battle.prototype.onEnemyOk = function() {
		_Scene_Battle_onEnemyOk.call(this);
		this.hideArrowXp();
	};

	const _Scene_Battle_onEnemyCancel = Scene_Battle.prototype.onEnemyCancel;
	Scene_Battle.prototype.onEnemyCancel = function() {
		_Scene_Battle_onEnemyCancel.call(this);
		switch (this._actorCommandWindow.currentSymbol()) {
		case "attack":
			this._actorCommandWindow.show();
			break;
		case "skill":
			this._actorCommandWindow.show()
			break;
		case "item":
			this._actorCommandWindow.show()
			break;
		}
		this.hideArrowXp();
	};

	Scene_Battle.prototype.showArrowXp = function() {
		if (arrowImage) this._arrowXp.show();
	};

	Scene_Battle.prototype.hideArrowXp = function() {
		if (arrowImage) this._arrowXp.hide();
	};

	//-----------------------------------------------------------------------------
	// Sprite_BattlerHelpGauge

	function Sprite_BattlerHelpGauge() {
		this.initialize(...arguments);
	}

	Sprite_BattlerHelpGauge.prototype = Object.create(Sprite_Gauge.prototype);
	Sprite_BattlerHelpGauge.prototype.constructor = Sprite_BattlerHelpGauge;

	Sprite_BattlerHelpGauge.prototype.initialize = function(width) {
		Sprite.prototype.initialize.call(this);
		this.initMembers();
		this.createBitmap();
	};

	Sprite_BattlerHelpGauge.prototype.initMembers = function() {
		Sprite_Gauge.prototype.initMembers.call(this);
		this._gaugeWidth = battlerHelpWidth;
	};

	Sprite_BattlerHelpGauge.prototype.bitmapWidth = function() {
		return this._gaugeWidth;
	};

	//-----------------------------------------------------------------------------
	// Sprite_BattlerHelpName

	function Sprite_BattlerHelpName() {
		this.initialize(...arguments);
	}

	Sprite_BattlerHelpName.prototype = Object.create(Sprite_Name.prototype);
	Sprite_BattlerHelpName.prototype.constructor = Sprite_BattlerHelpName;

	Sprite_BattlerHelpName.prototype.initMembers = function() {
		Sprite_Name.prototype.initMembers.call(this);
		this._gaugeWidth = battlerHelpWidth;
	};

	Sprite_BattlerHelpName.prototype.bitmapWidth = function() {
		return this._gaugeWidth;
	};

	Sprite_BattlerHelpName.prototype.outlineWidth = function() {
		return battlerNameOutlineWidth;
	};

	Sprite_BattlerHelpName.prototype.textColor = function() {
		return battlerHelpAlign === "center" ? ColorManager.normalColor() : ColorManager.hpColor(this._battler);
	};

	Sprite_BattlerHelpName.prototype.redraw = function() {
		const name = this.name();
		const width = this.bitmapWidth();
		const height = this.bitmapHeight();
		this.setupFont();
		this.bitmap.clear();
		this.bitmap.drawText(name, 0, 0, width, height, battlerHelpAlign);
	};

	//-----------------------------------------------------------------------------
	// Sprite_ArrowXp

	function Sprite_ArrowXp() {
		this.initialize(...arguments);
	}

	Sprite_ArrowXp.prototype = Object.create(Sprite.prototype);
	Sprite_ArrowXp.prototype.constructor = Sprite_ArrowXp;

	Sprite_ArrowXp.prototype.initialize = function() {
		Sprite.prototype.initialize.call(this);
		this.initMembers();
		this.loadBitmap();
		this.setup();
	};

	Sprite_ArrowXp.prototype.initMembers = function() {
		this._duration = 0;
		this._frameIndex = 0;
		this.anchor.x = 0.5;
		this.anchor.y = originY;
	};

	Sprite_ArrowXp.prototype.loadBitmap = function() {
		this.bitmap = ImageManager.loadSystem(arrowImage);
	};

	Sprite_ArrowXp.prototype.setup = function() {
		this._duration = this.speed();
		this._frameIndex = 0;
	};

	Sprite_ArrowXp.prototype.update = function() {
		Sprite.prototype.update.call(this);
		this._duration--;
		if (this._duration <= 0) {
			this._duration = this.speed();
			this._frameIndex++;
			if (this.frameIndex() > maxIndex) {
				this._frameIndex = 0;
			}
		}
		this.updatePosition();
		this.updateFrame();
	};

	Sprite_ArrowXp.prototype.updatePosition = function() {};

	Sprite_ArrowXp.prototype.updateFrame = function() {
		const w = this.bitmap.width / numberOfFrames;
		const h = this.bitmap.height;
		const sx = this._frameIndex * w;
		const sy = 0;
		this.setFrame(sx, sy, w, h);
	};

	Sprite_ArrowXp.prototype.speed = function() {
		return speed;
	};

	Sprite_ArrowXp.prototype.frameIndex = function() {
		return this._frameIndex;
	};

	Sprite_ArrowXp.prototype.show = function() {
		Sprite.prototype.show.call(this);
		this.setup();
	};

}